Package: ggpower
Title: Publication-Ready Power Analysis and Visualization
Version: 0.1.2
Authors@R: 
    person("Yaoxiang", "Li", , "liyaoxiang@outlook.com", role = c("aut", "cre"))
Author: Yaoxiang Li [aut, cre]
Maintainer: Yaoxiang Li <liyaoxiang@outlook.com>
Description: Provides statistical power analysis and sample size calculations
    for t-tests, ANOVA, regression, chi-square, proportion, correlation,
    nonparametric, biomarker, and clinical trial designs. Includes a
    scriptable API via power_compute(), publication-ready 'ggplot2'
    visualizations, and an optional 'Shiny' application.
URL: https://github.com/YaoxiangLi/ggpower,
        https://yaoxiangli.github.io/ggpower/
BugReports: https://github.com/YaoxiangLi/ggpower/issues
License: GPL (>= 3)
Imports: bs4Dash, config, ggplot2, golem, shiny
Encoding: UTF-8
RoxygenNote: 7.3.2
Suggests: knitr, pkgdown, rmarkdown, spelling, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Language: en-US
NeedsCompilation: no
Packaged: 2026-07-03 19:58:15 UTC; Li
Repository: CRAN
Date/Publication: 2026-07-10 20:50:17 UTC

# Generated by roxygen2: do not edit by hand

export(format_result_html)
export(ggpower_t_one_sample)
export(ggpower_ttest)
export(ggpower_result)
export(ggpower_calculator)
export(ggpower_tests)
export(plot_distribution)
export(plot_power_curve)
export(effect_size_d)
export(effect_size_f)
export(effect_size_f2)
export(effect_size_f2_increase)
export(effect_size_h)
export(effect_size_q)
export(effect_size_w)
export(eta2_from_f)
export(odds_ratio_from_probs)
export(power_t_one_sample)
export(power_compute)
export(power_t_paired)
export(power_t_two_sample)
export(r2_from_f2)
export(run_app)
export(save_distribution_plot)
export(save_power_plot)
export(theme_ggpower)
S3method(print,ggpower_result)
import(shiny)
importFrom(bs4Dash,bs4DashBody)
importFrom(bs4Dash,bs4DashControlbar)
importFrom(bs4Dash,bs4DashFooter)
importFrom(bs4Dash,bs4DashNavbar)
importFrom(bs4Dash,bs4DashPage)
importFrom(bs4Dash,bs4DashSidebar)
importFrom(bs4Dash,bs4TabItem)
importFrom(bs4Dash,bs4TabItems)
importFrom(bs4Dash,menuItem)
importFrom(bs4Dash,sidebarMenu)
importFrom(golem,activate_js)
importFrom(golem,add_resource_path)
importFrom(golem,bundle_resources)
importFrom(golem,favicon)
importFrom(golem,with_golem_options)
importFrom(shiny,NS)
importFrom(shiny,div)
importFrom(shiny,icon)
importFrom(shiny,shinyApp)
importFrom(shiny,tagList)
importFrom(shiny,tags)
importFrom(stats,pt)
importFrom(stats,qt)
importFrom(utils,capture.output)

# ggpower

[![R-CMD-check](https://github.com/YaoxiangLi/ggpower/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/YaoxiangLi/ggpower/actions/workflows/R-CMD-check.yaml)

Documentation: <https://yaoxiangli.github.io/ggpower/>

`ggpower` is an R package and Shiny application for statistical power analysis.
It provides a scriptable API through `power_compute()` and a wide-screen GUI with
six modules:

1. Choose a test family and statistical test.
2. Choose a power-analysis type.
3. Enter input parameters.
4. Calculate output parameters.
5. Inspect distribution plots, X-Y power plots, and protocol history.

## Supported Analysis Modes

- A priori: compute required sample size.
- Compromise: compute alpha and beta from beta / alpha.
- Criterion: compute alpha for a target power.
- Post hoc: compute achieved power.
- Sensitivity: compute the effect size required for target power.

## Supported Test Families

- t tests: one-sample, paired, two independent means, point-biserial, generic t.
- F tests: one-way ANOVA, special ANOVA effects, multiple-regression omnibus,
  multiple-regression R2 increase, two variances.
- Chi-square tests: variance, goodness-of-fit, contingency tables.
- Exact/proportion tests: binomial, one proportion, sign test, McNemar-style
  discordant-pair binomial planning, Fisher-style two-proportion planning.
- z tests: independent correlations, logistic regression, Poisson regression.
- Nonparametric tests: Wilcoxon signed-rank and Wilcoxon-Mann-Whitney using
  asymptotic relative efficiency planning.
- Biomarker: ROC/AUC, diagnostic accuracy, survival, Cox, FDR screening.
- Clinical: superiority, non-inferiority, equivalence, Simon two-stage, cluster RCT.

Some advanced procedures use documented approximations. The result object includes
method notes so these cases are visible in the GUI and protocol. See the
[approximation catalog](https://yaoxiangli.github.io/ggpower/articles/approximation-catalog.html).

## Examples

```r
power_compute(
  "t_one_sample",
  "a_priori",
  d = 0.625,
  alpha = 0.05,
  power = 0.95,
  tails = "one"
)

power_compute(
  "f_mreg_omnibus",
  "post_hoc",
  f2 = 0.1111111,
  alpha = 0.05,
  total_n = 95,
  predictors = 5
)
```

Run the app with:

```r
ggpower::run_app()
```

## Validation

The test suite includes golden reference examples, solver tests for the five
analysis modes, registry checks, and Shiny module smoke tests.

#' Plot Power Curve for a One-Sample t-Test
#'
#' This function creates a ggplot2 power curve for a one-sample t test.
#'
#' @param d Numeric. The effect size (d).
#' @param alpha Numeric. The significance level (default 0.05).
#' @param n_range Numeric vector. A vector of total sample sizes (default is seq(20, 100, by = 5)).
#' @param tails Character. `"two"` or `"one"`.
#'
#' @return A ggplot object showing the power curve.
#' @export
#'
#' @examples
#' # Plot power curve for d = 0.5 over sample sizes from 20 to 100
#' ggpower_t_one_sample(d = 0.5, alpha = 0.05, n_range = seq(20, 100, by = 5))
ggpower_t_one_sample <- function(d, alpha = 0.05, n_range = seq(20, 100, by = 5), tails = "two") {
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' must be installed for plotting.")
  }

  power_vals <- sapply(n_range, function(n) {
    power_t_one_sample(d = d, n = n, alpha = alpha, tails = tails)
  })

  df_plot <- data.frame(
    n = n_range,
    power = power_vals
  )

  p <- ggplot2::ggplot(df_plot, ggplot2::aes(x = n, y = power)) +
    ggplot2::geom_line(color = "steelblue", linewidth = 1.2) +
    ggplot2::geom_point(color = "darkred", size = 2) +
    ggplot2::labs(
      title = paste("Power Curve for One-Sample t-Test (d =", d, ")"),
      x = "Total Sample Size",
      y = "Power (1 - Beta)"
    ) +
    ggplot2::theme_minimal()

  return(p)
}

library(testthat)
library(ggpower)

test_that("power_t_one_sample returns a number between 0 and 1", {
  pwr <- power_t_one_sample(d = 0.5, n = 40)
  expect_true(is.numeric(pwr))
  expect_true(pwr > 0 && pwr < 1)
})

test_that("ggpower_t_one_sample returns a ggplot object", {
  p <- ggpower_t_one_sample(d = 0.5, alpha = 0.05, n_range = seq(20, 100, by = 10))
  expect_s3_class(p, "ggplot")
})

test_that("Increasing sample size increases power", {
  power_small <- power_t_one_sample(d = 0.5, n = 30)
  power_large <- power_t_one_sample(d = 0.5, n = 60)
  expect_true(power_large > power_small)
})

.gp_roc_auc_one <- function(params, analysis) {
  auc <- .gp_numeric(params, "auc", 0.75, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  auc0 <- .gp_numeric(params, "auc0", 0.5, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  n_pos <- .gp_count(params, "n_pos", .gp_arg(params, "n1", 50), min = 2)
  n_neg <- .gp_count(params, "n_neg", .gp_arg(params, "n2", 50), min = 2)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)

  q1 <- auc / (2 - auc)
  q2 <- (2 * auc^2) / (1 + auc)
  var_auc <- (auc * (1 - auc) +
    (n_pos - 1) * (q1 - auc^2) +
    (n_neg - 1) * (q2 - auc^2)) / (n_pos * n_neg)
  se <- sqrt(max(var_auc, 1e-12))
  z_ncp <- (auc - auc0) / se
  pwr <- .gp_power_z(z_ncp, alpha, tails)

  .gp_result(
    "Biomarker: One-sample ROC AUC vs null", analysis,
    list(
      tails = pwr$direction, auc_h1 = auc, auc_h0 = auc0,
      n_positive = n_pos, n_negative = n_neg, alpha = alpha
    ),
    list(
      z_statistic = z_ncp, se_auc = se, total_sample_size = n_pos + n_neg,
      power = pwr$power
    ),
    notes = "Hanley-McNeil normal approximation for AUC variance.",
    distribution = list(type = "z", mean = z_ncp)
  )
}

.gp_roc_auc_two <- function(params, analysis) {
  auc1 <- .gp_numeric(params, "auc1", 0.75, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  auc2 <- .gp_numeric(params, "auc2", 0.65, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  n1 <- .gp_count(params, "n1", 50, min = 2)
  n2 <- .gp_count(params, "n2", 50, min = 2)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)

  q11 <- auc1 / (2 - auc1)
  q12 <- (2 * auc1^2) / (1 + auc1)
  q21 <- auc2 / (2 - auc2)
  q22 <- (2 * auc2^2) / (1 + auc2)
  var1 <- (auc1 * (1 - auc1) + (n1 - 1) * (q11 - auc1^2) + (n1 - 1) * (q12 - auc1^2)) / (n1^2)
  var2 <- (auc2 * (1 - auc2) + (n2 - 1) * (q21 - auc2^2) + (n2 - 1) * (q22 - auc2^2)) / (n2^2)
  se_diff <- sqrt(max(var1 + var2, 1e-12))
  z_ncp <- (auc1 - auc2) / se_diff
  pwr <- .gp_power_z(z_ncp, alpha, tails)

  .gp_result(
    "Biomarker: Two-sample ROC AUC comparison", analysis,
    list(
      tails = pwr$direction, auc_group_1 = auc1, auc_group_2 = auc2,
      sample_size_group_1 = n1, sample_size_group_2 = n2, alpha = alpha
    ),
    list(
      z_statistic = z_ncp, se_difference = se_diff, total_sample_size = n1 + n2,
      power = pwr$power
    ),
    notes = "DeLong-style normal approximation for AUC difference.",
    distribution = list(type = "z", mean = z_ncp)
  )
}

.gp_diagnostic_acc <- function(params, analysis) {
  sens <- .gp_numeric(params, "sensitivity", 0.85, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  spec <- .gp_numeric(params, "specificity", 0.85, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  n_pos <- .gp_count(params, "n_pos", 50, min = 2)
  n_neg <- .gp_count(params, "n_neg", 50, min = 2)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)

  se_sens <- sqrt(sens * (1 - sens) / n_pos)
  se_spec <- sqrt(spec * (1 - spec) / n_neg)
  z_sens <- sens / se_sens
  z_spec <- spec / se_spec
  pwr_sens <- .gp_power_z(z_sens, alpha, tails)
  pwr_spec <- .gp_power_z(z_spec, alpha, tails)
  power <- min(pwr_sens$power, pwr_spec$power)

  .gp_result(
    "Biomarker: Diagnostic accuracy (sensitivity and specificity)", analysis,
    list(
      tails = tails, sensitivity_h1 = sens, specificity_h1 = spec,
      n_positive = n_pos, n_negative = n_neg, alpha = alpha
    ),
    list(
      z_sensitivity = z_sens, z_specificity = z_spec,
      power_sensitivity = pwr_sens$power, power_specificity = pwr_spec$power,
      power = power, total_sample_size = n_pos + n_neg
    ),
    notes = "Joint power uses the minimum of sensitivity and specificity power (Bonferroni-style).",
    distribution = list(type = "z", mean = mean(c(z_sens, z_spec)))
  )
}

.gp_survival_logrank <- function(params, analysis) {
  hr <- .gp_numeric(params, "hazard_ratio", 0.65, min = 0, strict_min = TRUE)
  p_event <- .gp_numeric(params, "event_rate", 0.5, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  allocation <- .gp_numeric(params, "allocation_ratio", 1, min = 0, strict_min = TRUE)
  total_n <- .gp_count(params, "total_n", .gp_arg(params, "n", 200), min = 4)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)

  p <- allocation / (1 + allocation)
  events <- total_n * p_event
  log_hr <- log(hr)
  z_ncp <- abs(log_hr) * sqrt(events * p * (1 - p))
  pwr <- .gp_power_z(z_ncp, alpha, tails)

  .gp_result(
    "Biomarker: Survival log-rank test", analysis,
    list(
      tails = pwr$direction, hazard_ratio = hr, event_rate = p_event,
      allocation_ratio = allocation, total_sample_size = total_n, alpha = alpha
    ),
    list(
      expected_events = events, z_statistic = z_ncp, power = pwr$power
    ),
    notes = "Schoenfeld/Freedman log-rank approximation for equal follow-up.",
    distribution = list(type = "z", mean = z_ncp)
  )
}

.gp_cox_regression <- function(params, analysis) {
  hr <- .gp_numeric(params, "hazard_ratio", 1.5, min = 0, strict_min = TRUE)
  events <- .gp_count(params, "events", .gp_arg(params, "n", 100), min = 2)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)

  log_hr <- log(hr)
  z_ncp <- abs(log_hr) * sqrt(events)
  pwr <- .gp_power_z(z_ncp, alpha, tails)

  .gp_result(
    "Biomarker: Cox proportional hazards (single covariate)", analysis,
    list(tails = pwr$direction, hazard_ratio = hr, events = events, alpha = alpha),
    list(z_statistic = z_ncp, power = pwr$power),
    notes = "Wald test power from expected number of events.",
    distribution = list(type = "z", mean = z_ncp)
  )
}

.gp_discovery_fdr <- function(params, analysis) {
  m_tests <- .gp_count(params, "m_tests", 1000, min = 2)
  pi0 <- .gp_numeric(params, "pi0", 0.9, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  effect_d <- .gp_numeric(params, "effect_d", 0.5, min = 0, strict_min = TRUE)
  n_per_test <- .gp_count(params, "n", 30, min = 2)
  fdr <- .gp_numeric(params, "fdr_level", 0.05, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  alpha <- .gp_alpha(params)

  m1 <- ceiling(m_tests * (1 - pi0))
  df <- n_per_test - 1
  ncp <- effect_d * sqrt(n_per_test)
  pwr_single <- .gp_power_t(df, ncp, alpha, "two")$power
  expected_discoveries <- m1 * pwr_single
  expected_fdr <- (m_tests - m1) * alpha / max(expected_discoveries, 1e-8)
  power <- if (expected_fdr <= fdr) pwr_single else pwr_single * (fdr / expected_fdr)

  .gp_result(
    "Biomarker: Discovery power under FDR control", analysis,
    list(
      m_tests = m_tests, proportion_null = pi0, effect_size_d = effect_d,
      n_per_comparison = n_per_test, fdr_level = fdr, alpha = alpha
    ),
    list(
      alternative_hypotheses = m1, single_test_power = pwr_single,
      expected_discoveries = expected_discoveries, expected_fdr = expected_fdr,
      power = min(power, 1)
    ),
    notes = "BH-FDR framework with independent t-test approximations per biomarker.",
    distribution = list(type = "t", df = df, ncp = ncp)
  )
}

.gp_ttest_biomarker <- function(params, analysis) {
  result <- .gp_t_two_sample(params, analysis)
  result$test <- "Biomarker: Two-group differential expression (t test)"
  if (!is.null(result$inputs$effect_size_d)) {
    result$inputs$log_fold_change_sd <- result$inputs$effect_size_d
    result$inputs$effect_size_d <- NULL
  }
  result
}

#' ggpower_one_sample UI Function
#'
#' @description A shiny Module.
#'
#' @param id,input,output,session Internal parameters for {shiny}.
#'
#' @noRd
#'
#' @importFrom shiny NS tagList
mod_ggpower_one_sample_ui <- function(id) {
  ns <- shiny::NS(id)
  shiny::tagList(
    shiny::sliderInput(
      ns("n_range"),
      "Total Sample Size:",
      min = 20, max = 200, value = c(20, 200), step = 5
    ),
    shiny::numericInput(
      ns("d"),
      "Effect Size (d):",
      value = 0.5
    ),
    shiny::numericInput(
      ns("alpha"),
      "Alpha:",
      value = 0.05, min = 0, max = 1, step = 0.01
    ),
    shiny::plotOutput(ns("powerPlot"))
  )
}

#' ggpower_one_sample Server Functions
#'
#' @noRd
mod_ggpower_one_sample_server <- function(id) {
  shiny::moduleServer(id, function(input, output, session) {
    output$powerPlot <- shiny::renderPlot({
      # Create a sequence of total sample sizes based on the slider input
      n_seq <- seq(input$n_range[1], input$n_range[2], by = 5)
      # Generate and return the ggplot power curve for one-sample t-test
      ggpower_t_one_sample(d = input$d, alpha = input$alpha, n_range = n_seq)
    })
  })
}

## To be copied in the UI
# mod_ggpower_one_sample_ui("ggpower_one_sample_1")

## To be copied in the server
# mod_ggpower_one_sample_server("ggpower_one_sample_1")

mod_calculator_ui <- function(id) {
  ns <- shiny::NS(id)
  shiny::div(
    class = "gp-window gp-page gp-page-fit",
    gp_page_header_ui(
      title = "Calculator",
      subtitle = "Evaluate distribution functions and statistical expressions interactively.",
      compact = TRUE
    ),
    shiny::div(
      class = "gp-page-panel gp-calculator-panel gp-calculator-split",
      shiny::div(
        class = "gp-calculator-editor",
        shiny::h4("Expression editor"),
        shiny::div(
          class = "gp-field",
          shiny::textAreaInput(
            ns("calculator_script"),
            "Expressions",
            value = "alpha <- 0.05\nnctcdf(1.699127, 29, 3.423266)",
            width = "100%",
            rows = 6
          )
        ),
        shiny::div(
          class = "gp-action-row gp-action-row-inline",
          shiny::actionButton(ns("run_calculator"), "Run", class = "btn-primary")
        )
      ),
      shiny::div(
        class = "gp-calculator-output",
        shiny::h4("Output"),
        shiny::div(
          class = "gp-terminal",
          shiny::verbatimTextOutput(ns("calculator_result"), placeholder = TRUE)
        )
      )
    )
  )
}

mod_calculator_server <- function(id) {
  shiny::moduleServer(id, function(input, output, session) {
    calculator_result <- shiny::reactiveVal(NULL)

    output$calculator_result <- shiny::renderPrint({
      result <- calculator_result()
      if (is.null(result)) {
        return("Enter expressions and click Run.")
      }
      result
    })

    shiny::observeEvent(input$run_calculator, {
      result <- tryCatch(
        ggpower_calculator(input$calculator_script),
        error = function(e) e
      )
      if (inherits(result, "error")) {
        shiny::showNotification(result$message, type = "error")
        calculator_result(result$message)
        return()
      }
      calculator_result(result)
    })
  })
}

#' Access files in the current app
#'
#' NOTE: If you manually change your package name in the DESCRIPTION,
#' don't forget to change it here too, and in the config file.
#' For a safer name change mechanism, use the `golem::set_golem_name()` function.
#'
#' @param ... character vectors, specifying subdirectory and file(s)
#' within your package. The default, none, returns the root of the app.
#'
#' @noRd
app_sys <- function(...) {
  system.file(..., package = "ggpower")
}


#' Read App Config
#'
#' @param value Value to retrieve from the config file.
#' @param config GOLEM_CONFIG_ACTIVE value. If unset, R_CONFIG_ACTIVE.
#' If unset, "default".
#' @param use_parent Logical, scan the parent directory for config file.
#' @param file Location of the config file
#'
#' @noRd
get_golem_config <- function(
  value,
  config = Sys.getenv(
    "GOLEM_CONFIG_ACTIVE",
    Sys.getenv(
      "R_CONFIG_ACTIVE",
      "default"
    )
  ),
  use_parent = TRUE,
  # Modify this if your config file is somewhere else
  file = app_sys("golem-config.yml")
) {
  config::get(
    value = value,
    config = config,
    file = file,
    use_parent = use_parent
  )
}

`%||%` <- function(x, y) {
  if (is.null(x) || length(x) == 0 || (length(x) == 1 && is.na(x))) y else x
}

.gp_arg <- function(params, name, default = NULL) {
  params[[name]] %||% default
}

.gp_numeric <- function(params, name, default = NULL, min = -Inf, max = Inf,
                        strict_min = FALSE, strict_max = FALSE) {
  value <- .gp_arg(params, name, default)
  if (is.null(value) || !is.numeric(value) || length(value) != 1 || is.na(value)) {
    stop("'", name, "' must be a single numeric value.", call. = FALSE)
  }
  if ((strict_min && value <= min) || (!strict_min && value < min)) {
    stop("'", name, "' is below the allowed range.", call. = FALSE)
  }
  if ((strict_max && value >= max) || (!strict_max && value > max)) {
    stop("'", name, "' is above the allowed range.", call. = FALSE)
  }
  value
}

.gp_count <- function(params, name, default = NULL, min = 1) {
  value <- ceiling(.gp_numeric(params, name, default, min = min))
  as.integer(value)
}

.gp_alpha <- function(params) {
  .gp_numeric(params, "alpha", 0.05, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
}

.gp_power_target <- function(params) {
  .gp_numeric(params, "power", 0.8, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
}

.gp_tail <- function(params) {
  tails <- .gp_arg(params, "tails", "two")
  tails <- tolower(as.character(tails))
  if (tails %in% c("two", "two.sided", "two-sided", "2")) {
    return("two")
  }
  if (tails %in% c("one", "one.sided", "one-sided", "1", "greater", "less")) {
    return("one")
  }
  stop("'tails' must be 'one' or 'two'.", call. = FALSE)
}

.gp_tail_direction <- function(effect, tails) {
  if (tails == "two") {
    return("two")
  }
  if (effect < 0) "less" else "greater"
}

.gp_power_t <- function(df, ncp, alpha, tails = "two") {
  if (tails == "two") {
    critical <- stats::qt(1 - alpha / 2, df)
    power <- stats::pt(-critical, df, ncp = ncp) +
      1 - stats::pt(critical, df, ncp = ncp)
    return(list(
      power = power,
      critical = c(lower = -critical, upper = critical),
      direction = "two"
    ))
  }

  if (ncp < 0) {
    critical <- stats::qt(alpha, df)
    power <- stats::pt(critical, df, ncp = ncp)
    direction <- "less"
  } else {
    critical <- stats::qt(1 - alpha, df)
    power <- 1 - stats::pt(critical, df, ncp = ncp)
    direction <- "greater"
  }

  list(power = power, critical = critical, direction = direction)
}

.gp_power_z <- function(mean_h1, alpha, tails = "two", sd_h1 = 1) {
  if (tails == "two") {
    critical <- stats::qnorm(1 - alpha / 2)
    power <- stats::pnorm(-critical, mean = mean_h1, sd = sd_h1) +
      1 - stats::pnorm(critical, mean = mean_h1, sd = sd_h1)
    return(list(
      power = power,
      critical = c(lower = -critical, upper = critical),
      direction = "two"
    ))
  }

  if (mean_h1 < 0) {
    critical <- stats::qnorm(alpha)
    power <- stats::pnorm(critical, mean = mean_h1, sd = sd_h1)
    direction <- "less"
  } else {
    critical <- stats::qnorm(1 - alpha)
    power <- 1 - stats::pnorm(critical, mean = mean_h1, sd = sd_h1)
    direction <- "greater"
  }

  list(power = power, critical = critical, direction = direction)
}

.gp_power_f <- function(df1, df2, ncp, alpha) {
  critical <- stats::qf(1 - alpha, df1 = df1, df2 = df2)
  power <- 1 - stats::pf(critical, df1 = df1, df2 = df2, ncp = ncp)
  list(power = power, critical = critical)
}

.gp_power_chisq <- function(df, ncp, alpha) {
  critical <- stats::qchisq(1 - alpha, df = df)
  power <- 1 - stats::pchisq(critical, df = df, ncp = ncp)
  list(power = power, critical = critical)
}

.gp_solve_root <- function(fn, lower, upper, name = "parameter") {
  f_lower <- fn(lower)
  f_upper <- fn(upper)

  if (!is.finite(f_lower) || !is.finite(f_upper) || f_lower * f_upper > 0) {
    stop("Could not bracket a solution for ", name, ".", call. = FALSE)
  }

  stats::uniroot(fn, lower = lower, upper = upper, tol = 1e-8)$root
}

.gp_phi_to_h <- function(p1, p2) {
  2 * asin(sqrt(p1)) - 2 * asin(sqrt(p2))
}

.gp_fisher_z <- function(r) {
  0.5 * log((1 + r) / (1 - r))
}

.gp_check_correlation <- function(r, name = "correlation") {
  if (!is.numeric(r) || length(r) != 1 || is.na(r) || r <= -1 || r >= 1) {
    stop("'", name, "' must be between -1 and 1.", call. = FALSE)
  }
  r
}

.gp_positive_semidefinite <- function(mat, tolerance = 1e-8) {
  if (!is.matrix(mat) || nrow(mat) != ncol(mat)) {
    return(FALSE)
  }
  if (!isTRUE(all.equal(mat, t(mat), tolerance = tolerance))) {
    return(FALSE)
  }
  min(eigen(mat, symmetric = TRUE, only.values = TRUE)$values) >= -tolerance
}

.gp_var_x <- function(params) {
  .gp_numeric(params, "x_variance", 1, min = 0, strict_min = TRUE)
}

.gp_notes_method <- function(...) {
  paste(..., collapse = "")
}

mod_clinical_ui <- function(id) {
  n_tests <- nrow(ggpower_tests(module = "clinical"))
  mod_power_core_ui(
    id,
    list(
      module_filter = "clinical",
      title = "Clinical trials",
      subtitle = "Power for superiority, non-inferiority, equivalence, Simon two-stage, cluster RCT, and survival endpoints.",
      badges = list(
        list(label = paste0(n_tests, " trial designs")),
        list(label = "Phase II / III", class = "gp-badge gp-badge-accent")
      ),
      tip = "Protocol planning for randomized trials. Use non-inferiority or equivalence when the goal is similarity rather than superiority."
    )
  )
}

mod_clinical_server <- function(id, protocol = NULL) {
  mod_power_core_server(
    id,
    list(module_filter = "clinical"),
    protocol = protocol
  )
}

#' Shared UI components for the ggpower Shiny app
#' @noRd
gp_brand_ui <- function(compact = FALSE, grad_id = "gpLogoGrad") {
  shiny::tags$span(
    class = if (compact) "gp-brand gp-brand-compact" else "gp-brand",
    shiny::tags$span(
      class = "gp-brand-logo",
      shiny::HTML(sprintf(
        '<svg viewBox="0 0 40 40" aria-hidden="true" focusable="false" role="img">
          <defs>
            <linearGradient id="%1$s" x1="0%%" y1="0%%" x2="100%%" y2="100%%">
              <stop offset="0%%" stop-color="#0c4a6e"/>
              <stop offset="55%%" stop-color="#0e7490"/>
              <stop offset="100%%" stop-color="#22d3ee"/>
            </linearGradient>
            <linearGradient id="%1$sGlow" x1="0%%" y1="100%%" x2="100%%" y2="0%%">
              <stop offset="0%%" stop-color="#ffffff" stop-opacity="0"/>
              <stop offset="100%%" stop-color="#ffffff" stop-opacity="0.22"/>
            </linearGradient>
          </defs>
          <rect x="1" y="1" width="38" height="38" rx="11" fill="url(#%1$s)"/>
          <rect x="1" y="1" width="38" height="38" rx="11" fill="url(#%1$sGlow)"/>
          <path d="M10 27 C13.5 27, 15.5 22, 18.5 22 C21.5 22, 23 27, 27 27" fill="none" stroke="#ffffff" stroke-width="2.2" stroke-linecap="round" opacity="0.35"/>
          <path d="M11 26 C14.2 24.2, 16.4 19.5, 20 16.5 C23.2 13.8, 26.5 12.5, 29.5 11.5" fill="none" stroke="#ffffff" stroke-width="2.6" stroke-linecap="round"/>
          <circle cx="29.5" cy="11.5" r="2.4" fill="#ecfeff"/>
          <rect x="11" y="24" width="3" height="6" rx="1" fill="#ffffff" opacity="0.55"/>
          <rect x="16" y="21" width="3" height="9" rx="1" fill="#ffffff" opacity="0.72"/>
          <rect x="21" y="18" width="3" height="12" rx="1" fill="#ffffff" opacity="0.9"/>
        </svg>',
        grad_id
      ))
    ),
    shiny::tags$span(
      class = "gp-brand-wordmark",
      shiny::tags$span(class = "gp-brand-gg", "gg"),
      shiny::tags$span(class = "gp-brand-power", "power")
    )
  )
}

#' @noRd
gp_page_header_ui <- function(title, subtitle, badges = NULL, compact = FALSE) {
  shiny::div(
    class = if (compact) "gp-header gp-header-compact" else "gp-header",
    shiny::div(
      class = "gp-header-copy",
      shiny::h1(class = "gp-title", title),
      if (!compact && !is.null(subtitle) && nzchar(subtitle)) {
        shiny::p(class = "gp-subtitle", subtitle)
      }
    ),
    if (!is.null(badges) && length(badges) > 0) {
      shiny::div(class = "gp-header-badges", badges)
    }
  )
}

#' @noRd
gp_inline_tip_ui <- function(text) {
  shiny::div(class = "gp-inline-tip", shiny::icon("circle-info"), shiny::span(text))
}

#' @noRd
gp_select_ui <- function(id, label, choices, selected = NULL, width = "100%") {
  shiny::div(
    class = "gp-field",
    shiny::selectInput(
      inputId = id,
      label = label,
      choices = choices,
      selected = selected,
      width = width
    )
  )
}

#' Run the Shiny Application
#'
#' @param ... arguments to pass to golem_opts.
#' See `?golem::get_golem_options` for more details.
#' @inheritParams shiny::shinyApp
#'
#' @return A Shiny application object (class `"shiny.appobj"`), returned
#'   invisibly. Launch the GUI with `shiny::runApp(run_app())` or from the
#'   development helper described in the package overview.
#' @export
#' @importFrom shiny shinyApp
#' @importFrom golem with_golem_options
run_app <- function(
  onStart = NULL,
  options = list(),
  enableBookmarking = NULL,
  uiPattern = "/",
  ...
) {
  with_golem_options(
    app = shinyApp(
      ui = app_ui,
      server = app_server,
      onStart = onStart,
      options = options,
      enableBookmarking = enableBookmarking,
      uiPattern = uiPattern
    ),
    golem_opts = list(...)
  )
}

#' Compute Power for a One-Sample t-Test
#'
#' Calculates the power for a one-sample t-test given the effect size (d),
#' total sample size (n), and significance level (alpha).
#'
#' @param d Numeric. The effect size (difference from the constant divided by sigma).
#' @param n Integer. Total sample size.
#' @param alpha Numeric. The significance level (default is 0.05).
#' @param tails Character. `"two"` for a two-tailed test or `"one"` for a one-tailed test.
#'
#' @return Numeric. The computed power (1 - beta).
#' @export
#'
#' @examples
#' # Calculate power for an effect size of 0.5 with n = 40 subjects
#' power_t_one_sample(d = 0.5, n = 40, alpha = 0.05)
power_t_one_sample <- function(d, n, alpha = 0.05, tails = "two") {
  result <- power_compute(
    "t_one_sample",
    analysis = "post_hoc",
    d = d,
    n = n,
    alpha = alpha,
    tails = tails
  )
  result$outputs$power
}

#' Compute Power for a Two-Sample t-Test (Equal Sample Sizes)
#'
#' This function calculates the power for a two-sample t-test when the two groups have equal sample sizes.
#'
#' @param d Numeric. The effect size (Cohen's d).
#' @param n_per_group Integer. The sample size per group.
#' @param alpha Numeric. The significance level (default is 0.05).
#' @param tails Character. `"two"` for a two-tailed test or `"one"` for a one-tailed test.
#' @param n2 Optional second-group sample size. If omitted, equal group sizes are used.
#'
#' @return Numeric. The computed power (1 - beta).
#' @export
#'
#' @examples
#' # Compute power for an effect size d = 0.5 with 30 subjects per group
#' power_t_two_sample(d = 0.5, n_per_group = 30)
power_t_two_sample <- function(d, n_per_group, alpha = 0.05, tails = "two", n2 = NULL) {
  result <- power_compute(
    "t_two_sample",
    analysis = "post_hoc",
    d = d,
    n1 = n_per_group,
    n2 = n2 %||% n_per_group,
    alpha = alpha,
    tails = tails
  )
  result$outputs$power
}

#' Compute Power for a Paired-Samples t-Test
#'
#' @param d Numeric. The paired-samples effect size dz.
#' @param n Integer. Number of pairs.
#' @param alpha Numeric. The significance level.
#' @param tails Character. `"two"` or `"one"`.
#'
#' @return Numeric. The computed power.
#' @export
power_t_paired <- function(d, n, alpha = 0.05, tails = "two") {
  result <- power_compute(
    "t_paired",
    analysis = "post_hoc",
    d = d,
    n = n,
    alpha = alpha,
    tails = tails
  )
  result$outputs$power
}

library(testthat)
library(ggpower)  # Ensure ggpower is loaded

test_that("power_t_two_sample returns a valid power between 0 and 1", {
  pwr <- power_t_two_sample(d = 0.5, n_per_group = 30)
  expect_true(is.numeric(pwr))
  expect_true(pwr > 0 && pwr < 1)
})

test_that("ggpower_ttest returns a ggplot object", {
  p <- ggpower_ttest(d = 0.5, alpha = 0.05, n_range = seq(10, 100, by = 10))
  expect_s3_class(p, "ggplot")
})

test_that("Power increases with sample size", {
  # Compute power for two different sample sizes
  power_small <- power_t_two_sample(d = 0.5, n_per_group = 20)
  power_large <- power_t_two_sample(d = 0.5, n_per_group = 40)
  expect_true(power_large > power_small)
})

#' List supported statistical power tests
#'
#' @param domain Optional character vector to filter by domain (`general`,
#'   `biomarker`, `pharma`).
#' @param module Optional character vector to filter by app module (`workspace`,
#'   `biomarker`, `clinical`).
#' @return A data frame describing tests available to `power_compute()`.
#' @export
ggpower_tests <- function(domain = NULL, module = NULL) {
  specs <- ggpower_test_registry()
  df <- data.frame(
    id = vapply(specs, `[[`, character(1), "id"),
    family = vapply(specs, `[[`, character(1), "family"),
    label = vapply(specs, `[[`, character(1), "label"),
    method = vapply(specs, `[[`, character(1), "method"),
    parity = vapply(specs, `[[`, character(1), "parity"),
    domain = vapply(specs, `[[`, character(1), "domain"),
    module = vapply(specs, `[[`, character(1), "module"),
    modes = vapply(specs, function(x) paste(x$modes, collapse = ", "), character(1)),
    stringsAsFactors = FALSE
  )
  if (!is.null(domain)) {
    df <- df[df$domain %in% domain, , drop = FALSE]
  }
  if (!is.null(module)) {
    df <- df[df$module %in% module, , drop = FALSE]
  }
  df
}

ggpower_test_registry <- function() {
  list(
    t_one_sample = .gp_spec(
      "t_one_sample", "t tests",
      "Means: Difference from constant (one sample case)",
      effect = "d", sample = "n",
      fields = c("tails", "d", "alpha", "power", "n", "q")
    ),
    t_paired = .gp_spec(
      "t_paired", "t tests",
      "Means: Difference between two dependent means (matched pairs)",
      effect = "d", sample = "n",
      fields = c("tails", "d", "alpha", "power", "n", "q")
    ),
    t_two_sample = .gp_spec(
      "t_two_sample", "t tests",
      "Means: Difference between two independent means (two groups)",
      effect = "d", sample = "total_n",
      fields = c("tails", "d", "alpha", "power", "n1", "n2", "allocation_ratio", "q")
    ),
    t_point_biserial = .gp_spec(
      "t_point_biserial", "t tests",
      "Correlation: Point biserial model",
      effect = "rho", sample = "n",
      fields = c("tails", "rho", "alpha", "power", "n", "q")
    ),
    t_linear_regression = .gp_spec(
      "t_linear_regression", "t tests",
      "Linear regression: Size of slope (one group)",
      effect = "slope_h1", sample = "n",
      fields = c("tails", "slope_h1", "slope_h0", "sd_x", "sd_y", "alpha", "power", "n", "q")
    ),
    t_linear_regression_two_groups = .gp_spec(
      "t_linear_regression_two_groups", "t tests",
      "Linear regression: Difference between two slopes (two groups)",
      effect = "delta_slope", sample = "total_n",
      fields = c("tails", "delta_slope", "residual_sd", "sd_x1", "sd_x2", "alpha", "power", "n1", "n2", "allocation_ratio", "q")
    ),
    t_generic = .gp_spec(
      "t_generic", "t tests",
      "Generic t test",
      effect = "ncp", sample = "n",
      modes = c("post_hoc", "criterion", "sensitivity", "compromise"),
      fields = c("tails", "ncp", "alpha", "power", "n", "df", "q")
    ),
    f_anova_one_way = .gp_spec(
      "f_anova_one_way", "F tests",
      "ANOVA: Fixed effects, omnibus, one-way",
      effect = "f", sample = "total_n",
      fields = c("f", "alpha", "power", "total_n", "groups", "q")
    ),
    f_anova_special = .gp_spec(
      "f_anova_special", "F tests",
      "ANOVA: Fixed effects, special, main effects and interactions",
      effect = "f", sample = "total_n",
      fields = c("f", "alpha", "power", "total_n", "df1", "groups", "q")
    ),
    f_mreg_omnibus = .gp_spec(
      "f_mreg_omnibus", "F tests",
      "Multiple regression: Omnibus (R2 deviation from zero)",
      effect = "f2", sample = "total_n",
      fields = c("f2", "alpha", "power", "total_n", "predictors", "q")
    ),
    f_mreg_increase = .gp_spec(
      "f_mreg_increase", "F tests",
      "Multiple regression: Special (R2 increase)",
      effect = "f2", sample = "total_n",
      fields = c("f2", "alpha", "power", "total_n", "tested_predictors", "predictors", "q")
    ),
    f_variance_two = .gp_spec(
      "f_variance_two", "F tests",
      "Variance: Inequality of two variances",
      effect = "variance_ratio", sample = "total_n",
      fields = c("tails", "variance_ratio", "alpha", "power", "n1", "n2", "allocation_ratio", "q")
    ),
    chisq_variance_one = .gp_spec(
      "chisq_variance_one", "chi-square tests",
      "Variance: Difference from constant (one sample case)",
      effect = "variance_ratio", sample = "n",
      fields = c("tails", "variance_ratio", "alpha", "power", "n", "q")
    ),
    chisq_gof = .gp_spec(
      "chisq_gof", "chi-square tests",
      "Goodness-of-fit tests: Contingency tables",
      effect = "w", sample = "total_n",
      fields = c("w", "alpha", "power", "total_n", "df", "q")
    ),
    chisq_contingency = .gp_spec(
      "chisq_contingency", "chi-square tests",
      "Contingency tables",
      effect = "w", sample = "total_n",
      fields = c("w", "alpha", "power", "total_n", "df", "q")
    ),
    exact_binomial = .gp_spec(
      "exact_binomial", "Exact",
      "Generic binomial test",
      effect = "p1", sample = "n",
      fields = c("tails", "p0", "p1", "alpha", "power", "n", "q")
    ),
    exact_one_proportion = .gp_spec(
      "exact_one_proportion", "Exact",
      "Proportion: Difference from constant (one sample case)",
      effect = "p1", sample = "n",
      fields = c("tails", "p0", "p1", "alpha", "power", "n", "q")
    ),
    exact_sign = .gp_spec(
      "exact_sign", "Exact",
      "Proportion: Sign test",
      effect = "p1", sample = "n",
      fields = c("tails", "p0", "p1", "alpha", "power", "n", "q")
    ),
    exact_fisher = .gp_spec(
      "exact_fisher", "Exact",
      "Proportions: Inequality of two independent groups (Fisher exact)",
      effect = "p1", sample = "total_n",
      fields = c("tails", "p0", "p1", "alpha", "power", "n1", "n2", "allocation_ratio", "q")
    ),
    exact_mcnemar = .gp_spec(
      "exact_mcnemar", "Exact",
      "Proportion: Inequality, two dependent groups (McNemar)",
      effect = "p1", sample = "n",
      fields = c("tails", "p0", "p1", "alpha", "power", "n", "q")
    ),
    exact_correlation = .gp_spec(
      "exact_correlation", "Exact",
      "Correlation: Difference from constant (one sample case)",
      effect = "rho", sample = "n",
      fields = c("tails", "rho0", "rho", "alpha", "power", "n", "q"),
      method = "Fisher Z with exact-kernel TODO",
      parity = "approximation"
    ),
    exact_mreg_random = .gp_spec(
      "exact_mreg_random", "Exact",
      "Multiple regression: Random model",
      effect = "rho2", sample = "total_n",
      fields = c("rho2", "alpha", "power", "total_n", "predictors", "q"),
      method = "exact R2 distribution approximation",
      parity = "supported"
    ),
    z_corr_independent = .gp_spec(
      "z_corr_independent", "z tests",
      "Correlation: Inequality of two independent Pearson r's",
      effect = "q_effect", sample = "total_n",
      fields = c("tails", "q_effect", "alpha", "power", "n1", "n2", "allocation_ratio", "q")
    ),
    z_corr_dependent_common = .gp_spec(
      "z_corr_dependent_common", "z tests",
      "Correlation: Inequality of two dependent Pearson r's (common index)",
      effect = "rho_ac", sample = "n",
      fields = c("tails", "rho_ab", "rho_ac", "rho_bc", "alpha", "power", "n", "q")
    ),
    z_corr_dependent_no_common = .gp_spec(
      "z_corr_dependent_no_common", "z tests",
      "Correlation: Inequality of two dependent Pearson r's (no common index)",
      effect = "rho_cd", sample = "n",
      fields = c("tails", "rho_ab", "rho_cd", "rho_ac", "rho_ad", "rho_bc", "rho_bd", "alpha", "power", "n", "q")
    ),
    z_logistic = .gp_spec(
      "z_logistic", "z tests",
      "Regression: Logistic regression",
      effect = "odds_ratio", sample = "total_n",
      fields = c("tails", "odds_ratio", "p0", "alpha", "power", "total_n", "r2_other", "x_variance", "q")
    ),
    z_poisson = .gp_spec(
      "z_poisson", "z tests",
      "Regression: Poisson regression",
      effect = "exp_beta1", sample = "total_n",
      fields = c("tails", "exp_beta1", "base_rate", "exposure", "alpha", "power", "total_n", "r2_other", "x_variance", "q")
    ),
    z_tetrachoric = .gp_spec(
      "z_tetrachoric", "z tests",
      "Correlation: Tetrachoric correlation",
      effect = "rho", sample = "n",
      fields = c("tails", "rho0", "rho", "alpha", "power", "n", "q"),
      method = "large-sample tetrachoric z approximation",
      parity = "approximation"
    ),
    wilcoxon_signed = .gp_spec(
      "wilcoxon_signed", "nonparametric",
      "Wilcoxon signed-rank test: One sample or matched pairs",
      effect = "d", sample = "n",
      fields = c("tails", "d", "alpha", "power", "n", "are", "q")
    ),
    wilcoxon_mann_whitney = .gp_spec(
      "wilcoxon_mann_whitney", "nonparametric",
      "Wilcoxon-Mann-Whitney test: Two independent means",
      effect = "d", sample = "total_n",
      fields = c("tails", "d", "alpha", "power", "n1", "n2", "allocation_ratio", "are", "q")
    ),
    roc_auc_one = .gp_spec(
      "roc_auc_one", "biomarker",
      "ROC AUC: One sample vs null AUC",
      effect = "auc", sample = "total_n",
      fields = c("tails", "auc", "auc0", "n_pos", "n_neg", "alpha", "power", "q"),
      method = "Hanley-McNeil AUC variance approximation",
      domain = "biomarker", module = "biomarker"
    ),
    roc_auc_two = .gp_spec(
      "roc_auc_two", "biomarker",
      "ROC AUC: Compare two independent AUCs",
      effect = "auc1", sample = "total_n",
      fields = c("tails", "auc1", "auc2", "n1", "n2", "alpha", "power", "q"),
      method = "DeLong-style AUC difference approximation",
      domain = "biomarker", module = "biomarker"
    ),
    diagnostic_acc = .gp_spec(
      "diagnostic_acc", "biomarker",
      "Diagnostic accuracy: Sensitivity and specificity",
      effect = "sensitivity", sample = "total_n",
      fields = c("tails", "sensitivity", "specificity", "n_pos", "n_neg", "alpha", "power", "q"),
      method = "Binomial normal approximation",
      domain = "biomarker", module = "biomarker"
    ),
    survival_logrank = .gp_spec(
      "survival_logrank", "biomarker",
      "Survival: Log-rank test",
      effect = "hazard_ratio", sample = "total_n",
      fields = c("tails", "hazard_ratio", "event_rate", "allocation_ratio", "total_n", "alpha", "power", "q"),
      method = "Schoenfeld/Freedman log-rank approximation",
      domain = "biomarker", module = "biomarker"
    ),
    cox_regression = .gp_spec(
      "cox_regression", "biomarker",
      "Survival: Cox PH single covariate",
      effect = "hazard_ratio", sample = "events",
      fields = c("tails", "hazard_ratio", "events", "alpha", "power", "q"),
      method = "Cox Wald z approximation",
      domain = "biomarker", module = "biomarker"
    ),
    discovery_fdr = .gp_spec(
      "discovery_fdr", "biomarker",
      "Discovery: Multiplicity-adjusted FDR screening",
      effect = "effect_d", sample = "n",
      fields = c("m_tests", "pi0", "effect_d", "n", "fdr_level", "alpha", "power", "q"),
      method = "Benjamini-Hochberg FDR framework",
      domain = "biomarker", module = "biomarker"
    ),
    ttest_biomarker = .gp_spec(
      "ttest_biomarker", "biomarker",
      "Differential expression: Two-group t test",
      effect = "d", sample = "total_n",
      fields = c("tails", "d", "alpha", "power", "n1", "n2", "allocation_ratio", "q"),
      method = "Two-sample t test (biomarker wrapper)",
      domain = "biomarker", module = "biomarker"
    ),
    rct_superiority_continuous = .gp_spec(
      "rct_superiority_continuous", "clinical",
      "RCT superiority: Continuous endpoint",
      effect = "d", sample = "total_n",
      fields = c("d", "alpha", "power", "n1", "n2", "allocation_ratio", "q"),
      method = "One-sided two-sample t test",
      domain = "pharma", module = "clinical"
    ),
    rct_superiority_binary = .gp_spec(
      "rct_superiority_binary", "clinical",
      "RCT superiority: Binary endpoint",
      effect = "p1", sample = "total_n",
      fields = c("p0", "p1", "alpha", "power", "n1", "n2", "allocation_ratio", "q"),
      method = "One-sided Fisher exact / proportions",
      domain = "pharma", module = "clinical"
    ),
    rct_noninferiority_continuous = .gp_spec(
      "rct_noninferiority_continuous", "clinical",
      "Non-inferiority: Continuous endpoint",
      effect = "d", sample = "total_n",
      fields = c("d", "ni_margin", "alpha", "power", "n1", "n2", "allocation_ratio", "q"),
      method = "One-sided NI t test",
      domain = "pharma", module = "clinical"
    ),
    rct_noninferiority_binary = .gp_spec(
      "rct_noninferiority_binary", "clinical",
      "Non-inferiority: Binary endpoint",
      effect = "p1", sample = "total_n",
      fields = c("p0", "p1", "ni_margin", "alpha", "power", "n1", "n2", "allocation_ratio", "q"),
      method = "One-sided NI proportions test",
      domain = "pharma", module = "clinical"
    ),
    rct_equivalence_continuous = .gp_spec(
      "rct_equivalence_continuous", "clinical",
      "Equivalence: Continuous endpoint (TOST)",
      effect = "d", sample = "total_n",
      fields = c("d", "eq_margin", "alpha", "power", "n1", "n2", "allocation_ratio", "q"),
      method = "Two one-sided t tests (TOST)",
      domain = "pharma", module = "clinical"
    ),
    rct_equivalence_proportion = .gp_spec(
      "rct_equivalence_proportion", "clinical",
      "Equivalence: Binary endpoint (TOST)",
      effect = "p1", sample = "total_n",
      fields = c("p0", "p1", "eq_margin", "alpha", "power", "n1", "n2", "allocation_ratio", "q"),
      method = "Two one-sided proportion tests (TOST)",
      domain = "pharma", module = "clinical"
    ),
    simon_two_stage = .gp_spec(
      "simon_two_stage", "clinical",
      "Simon two-stage Phase II design",
      effect = "p1", sample = "total_n",
      fields = c("p0", "p1", "alpha", "power", "n1", "r1", "n2", "r"),
      method = "Simon 1989 two-stage binomial design",
      domain = "pharma", module = "clinical",
      modes = c("post_hoc", "sensitivity")
    ),
    cluster_rct = .gp_spec(
      "cluster_rct", "clinical",
      "Cluster-randomized trial",
      effect = "d", sample = "total_n",
      fields = c("tails", "d", "icc", "cluster_size", "n_clusters", "alpha", "power", "q"),
      method = "Design effect from ICC",
      domain = "pharma", module = "clinical"
    ),
    multi_arm_superiority = .gp_spec(
      "multi_arm_superiority", "clinical",
      "Multi-arm superiority (ANOVA)",
      effect = "f", sample = "total_n",
      fields = c("f", "alpha", "power", "total_n", "groups", "q"),
      method = "One-way ANOVA F test",
      domain = "pharma", module = "clinical"
    ),
    count_endpoint_poisson = .gp_spec(
      "count_endpoint_poisson", "clinical",
      "Count endpoint: Poisson regression",
      effect = "exp_beta1", sample = "total_n",
      fields = c("tails", "exp_beta1", "base_rate", "exposure", "alpha", "power", "total_n", "r2_other", "x_variance", "q"),
      method = "Poisson regression z approximation",
      domain = "pharma", module = "clinical"
    ),
    survival_pmu = .gp_spec(
      "survival_pmu", "clinical",
      "Survival endpoint: Log-rank primary analysis",
      effect = "hazard_ratio", sample = "total_n",
      fields = c("tails", "hazard_ratio", "event_rate", "allocation_ratio", "total_n", "alpha", "power", "q"),
      method = "Schoenfeld/Freedman log-rank approximation",
      domain = "pharma", module = "clinical"
    )
  )
}

ggpower_get_test <- function(test) {
  registry <- ggpower_test_registry()
  if (!test %in% names(registry)) {
    stop(
      "Unknown test '", test, "'. Use ggpower_tests() to list supported tests.",
      call. = FALSE
    )
  }
  registry[[test]]
}

.gp_spec <- function(id, family, label, effect, sample,
                     modes = c("a_priori", "compromise", "criterion", "post_hoc", "sensitivity"),
                     fields, method = "distribution kernel",
                     parity = "supported",
                     domain = "general", module = "workspace") {
  list(
    id = id,
    family = family,
    label = label,
    effect = effect,
    sample = sample,
    modes = modes,
    fields = fields,
    method = method,
    parity = parity,
    domain = domain,
    module = module
  )
}

test_that("clinical kernels return valid post_hoc power", {
  r1 <- power_compute("rct_superiority_continuous", analysis = "post_hoc", d = 0.5, n1 = 50, n2 = 50)
  expect_s3_class(r1, "ggpower_result")
  expect_true(r1$outputs$power > 0.5)

  r2 <- power_compute("rct_superiority_binary", analysis = "post_hoc", p0 = 0.3, p1 = 0.45, n1 = 120, n2 = 120)
  expect_true(r2$outputs$power > 0.4)

  r3 <- power_compute(
    "rct_noninferiority_continuous", analysis = "post_hoc",
    d = 0, ni_margin = 0.2, n1 = 80, n2 = 80
  )
  expect_true(r3$outputs$power > 0.3)

  r4 <- power_compute(
    "rct_noninferiority_binary", analysis = "post_hoc",
    p0 = 0.5, p1 = 0.55, ni_margin = 0.1, n1 = 200, n2 = 200
  )
  expect_true(r4$outputs$power > 0.4)

  r5 <- power_compute(
    "rct_equivalence_continuous", analysis = "post_hoc",
    d = 0, eq_margin = 0.25, n1 = 60, n2 = 60
  )
  expect_true(r5$outputs$power > 0.1)

  r6 <- power_compute(
    "simon_two_stage", analysis = "post_hoc",
    p0 = 0.2, p1 = 0.35, n1 = 15, r1 = 4, n2 = 20, r = 10
  )
  expect_true(r6$outputs$power > 0.3)

  r7 <- power_compute(
    "cluster_rct", analysis = "post_hoc",
    d = 0.5, icc = 0.05, cluster_size = 10, n_clusters = 20
  )
  expect_true(r7$outputs$power > 0.4)

  r8 <- power_compute("survival_pmu", analysis = "post_hoc", hazard_ratio = 0.7, event_rate = 0.6, total_n = 250)
  expect_true(r8$outputs$power > 0.4)
})

test_that("registry domain tags are present", {
  tests <- ggpower_tests()
  expect_true(all(c("domain", "module") %in% names(tests)))
  expect_true(any(tests$domain == "biomarker"))
  expect_true(any(tests$domain == "pharma"))
  expect_equal(nrow(ggpower_tests(module = "biomarker")), 7)
  expect_equal(nrow(ggpower_tests(module = "clinical")), 11)
})

mod_biomarker_ui <- function(id) {
  n_tests <- nrow(ggpower_tests(module = "biomarker"))
  mod_power_core_ui(
    id,
    list(
      module_filter = "biomarker",
      title = "Biomarker discovery",
      subtitle = "Power for ROC/AUC, diagnostic accuracy, survival, Cox regression, and FDR-controlled screening.",
      badges = list(
        list(label = paste0(n_tests, " biomarker tests")),
        list(label = "Discovery workflows", class = "gp-badge gp-badge-accent")
      ),
      tip = "Biomarker validation, omics screening, and diagnostic classifiers. Start with ROC AUC or differential expression; use FDR for panel studies."
    )
  )
}

mod_biomarker_server <- function(id, protocol = NULL) {
  mod_power_core_server(
    id,
    list(module_filter = "biomarker"),
    protocol = protocol
  )
}

#' Evaluate a ggpower calculator script
#'
#' Evaluates distribution-function calculator expressions, including helpers
#' such as `zcdf()`, `tinv()`, and `ncfcdf()`.
#'
#' @param script Character calculator script. Lines may contain R-style
#'   arithmetic, assignments, comments starting with `#`, and distribution
#'   helper functions such as `zcdf()`, `tinv()`, and `ncfcdf()`.
#'
#' @return The value of the final expression.
#' @export
ggpower_calculator <- function(script) {
  env <- .gp_calculator_env()
  expr <- parse(text = script)
  value <- NULL
  for (i in seq_along(expr)) {
    value <- eval(expr[[i]], envir = env)
  }
  value
}

.gp_calculator_env <- function() {
  env <- new.env(parent = baseenv())
  env$abs <- base::abs
  env$sin <- base::sin
  env$asin <- base::asin
  env$cos <- base::cos
  env$acos <- base::acos
  env$tan <- base::tan
  env$atan <- base::atan
  env$atan2 <- base::atan2
  env$exp <- base::exp
  env$log <- base::log
  env$sqrt <- base::sqrt
  env$sqr <- function(x) x^2
  env$sign <- base::sign
  env$lngamma <- base::lgamma
  env$frac <- function(x) x - floor(x)
  env$int <- base::trunc
  env$min <- base::min
  env$max <- base::max
  env$uround <- function(x, m = 1) ceiling(x / m) * m

  env$zcdf <- stats::pnorm
  env$zpdf <- stats::dnorm
  env$zinv <- stats::qnorm
  env$normcdf <- function(x, m = 0, s = 1) stats::pnorm(x, m, s)
  env$normpdf <- function(x, m = 0, s = 1) stats::dnorm(x, m, s)
  env$norminv <- function(p, m = 0, s = 1) stats::qnorm(p, m, s)
  env$chi2cdf <- stats::pchisq
  env$chi2pdf <- stats::dchisq
  env$chi2inv <- stats::qchisq
  env$fcdf <- stats::pf
  env$fpdf <- stats::df
  env$finv <- stats::qf
  env$tcdf <- stats::pt
  env$tpdf <- stats::dt
  env$tinv <- stats::qt
  env$ncx2cdf <- function(x, df, nc) stats::pchisq(x, df, ncp = nc)
  env$ncx2pdf <- function(x, df, nc) stats::dchisq(x, df, ncp = nc)
  env$ncx2inv <- function(p, df, nc) stats::qchisq(p, df, ncp = nc)
  env$ncfcdf <- function(x, df1, df2, nc) stats::pf(x, df1, df2, ncp = nc)
  env$ncfpdf <- function(x, df1, df2, nc) stats::df(x, df1, df2, ncp = nc)
  env$ncfinv <- function(p, df1, df2, nc) stats::qf(p, df1, df2, ncp = nc)
  env$nctcdf <- function(x, df, nc) stats::pt(x, df, ncp = nc)
  env$nctpdf <- function(x, df, nc) stats::dt(x, df, ncp = nc)
  env$nctinv <- function(p, df, nc) stats::qt(p, df, ncp = nc)
  env$binocdf <- stats::pbinom
  env$binopdf <- stats::dbinom
  env$binoinv <- stats::qbinom
  env$poisscdf <- stats::ppois
  env$poisspdf <- stats::dpois
  env$poissinv <- stats::qpois
  env$betacdf <- stats::pbeta
  env$betapdf <- stats::dbeta
  env$betainv <- stats::qbeta
  env
}

#' Compute Cohen's d from means and standard deviation
#'
#' @param mean_h1 Mean assumed under H1.
#' @param mean_h0 Mean assumed under H0 or comparison group mean.
#' @param sd Common standard deviation.
#'
#' @return Numeric effect size d.
#' @export
effect_size_d <- function(mean_h1, mean_h0 = 0, sd) {
  sd <- .gp_numeric(list(sd = sd), "sd", min = 0, strict_min = TRUE)
  (mean_h1 - mean_h0) / sd
}

#' Compute Cohen's f from eta-squared
#'
#' @param eta2 Proportion of variance explained.
#'
#' @return Numeric effect size f.
#' @export
effect_size_f <- function(eta2) {
  eta2 <- .gp_numeric(list(eta2 = eta2), "eta2", min = 0, max = 1, strict_max = TRUE)
  sqrt(eta2 / (1 - eta2))
}

#' Convert Cohen's f to eta-squared
#'
#' @param f Cohen's f.
#'
#' @return Numeric eta-squared value.
#' @export
eta2_from_f <- function(f) {
  f <- .gp_numeric(list(f = f), "f", min = 0)
  f^2 / (1 + f^2)
}

#' Compute Cohen's f2 from R-squared
#'
#' @param r2 Coefficient of determination.
#'
#' @return Numeric effect size f2.
#' @export
effect_size_f2 <- function(r2) {
  r2 <- .gp_numeric(list(r2 = r2), "r2", min = 0, max = 1, strict_max = TRUE)
  r2 / (1 - r2)
}

#' Compute f2 for an R-squared increase
#'
#' @param r2_full R-squared for the full model.
#' @param r2_reduced R-squared for the reduced model.
#'
#' @return Numeric effect size f2.
#' @export
effect_size_f2_increase <- function(r2_full, r2_reduced) {
  r2_full <- .gp_numeric(list(r2_full = r2_full), "r2_full", min = 0, max = 1, strict_max = TRUE)
  r2_reduced <- .gp_numeric(list(r2_reduced = r2_reduced), "r2_reduced", min = 0, max = 1)
  if (r2_reduced > r2_full) {
    stop("'r2_reduced' cannot be greater than 'r2_full'.", call. = FALSE)
  }
  (r2_full - r2_reduced) / (1 - r2_full)
}

#' Convert f2 to R-squared
#'
#' @param f2 Cohen's f2.
#'
#' @return Numeric R-squared value.
#' @export
r2_from_f2 <- function(f2) {
  f2 <- .gp_numeric(list(f2 = f2), "f2", min = 0)
  f2 / (1 + f2)
}

#' Compute chi-square effect size w
#'
#' @param p0 Probabilities under H0.
#' @param p1 Probabilities under H1.
#'
#' @return Numeric effect size w.
#' @export
effect_size_w <- function(p0, p1) {
  if (length(p0) != length(p1)) {
    stop("'p0' and 'p1' must have the same length.", call. = FALSE)
  }
  if (any(p0 <= 0) || any(p1 < 0)) {
    stop("'p0' must be positive and 'p1' must be non-negative.", call. = FALSE)
  }
  p0 <- p0 / sum(p0)
  p1 <- p1 / sum(p1)
  sqrt(sum((p1 - p0)^2 / p0))
}

#' Compute Cohen's q for two correlations
#'
#' @param r1,r2 Correlations to compare.
#'
#' @return Numeric Fisher-z difference q.
#' @export
effect_size_q <- function(r1, r2) {
  r1 <- .gp_check_correlation(r1, "r1")
  r2 <- .gp_check_correlation(r2, "r2")
  .gp_fisher_z(r1) - .gp_fisher_z(r2)
}

#' Compute odds ratio from two probabilities
#'
#' @param p0 Baseline event probability.
#' @param p1 Event probability under H1.
#'
#' @return Numeric odds ratio.
#' @export
odds_ratio_from_probs <- function(p0, p1) {
  p0 <- .gp_numeric(list(p0 = p0), "p0", min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  p1 <- .gp_numeric(list(p1 = p1), "p1", min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  (p1 / (1 - p1)) / (p0 / (1 - p0))
}

#' Compute Cohen's h for two proportions
#'
#' @param p1,p2 Proportions.
#'
#' @return Numeric h.
#' @export
effect_size_h <- function(p1, p2) {
  p1 <- .gp_numeric(list(p1 = p1), "p1", min = 0, max = 1)
  p2 <- .gp_numeric(list(p2 = p2), "p2", min = 0, max = 1)
  .gp_phi_to_h(p1, p2)
}

test_that("test registry exposes required metadata", {
  tests <- ggpower_tests()

  expect_true(nrow(tests) >= 20)
  expect_true(all(c("id", "family", "label") %in% names(tests)))
  expect_true(all(nzchar(tests$id)))
  expect_true(all(nzchar(tests$family)))
  expect_true(all(nzchar(tests$label)))
})

test_that("registered tests have compute specs", {
  registry <- ggpower_test_registry()

  for (id in names(registry)) {
    spec <- ggpower_get_test(id)
    expect_equal(spec$id, id)
    expect_true(length(spec$modes) >= 1)
    expect_true(length(spec$fields) >= 1)
    expect_true(is.character(spec$effect))
    expect_true(is.character(spec$sample))
  }
})

test_that("effect size helpers compute standard conversions", {
  expect_equal(effect_size_d(15, 10, 8), 0.625)
  expect_equal(effect_size_f(0.2), 0.5)
  expect_equal(eta2_from_f(0.5), 0.2)
  expect_equal(effect_size_f2(0.1), 0.1111111, tolerance = 1e-6)
  expect_equal(r2_from_f2(0.1111111), 0.1, tolerance = 1e-7)
  expect_equal(odds_ratio_from_probs(0.5, 0.6), 1.5)
})

#' Plot Power Curve for a Two-Sample t-Test
#'
#' This function creates a ggplot2 power curve for a two-sample t test.
#'
#' @param d Numeric. The effect size (Cohen's d).
#' @param alpha Numeric. The significance level (default 0.05).
#' @param n_range Numeric vector. A vector of sample sizes per group (default is seq(10, 100, by = 5)).
#' @param tails Character. `"two"` or `"one"`.
#'
#' @return A ggplot object showing the power curve.
#' @export
#'
#' @examples
#' # Create a power curve for d = 0.5 over a range of sample sizes per group
#' ggpower_ttest(d = 0.5, alpha = 0.05, n_range = seq(10, 100, by = 5))
ggpower_ttest <- function(d, alpha = 0.05, n_range = seq(10, 100, by = 5), tails = "two") {
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' must be installed for plotting.")
  }

  # Compute power for each sample size using the core power function
  power_vals <- sapply(n_range, function(n) {
    power_t_two_sample(d = d, n_per_group = n, alpha = alpha, tails = tails)
  })

  # Create a data frame for plotting
  df_plot <- data.frame(
    n_per_group = n_range,
    power = power_vals
  )

  # Build the ggplot power curve
  p <- ggplot2::ggplot(df_plot, ggplot2::aes(x = n_per_group, y = power)) +
    ggplot2::geom_line(color = "steelblue", linewidth = 1.2) +  # Use 'linewidth' instead of 'size'
    ggplot2::geom_point(color = "darkred", size = 2) +
    ggplot2::labs(
      title = paste("Power Curve for Two-Sample t-Test (d =", d, ")"),
      x = "Sample Size per Group",
      y = "Power (1 - Beta)"
    ) +
    ggplot2::theme_minimal()

  return(p)
}

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(ggpower)

## ----matrix-------------------------------------------------------------------
tests <- ggpower_tests()
knitr::kable(
  tests[, c("id", "module", "domain", "family", "parity", "method", "modes")],
  row.names = FALSE
)

## ----counts-------------------------------------------------------------------
as.data.frame(table(tests$module))

## ----links, echo=FALSE--------------------------------------------------------
vignette_map <- data.frame(
  module = c(
    rep("getting started", 3), rep("workspace", 2), rep("biomarker", 1),
    rep("clinical", 1), rep("reference", 3)
  ),
  article = c(
    "choosing-a-power-analysis", "getting-started-gui", "scenario-guide",
    "t-tests", "workspace-test-families",
    "biomarker-endpoints", "clinical-trials",
    "analysis-modes", "support-matrix", "reference-validation"
  ),
  stringsAsFactors = FALSE
)
knitr::kable(vignette_map, col.names = c("Module area", "Vignette slug"))

mod_help_ui <- function(id) {
  ns <- shiny::NS(id)
  articles <- .gp_vignette_index()

  shiny::div(
    class = "gp-window gp-page gp-page-fit",
    gp_page_header_ui(
      title = "Help",
      subtitle = "Guides and reference articles for every workflow.",
      compact = TRUE
    ),
    shiny::div(
      class = "gp-page-panel gp-help-panel gp-help-fit",
      do.call(
        shiny::tabsetPanel,
        c(
          list(id = ns("section"), type = "pills"),
          lapply(names(articles), function(section) {
            shiny::tabPanel(
              title = section,
              shiny::div(
                class = "gp-help-grid",
                lapply(articles[[section]], function(article) {
                  shiny::div(
                    class = "gp-help-card",
                    shiny::h4(article$title),
                    shiny::p(article$summary),
                    shiny::tags$a(
                      class = "gp-help-link",
                      href = article$url,
                      target = "_blank",
                      shiny::icon("arrow-up-right-from-square"),
                      "Read article"
                    )
                  )
                })
              )
            )
          })
        )
      )
    )
  )
}

mod_help_server <- function(id) {
  shiny::moduleServer(id, function(input, output, session) {
    invisible(NULL)
  })
}

.gp_vignette_index <- function() {
  base <- "https://yaoxiangli.github.io/ggpower/articles"
  list(
    "Getting started" = list(
      list(title = "Choosing a power analysis", summary = "Pick the test family, module, and analysis mode.", url = paste0(base, "/choosing-a-power-analysis.html")),
      list(title = "Getting started with the GUI", summary = "Navigate modules, wide-screen layout, and workflow tips.", url = paste0(base, "/getting-started-gui.html")),
      list(title = "Scenario guide", summary = "Which module and test for your study design.", url = paste0(base, "/scenario-guide.html")),
      list(title = "Reference validation", summary = "Golden examples with expected outputs.", url = paste0(base, "/reference-validation.html"))
    ),
    "Test families" = list(
      list(title = "t Tests", summary = "One-sample, paired, two-sample, point-biserial, slopes.", url = paste0(base, "/t-tests.html")),
      list(title = "Workspace test families", summary = "ANOVA, exact tests, correlations, GLM, nonparametric.", url = paste0(base, "/workspace-test-families.html")),
      list(title = "ANOVA and regression", summary = "F tests, ANOVA, multiple regression, variances.", url = paste0(base, "/anova-regression.html")),
      list(title = "Exact and proportions", summary = "Binomial, Fisher, McNemar, chi-square.", url = paste0(base, "/exact-and-proportions.html")),
      list(title = "Correlation and z tests", summary = "Fisher Z, dependent correlations, GLM.", url = paste0(base, "/correlation-and-z-tests.html")),
      list(title = "Nonparametric tests", summary = "Wilcoxon signed-rank and Mann-Whitney.", url = paste0(base, "/nonparametric-tests.html")),
      list(title = "Logistic and Poisson", summary = "GLM Wald power for binary and count outcomes.", url = paste0(base, "/logistic-poisson.html"))
    ),
    "Biomarker discovery" = list(
      list(title = "Biomarker endpoints", summary = "Overview of all biomarker discovery tests.", url = paste0(base, "/biomarker-endpoints.html")),
      list(title = "Differential expression", summary = "Two-group biomarker mean differences.", url = paste0(base, "/biomarker-differential-expression.html")),
      list(title = "ROC / AUC", summary = "Diagnostic classifier discrimination power.", url = paste0(base, "/biomarker-roc-auc.html")),
      list(title = "Diagnostic accuracy", summary = "Sensitivity and specificity validation.", url = paste0(base, "/biomarker-diagnostic-accuracy.html")),
      list(title = "Survival biomarkers", summary = "Log-rank and time-to-event stratification.", url = paste0(base, "/biomarker-survival.html")),
      list(title = "Cox prognostic models", summary = "Hazard ratio power for scored biomarkers.", url = paste0(base, "/biomarker-cox-prognostic.html")),
      list(title = "Multiplicity and FDR", summary = "Omics screening under FDR control.", url = paste0(base, "/biomarker-multiplicity-fdr.html"))
    ),
    "Clinical trials" = list(
      list(title = "Clinical trials", summary = "Overview of superiority, NI, equivalence, and more.", url = paste0(base, "/clinical-trials.html")),
      list(title = "Phase III superiority", summary = "Continuous and binary superiority trials.", url = paste0(base, "/pharma-phase-iii-superiority.html")),
      list(title = "Non-inferiority", summary = "NI margins for means and proportions.", url = paste0(base, "/pharma-non-inferiority.html")),
      list(title = "Equivalence (TOST)", summary = "Bioequivalence and similarity trials.", url = paste0(base, "/pharma-equivalence-tost.html")),
      list(title = "Simon two-stage", summary = "Oncology Phase II single-arm designs.", url = paste0(base, "/pharma-simon-two-stage.html")),
      list(title = "Cluster RCT", summary = "Design effect from intraclass correlation.", url = paste0(base, "/pharma-cluster-rct.html")),
      list(title = "Survival endpoints", summary = "OS, PFS, and event-driven sample size.", url = paste0(base, "/pharma-survival-endpoints.html")),
      list(title = "Binary and count endpoints", summary = "Response rates and Poisson counts.", url = paste0(base, "/pharma-binary-and-count-endpoints.html"))
    ),
    "Reference" = list(
      list(title = "Formula reference", summary = "All power formulas in one place.", url = paste0(base, "/formula-reference.html")),
      list(title = "Support matrix", summary = "Test x mode x domain coverage table.", url = paste0(base, "/support-matrix.html")),
      list(title = "Analysis modes", summary = "Worked examples for all five modes.", url = paste0(base, "/analysis-modes.html")),
      list(title = "Effect size conversions", summary = "Helper functions with formulas.", url = paste0(base, "/effect-size-conversions.html")),
      list(title = "Calculator", summary = "Distribution-function scripts.", url = paste0(base, "/calculator.html")),
      list(title = "Approximation catalog", summary = "Parity limits and method notes.", url = paste0(base, "/approximation-catalog.html")),
      list(title = "Publication figures", summary = "Plotting and export helpers.", url = paste0(base, "/publication-figures.html"))
    )
  )
}

#' The application server-side
#'
#' @param input,output,session Internal parameters for {shiny}.
#'     DO NOT REMOVE.
#' @import shiny
#' @noRd
app_server <- function(input, output, session) {
  protocol <- shiny::reactiveVal(character())

  mod_power_workspace_server("power_workspace", protocol = protocol)
  mod_biomarker_server("biomarker", protocol = protocol)
  mod_clinical_server("clinical", protocol = protocol)
  mod_calculator_server("calculator")
  mod_protocol_server("protocol", protocol = protocol)
  mod_help_server("help")
}

mod_protocol_ui <- function(id) {
  ns <- shiny::NS(id)
  shiny::div(
    class = "gp-window gp-page gp-page-fit",
    gp_page_header_ui(
      title = "Protocol",
      subtitle = "Session log of every power analysis run in this workspace.",
      compact = TRUE
    ),
    shiny::div(
      class = "gp-page-panel gp-protocol-panel gp-protocol-scroll",
      shiny::div(
        class = "gp-panel-toolbar",
        shiny::h4("Session protocol"),
        shiny::downloadButton(ns("download_protocol"), "Download", class = "btn-primary btn-sm")
      ),
      shiny::div(class = "gp-protocol-body", shiny::uiOutput(ns("protocol_cards")))
    )
  )
}

mod_protocol_server <- function(id, protocol) {
  shiny::moduleServer(id, function(input, output, session) {
    output$protocol_cards <- shiny::renderUI({
      entries <- protocol()
      if (length(entries) == 0) {
        return(shiny::div(class = "gp-empty-state", "No calculations yet. Run an analysis from any module."))
      }
      shiny::tagList(lapply(seq_along(entries), function(i) {
        shiny::div(
          class = "gp-protocol-card",
          shiny::div(class = "gp-protocol-card-title", paste("Analysis", i)),
          shiny::tags$pre(class = "gp-protocol-card-body", entries[[i]])
        )
      }))
    })

    output$download_protocol <- shiny::downloadHandler(
      filename = function() paste0("ggpower-protocol-", Sys.Date(), ".txt"),
      content = function(file) {
        writeLines(paste(protocol(), collapse = "\n\n---\n\n"), file)
      }
    )
  })
}

.gp_rct_superiority_continuous <- function(params, analysis) {
  params$tails <- "one"
  result <- .gp_t_two_sample(params, analysis)
  result$test <- "Clinical: RCT superiority (continuous endpoint)"
  result
}

.gp_rct_superiority_binary <- function(params, analysis) {
  params$tails <- "one"
  result <- .gp_exact_fisher(params, analysis)
  result$test <- "Clinical: RCT superiority (binary endpoint)"
  result
}

.gp_rct_noninferiority_continuous <- function(params, analysis) {
  d <- .gp_numeric(params, "d", 0)
  margin <- .gp_numeric(params, "ni_margin", 0.2, min = 0, strict_min = TRUE)
  n1 <- .gp_count(params, "n1", 30, min = 2)
  n2 <- .gp_count(params, "n2", .gp_arg(params, "n_per_group", n1), min = 2)
  alpha <- .gp_alpha(params)
  df <- n1 + n2 - 2
  ncp <- (d + margin) * sqrt(n1 * n2 / (n1 + n2))
  pwr <- .gp_power_t(df, ncp, alpha, "one")

  .gp_result(
    "Clinical: Non-inferiority trial (continuous)", analysis,
    list(
      tails = "one", effect_size_d = d, ni_margin = margin,
      sample_size_group_1 = n1, sample_size_group_2 = n2, alpha = alpha
    ),
    list(
      noncentrality_parameter = ncp, critical_t = pwr$critical, df = df,
      total_sample_size = n1 + n2, power = pwr$power
    ),
    notes = "One-sided NI test: H0 difference <= -margin vs H1 difference > -margin.",
    distribution = list(type = "t", df = df, ncp = ncp)
  )
}

.gp_rct_noninferiority_binary <- function(params, analysis) {
  p_treat <- .gp_numeric(params, "p1", 0.6, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  p_control <- .gp_numeric(params, "p0", 0.5, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  margin <- .gp_numeric(params, "ni_margin", 0.1, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  n1 <- .gp_count(params, "n1", 100, min = 2)
  n2 <- .gp_count(params, "n2", .gp_arg(params, "n_per_group", n1), min = 2)
  alpha <- .gp_alpha(params)

  p_eff <- p_treat - p_control + margin
  se <- sqrt(p_control * (1 - p_control) / n2 + p_treat * (1 - p_treat) / n1)
  z_ncp <- p_eff / se
  pwr <- .gp_power_z(z_ncp, alpha, "one")

  .gp_result(
    "Clinical: Non-inferiority trial (binary)", analysis,
    list(
      tails = "one", p_treatment = p_treat, p_control = p_control,
      ni_margin = margin, sample_size_group_1 = n1, sample_size_group_2 = n2, alpha = alpha
    ),
    list(
      z_statistic = z_ncp, total_sample_size = n1 + n2, power = pwr$power
    ),
    notes = "Normal approximation for NI on proportions (one-sided).",
    distribution = list(type = "z", mean = z_ncp)
  )
}

.gp_rct_equivalence_continuous <- function(params, analysis) {
  d <- .gp_numeric(params, "d", 0)
  margin <- .gp_numeric(params, "eq_margin", 0.2, min = 0, strict_min = TRUE)
  n1 <- .gp_count(params, "n1", 30, min = 2)
  n2 <- .gp_count(params, "n2", .gp_arg(params, "n_per_group", n1), min = 2)
  alpha <- .gp_alpha(params)
  df <- n1 + n2 - 2
  se_factor <- sqrt(n1 * n2 / (n1 + n2))
  ncp_upper <- (margin - d) * se_factor
  ncp_lower <- (margin + d) * se_factor
  pwr_upper <- .gp_power_t(df, ncp_upper, alpha, "one")
  pwr_lower <- .gp_power_t(df, ncp_lower, alpha, "one")
  power <- pwr_upper$power * pwr_lower$power

  .gp_result(
    "Clinical: Equivalence trial (continuous, TOST)", analysis,
    list(
      effect_size_d = d, eq_margin = margin,
      sample_size_group_1 = n1, sample_size_group_2 = n2, alpha = alpha
    ),
    list(
      power_upper = pwr_upper$power, power_lower = pwr_lower$power,
      total_sample_size = n1 + n2, power = power
    ),
    notes = "Two one-sided t tests (TOST); overall power is the product of both one-sided powers.",
    distribution = list(type = "t", df = df, ncp = abs(d) * se_factor)
  )
}

.gp_rct_equivalence_proportion <- function(params, analysis) {
  p1 <- .gp_numeric(params, "p1", 0.55, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  p0 <- .gp_numeric(params, "p0", 0.5, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  margin <- .gp_numeric(params, "eq_margin", 0.1, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  n1 <- .gp_count(params, "n1", 100, min = 2)
  n2 <- .gp_count(params, "n2", .gp_arg(params, "n_per_group", n1), min = 2)
  alpha <- .gp_alpha(params)

  diff <- p1 - p0
  se <- sqrt(p1 * (1 - p1) / n1 + p0 * (1 - p0) / n2)
  z_upper <- (margin - diff) / se
  z_lower <- (margin + diff) / se
  pwr_upper <- .gp_power_z(z_upper, alpha, "one")
  pwr_lower <- .gp_power_z(z_lower, alpha, "one")
  power <- pwr_upper$power * pwr_lower$power

  .gp_result(
    "Clinical: Equivalence trial (proportions, TOST)", analysis,
    list(
      p_treatment = p1, p_control = p0, eq_margin = margin,
      sample_size_group_1 = n1, sample_size_group_2 = n2, alpha = alpha
    ),
    list(
      power_upper = pwr_upper$power, power_lower = pwr_lower$power,
      total_sample_size = n1 + n2, power = power
    ),
    notes = "TOST on proportion difference using normal approximation.",
    distribution = list(type = "z", mean = diff / se)
  )
}

.gp_simon_two_stage <- function(params, analysis) {
  p0 <- .gp_numeric(params, "p0", 0.2, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  p1 <- .gp_numeric(params, "p1", 0.35, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  alpha <- .gp_alpha(params)
  beta <- 1 - .gp_power_target(params)
  n1 <- .gp_count(params, "n1", 15, min = 2)
  r1 <- .gp_count(params, "r1", 4, min = 0)
  n2 <- .gp_count(params, "n2", 20, min = 0)
  r <- .gp_count(params, "r", 10, min = 0)

  prob_stage1 <- stats::pbinom(r1, n1, p1, lower.tail = FALSE)
  prob_stage2 <- stats::pbinom(r, n1 + n2, p1, lower.tail = FALSE)
  prob_null <- stats::pbinom(r1, n1, p0, lower.tail = FALSE) *
    stats::pbinom(r, n1 + n2, p0, lower.tail = FALSE)
  power <- prob_stage1 * prob_stage2

  .gp_result(
    "Clinical: Simon two-stage Phase II design", analysis,
    list(
      p0 = p0, p1 = p1, alpha = alpha, target_power = 1 - beta,
      stage1_n = n1, stage1_r = r1, stage2_n = n2, total_r = r
    ),
    list(
      power = power, type_i_error = prob_null,
      total_sample_size = n1 + n2, expected_sample_size = n1 + n2 * prob_stage1
    ),
    notes = "Simon optimal/minimax design power for specified (n1,r1,n2,r).",
    distribution = list(type = "binomial", n = n1 + n2, p = p1)
  )
}

.gp_cluster_rct <- function(params, analysis) {
  d <- .gp_numeric(params, "d", 0.5)
  icc <- .gp_numeric(params, "icc", 0.05, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  cluster_size <- .gp_count(params, "cluster_size", 10, min = 2)
  n_clusters <- .gp_count(params, "n_clusters", .gp_arg(params, "total_n", 20), min = 2)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)

  deff <- 1 + (cluster_size - 1) * icc
  n_eff <- n_clusters * cluster_size / deff
  df <- max(2, 2 * n_clusters - 2)
  ncp <- d * sqrt(n_eff / 2)
  pwr <- .gp_power_t(df, ncp, alpha, tails)

  .gp_result(
    "Clinical: Cluster-randomized trial", analysis,
    list(
      tails = pwr$direction, effect_size_d = d, icc = icc,
      cluster_size = cluster_size, n_clusters_per_arm = n_clusters, alpha = alpha
    ),
    list(
      design_effect = deff, effective_n_per_arm = n_eff,
      noncentrality_parameter = ncp, power = pwr$power,
      total_sample_size = 2 * n_clusters * cluster_size
    ),
    notes = "Design effect DE = 1 + (m-1)*ICC applied to two-arm cluster RCT.",
    distribution = list(type = "t", df = df, ncp = ncp)
  )
}

.gp_multi_arm_superiority <- function(params, analysis) {
  result <- .gp_f_anova_one_way(params, analysis)
  result$test <- "Clinical: Multi-arm superiority (ANOVA)"
  result$notes <- c(result$notes, "Consider Dunnett adjustment for pairwise comparisons.")
  result
}

.gp_count_endpoint_poisson <- function(params, analysis) {
  result <- .gp_z_poisson(params, analysis)
  result$test <- "Clinical: Count endpoint (Poisson regression)"
  result
}

.gp_survival_pmu <- function(params, analysis) {
  result <- .gp_survival_logrank(params, analysis)
  result$test <- "Clinical: Survival endpoint (log-rank / Cox framework)"
  result
}

#' Publication-ready ggpower theme
#'
#' @param base_size Base font size.
#' @param base_family Base font family.
#'
#' @return A ggplot2 theme.
#' @export
theme_ggpower <- function(base_size = 12, base_family = "") {
  ggplot2::theme_minimal(base_size = base_size, base_family = base_family) +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold", margin = ggplot2::margin(b = 8)),
      plot.subtitle = ggplot2::element_text(color = "#555555", margin = ggplot2::margin(b = 8)),
      panel.grid.minor = ggplot2::element_blank(),
      panel.grid.major = ggplot2::element_line(color = "#e5e5e5", linewidth = 0.35),
      axis.title = ggplot2::element_text(face = "bold"),
      legend.position = "bottom",
      legend.title = ggplot2::element_blank()
    )
}

mod_power_workspace_ui <- function(id) {
  n_tests <- nrow(ggpower_tests(module = "workspace"))
  mod_power_core_ui(
    id,
    list(
      module_filter = "workspace",
      title = "Power analysis workspace",
      subtitle = "Choose a test family, set inputs, and calculate sample size, power, alpha, or effect size.",
      badges = list(
        list(label = paste0(n_tests, " statistical tests")),
        list(label = "5 analysis modes"),
        list(label = "Publication plots", class = "gp-badge gp-badge-accent")
      )
    )
  )
}

mod_power_workspace_server <- function(id, protocol = NULL) {
  mod_power_core_server(
    id,
    list(module_filter = "workspace"),
    protocol = protocol
  )
}

.gp_analysis_choice_map <- function() {
  c(
    "A priori: Compute required sample size" = "a_priori",
    "Compromise: Compute alpha and beta" = "compromise",
    "Criterion: Compute alpha" = "criterion",
    "Post hoc: Compute achieved power" = "post_hoc",
    "Sensitivity: Compute effect size" = "sensitivity"
  )
}

.gp_analysis_choices_for_modes <- function(modes) {
  all <- .gp_analysis_choice_map()
  all[all %in% modes]
}

.gp_fields_for_mode <- function(spec, mode) {
  fields <- spec$fields
  remove <- switch(
    mode,
    a_priori = spec$sample,
    post_hoc = "power",
    criterion = "alpha",
    sensitivity = spec$effect,
    compromise = c("alpha", "power"),
    character()
  )
  unique(fields[!fields %in% remove])
}

.gp_input_control <- function(ns, field) {
  labels <- .gp_field_labels()
  defaults <- .gp_field_defaults()

  if (field == "tails") {
    return(shiny::div(
      class = "gp-field gp-field-compact",
      shiny::selectInput(
        ns(field),
        labels[[field]],
        choices = c("One" = "one", "Two" = "two"),
        selected = defaults[[field]],
        width = "100%"
      )
    ))
  }

  shiny::div(
    class = "gp-field gp-field-compact",
    shiny::numericInput(
      ns(field),
      labels[[field]] %||% field,
      value = defaults[[field]] %||% 1,
      min = .gp_field_min()[[field]] %||% NA,
      step = .gp_field_step()[[field]] %||% NA,
      width = "100%"
    )
  )
}

.gp_collect_inputs <- function(input, test) {
  spec <- ggpower_get_test(test)
  defaults <- .gp_field_defaults()
  params <- list()
  for (field in spec$fields) {
    value <- input[[field]]
    if (is.null(value)) {
      value <- defaults[[field]]
    }
    params[[field]] <- value
  }
  params
}

.gp_make_xy_data <- function(test, analysis, params) {
  current_n <- .gp_current_total_n(test, params)
  start <- max(.gp_min_total_n(test, params), floor(current_n * 0.4))
  stop <- max(start + 10, ceiling(current_n * 1.8))
  values <- unique(ceiling(seq(start, stop, length.out = 30)))

  rows <- lapply(values, function(n_total) {
    trial <- .gp_set_total_n(test, params, n_total)
    trial$power <- NULL
    trial$q <- NULL
    result <- tryCatch(
      do.call(power_compute, c(list(test = test, analysis = "post_hoc"), trial)),
      error = function(e) NULL
    )
    if (is.null(result)) {
      return(NULL)
    }
    data.frame(
      total_n = .gp_current_total_n(test, trial),
      power = .gp_output_power(result)
    )
  })

  do.call(rbind, rows)
}

.gp_current_total_n <- function(test, params) {
  two_group <- c(
    "t_two_sample", "t_linear_regression_two_groups", "f_variance_two", "exact_fisher",
    "z_corr_independent", "wilcoxon_mann_whitney", "roc_auc_two", "ttest_biomarker",
    "rct_superiority_continuous", "rct_superiority_binary", "rct_noninferiority_continuous",
    "rct_noninferiority_binary", "rct_equivalence_continuous", "rct_equivalence_proportion"
  )
  if (test %in% two_group) {
    return((params$n1 %||% params$n_per_group %||% 30) + (params$n2 %||% params$n_per_group %||% 30))
  }
  if (test %in% c("roc_auc_one", "diagnostic_acc")) {
    return((params$n_pos %||% 30) + (params$n_neg %||% 30))
  }
  if (test == "cluster_rct") {
    cs <- params$cluster_size %||% 10
    nc <- params$n_clusters %||% 20
    return(2 * nc * cs)
  }
  params$total_n %||% params$n %||% params$events %||% 80
}

.gp_empty_plot <- function(message) {
  ggplot2::ggplot() +
    ggplot2::annotate("text", x = 0, y = 0, label = message) +
    ggplot2::theme_void()
}

.gp_effect_modal <- function(test, ns) {
  spec <- ggpower_get_test(test)
  shiny::modalDialog(
    title = paste("Determine effect size for", spec$label),
    shiny::p("Use the exported effect-size helpers in scripts or copy the formula below into the relevant field."),
    shiny::tags$ul(
      shiny::tags$li("d = (mean H1 - mean H0) / SD"),
      shiny::tags$li("f = sqrt(eta2 / (1 - eta2))"),
      shiny::tags$li("f2 = R2 / (1 - R2)"),
      shiny::tags$li("w = sqrt(sum((p1 - p0)^2 / p0))"),
      shiny::tags$li("q = FisherZ(r1) - FisherZ(r2)"),
      shiny::tags$li("AUC: Hanley-McNeil variance for ROC curves"),
      shiny::tags$li("HR: log hazard ratio for survival/Cox models")
    ),
    easyClose = TRUE,
    footer = shiny::modalButton("Close")
  )
}

.gp_field_labels <- function() {
  list(
    tails = "Tail(s)",
    d = "Effect size d / dz",
    f = "Effect size f",
    f2 = "Effect size f2",
    w = "Effect size w",
    h = "Effect size h",
    rho = "Correlation rho H1",
    rho0 = "Correlation rho H0",
    rho_ab = "H0 corr rho_ab",
    rho_ac = "H1 corr rho_ac",
    rho_cd = "H1 corr rho_cd",
    rho_bc = "Corr rho_bc",
    rho_ad = "Corr rho_ad",
    rho_bd = "Corr rho_bd",
    rho2 = "Population rho2",
    q_effect = "Effect size q",
    slope_h1 = "Slope H1",
    slope_h0 = "Slope H0",
    delta_slope = "Absolute slope difference",
    sd_x = "Std dev sigma_x",
    sd_y = "Std dev sigma_y",
    sd_x1 = "Std dev sigma_x1",
    sd_x2 = "Std dev sigma_x2",
    residual_sd = "Std dev residual sigma",
    ncp = "Noncentrality parameter",
    alpha = "Alpha err prob",
    power = "Power (1 - beta err prob)",
    q = "Beta/alpha ratio",
    n = "Total sample size",
    total_n = "Total sample size",
    n1 = "Sample size group 1",
    n2 = "Sample size group 2",
    n_pos = "N diseased / positive",
    n_neg = "N non-diseased / negative",
    allocation_ratio = "Allocation ratio N2/N1",
    df = "Degrees of freedom",
    df1 = "Numerator df",
    groups = "Number of groups",
    predictors = "Number of predictors",
    tested_predictors = "Number of tested predictors",
    variance_ratio = "Ratio var1/var0",
    p0 = "Probability H0",
    p1 = "Probability H1",
    odds_ratio = "Odds ratio",
    exp_beta1 = "Exp(beta1)",
    base_rate = "Base rate exp(beta0)",
    exposure = "Mean exposure",
    r2_other = "R2 other X",
    x_variance = "X variance",
    are = "Asymptotic relative efficiency",
    auc = "AUC H1",
    auc0 = "Null AUC H0",
    auc1 = "AUC group 1",
    auc2 = "AUC group 2",
    sensitivity = "Sensitivity H1",
    specificity = "Specificity H1",
    hazard_ratio = "Hazard ratio",
    event_rate = "Overall event rate",
    events = "Expected events",
    m_tests = "Number of tests",
    pi0 = "Proportion true null",
    effect_d = "Effect size d per test",
    fdr_level = "Target FDR level",
    ni_margin = "Non-inferiority margin",
    eq_margin = "Equivalence margin",
    icc = "Intraclass correlation",
    cluster_size = "Cluster size (m)",
    n_clusters = "Clusters per arm",
    r1 = "Stage 1 threshold (r1)",
    r = "Total threshold (r)"
  )
}

.gp_field_defaults <- function() {
  list(
    tails = "two",
    d = 0.5,
    f = 0.25,
    f2 = 0.15,
    w = 0.3,
    h = 0.3,
    rho = 0.3,
    rho0 = 0,
    rho_ab = 0.4,
    rho_ac = 0.2,
    rho_cd = 0.2,
    rho_bc = 0.5,
    rho_ad = 0.4,
    rho_bd = 0.8,
    rho2 = 0.1,
    q_effect = 0.3,
    slope_h1 = -0.0667,
    slope_h0 = 0,
    delta_slope = 0.1,
    sd_x = 1,
    sd_y = 1,
    sd_x1 = 1,
    sd_x2 = 1,
    residual_sd = 1,
    ncp = 1,
    alpha = 0.05,
    power = 0.8,
    q = 1,
    n = 40,
    total_n = 100,
    n1 = 30,
    n2 = 30,
    n_pos = 50,
    n_neg = 50,
    allocation_ratio = 1,
    df = 20,
    df1 = 1,
    groups = 3,
    predictors = 1,
    tested_predictors = 1,
    variance_ratio = 1.5,
    p0 = 0.5,
    p1 = 0.65,
    odds_ratio = 1.5,
    exp_beta1 = 1.3,
    base_rate = 0.85,
    exposure = 1,
    r2_other = 0,
    x_variance = 1,
    are = 3 / pi,
    auc = 0.75,
    auc0 = 0.5,
    auc1 = 0.75,
    auc2 = 0.65,
    sensitivity = 0.85,
    specificity = 0.85,
    hazard_ratio = 0.65,
    event_rate = 0.5,
    events = 100,
    m_tests = 1000,
    pi0 = 0.9,
    effect_d = 0.5,
    fdr_level = 0.05,
    ni_margin = 0.2,
    eq_margin = 0.2,
    icc = 0.05,
    cluster_size = 10,
    n_clusters = 20,
    r1 = 4,
    r = 10
  )
}

.gp_field_min <- function() {
  list(
    alpha = 0, power = 0, q = 0, n = 1, total_n = 1, n1 = 1, n2 = 1, n_pos = 1, n_neg = 1,
    allocation_ratio = 0, df = 1, df1 = 1, groups = 2, predictors = 1, tested_predictors = 1,
    p0 = 0, p1 = 0, odds_ratio = 0, exp_beta1 = 0, base_rate = 0, exposure = 0, r2_other = 0,
    x_variance = 0, are = 0, sd_x = 0, sd_y = 0, sd_x1 = 0, sd_x2 = 0, residual_sd = 0, rho2 = 0,
    auc = 0, auc0 = 0, auc1 = 0, auc2 = 0, sensitivity = 0, specificity = 0,
    hazard_ratio = 0, event_rate = 0, events = 1, m_tests = 2, pi0 = 0, effect_d = 0,
    fdr_level = 0, ni_margin = 0, eq_margin = 0, icc = 0, cluster_size = 2, n_clusters = 2,
    r1 = 0, r = 0
  )
}

.gp_field_step <- function() {
  list(
    alpha = 0.01, power = 0.01, q = 0.1, d = 0.1, f = 0.05, f2 = 0.05, w = 0.05, rho = 0.05,
    q_effect = 0.05, n = 1, total_n = 1, n1 = 1, n2 = 1, allocation_ratio = 0.1,
    auc = 0.01, hazard_ratio = 0.05, ni_margin = 0.05, eq_margin = 0.05, icc = 0.01
  )
}

test_that("power workspace ui renders", {
  ui <- mod_power_workspace_ui("workspace")
  expect_s3_class(ui, "shiny.tag.list")
})

test_that("power workspace server calculates a result", {
  testthat::skip_if_not_installed("shiny")

  protocol <- shiny::reactiveVal(character())
  shiny::testServer(
    mod_power_workspace_server,
    args = list(id = "workspace", protocol = protocol),
    {
      session$setInputs(
        family = "t tests",
        test = "t_one_sample",
        analysis = "post_hoc",
        tails = "two",
        d = 0.5,
        alpha = 0.05,
        n = 40
      )
      session$setInputs(calculate = 1)

      expect_length(protocol(), 1L)
      expect_match(protocol()[[1]], "one sample case")
    }
  )
})

#' The application User-Interface
#'
#' @param request Internal parameter for `{shiny}`.
#'     DO NOT REMOVE.
#' @importFrom shiny tagList icon tags
#' @importFrom bs4Dash bs4DashPage bs4DashNavbar bs4DashSidebar bs4DashBody bs4TabItems bs4TabItem bs4DashControlbar bs4DashFooter sidebarMenu menuItem
#' @noRd
app_ui <- function(request) {
  tagList(
    golem_add_external_resources(),
    bs4Dash::bs4DashPage(
      help = FALSE,
      dark = FALSE,
      fullscreen = FALSE,
      scrollToTop = FALSE,
      header = bs4Dash::bs4DashNavbar(
        title = gp_brand_ui(grad_id = "gpLogoNav"),
        skin = "light",
        status = "white",
        border = FALSE
      ),
      sidebar = bs4Dash::bs4DashSidebar(
        width = "240px",
        skin = "dark",
        status = "dark",
        elevation = 0,
        collapsed = FALSE,
        minified = FALSE,
        expandOnHover = FALSE,
        fixed = TRUE,
        title = gp_brand_ui(compact = TRUE, grad_id = "gpLogoSide"),
        sidebarMenu(
          id = "sidebar_menu",
          menuItem("Power Workspace", tabName = "workspace", icon = icon("chart-line")),
          menuItem("Biomarker Discovery", tabName = "biomarker", icon = icon("dna")),
          menuItem("Clinical Trials", tabName = "clinical", icon = icon("pills")),
          menuItem("Calculator", tabName = "calculator", icon = icon("calculator")),
          menuItem("Protocol", tabName = "protocol", icon = icon("clipboard-list")),
          menuItem("Help", tabName = "help", icon = icon("book-open"))
        )
      ),
      body = bs4Dash::bs4DashBody(
        bs4Dash::bs4TabItems(
          bs4Dash::bs4TabItem(tabName = "workspace", mod_power_workspace_ui("power_workspace")),
          bs4Dash::bs4TabItem(tabName = "biomarker", mod_biomarker_ui("biomarker")),
          bs4Dash::bs4TabItem(tabName = "clinical", mod_clinical_ui("clinical")),
          bs4Dash::bs4TabItem(tabName = "calculator", mod_calculator_ui("calculator")),
          bs4Dash::bs4TabItem(tabName = "protocol", mod_protocol_ui("protocol")),
          bs4Dash::bs4TabItem(tabName = "help", mod_help_ui("help"))
        )
      ),
      controlbar = bs4Dash::bs4DashControlbar(disable = TRUE),
      footer = bs4Dash::bs4DashFooter(
        left = tags$span(class = "gp-footer-left", gp_brand_ui(compact = TRUE, grad_id = "gpLogoFoot")),
        right = tags$span(
          class = "gp-footer-right",
          tags$a(href = "https://yaoxiangli.github.io/ggpower/", target = "_blank", "Documentation")
        )
      )
    )
  )
}

#' Add external Resources to the Application
#'
#' This function is internally used to add external
#' resources inside the Shiny application.
#'
#' @import shiny
#' @importFrom golem add_resource_path activate_js favicon bundle_resources
#' @noRd
golem_add_external_resources <- function() {
  add_resource_path(
    "www",
    app_sys("app/www")
  )

  tags$head(
    favicon(),
    bundle_resources(
      path = app_sys("app/www"),
      app_title = "ggpower"
    ),
    tags$link(rel = "preconnect", href = "https://fonts.googleapis.com"),
    tags$link(rel = "preconnect", href = "https://fonts.gstatic.com", crossorigin = "anonymous"),
    tags$link(
      rel = "stylesheet",
      href = "https://fonts.googleapis.com/css2?family=Inter:wght@400;500;600;700;800&display=swap"
    ),
    tags$link(rel = "stylesheet", type = "text/css", href = "www/custom.css"),
    tags$script(src = "www/layout.js")
  )
}

#' Create a ggpower result object
#'
#' @param test,analysis Character labels for the selected test and analysis.
#' @param inputs Named list of input parameters.
#' @param outputs Named list of computed output values.
#' @param notes Character vector with assumptions, warnings, or method notes.
#' @param distribution Named list describing the H0/H1 distributions.
#'
#' @return An object of class `ggpower_result`.
#' @export
ggpower_result <- function(test, analysis, inputs, outputs,
                           notes = character(), distribution = list()) {
  structure(
    list(
      test = test,
      analysis = analysis,
      inputs = inputs,
      outputs = outputs,
      notes = notes,
      distribution = distribution
    ),
    class = "ggpower_result"
  )
}

#' @export
print.ggpower_result <- function(x, ...) {
  cat("ggpower result\n")
  cat("Test: ", x$test, "\n", sep = "")
  cat("Analysis: ", x$analysis, "\n\n", sep = "")

  if (length(x$inputs) > 0) {
    cat("Input parameters\n")
    cat(.format_named_values(x$inputs), sep = "\n")
    cat("\n\n")
  }

  if (length(x$outputs) > 0) {
    cat("Output parameters\n")
    cat(.format_named_values(x$outputs), sep = "\n")
    cat("\n")
  }

  if (length(x$notes) > 0) {
    cat("\nNotes\n")
    cat(paste0("- ", x$notes), sep = "\n")
    cat("\n")
  }

  invisible(x)
}

format.ggpower_result <- function(x, ...) {
  capture.output(print(x))
}

.format_named_values <- function(values) {
  vapply(names(values), function(name) {
    value <- values[[name]]

    if (length(value) > 1) {
      value <- paste(format(value, digits = 7), collapse = ", ")
    } else if (is.numeric(value)) {
      value <- format(value, digits = 7)
    } else {
      value <- as.character(value)
    }

    paste0("  ", name, ": ", value)
  }, character(1))
}

#' Format a ggpower result as structured HTML for Shiny UI
#'
#' @param x A `ggpower_result` object.
#' @return A `shiny.tag` list for `renderUI`.
#' @export
format_result_html <- function(x) {
  power_val <- x$outputs$power %||% x$outputs$actual_power
  metrics <- list()
  if (!is.null(power_val)) {
    metrics <- c(metrics, list(list(label = "Power", value = format(power_val, digits = 4), accent = TRUE)))
  }
  if (!is.null(x$outputs$total_sample_size)) {
    metrics <- c(metrics, list(list(label = "Total N", value = as.character(x$outputs$total_sample_size))))
  }
  if (!is.null(x$outputs$alpha)) {
    metrics <- c(metrics, list(list(label = "Alpha", value = format(x$outputs$alpha, digits = 4))))
  }
  if (!is.null(x$outputs$beta)) {
    metrics <- c(metrics, list(list(label = "Beta", value = format(x$outputs$beta, digits = 4))))
  }

  metric_cards <- if (length(metrics) > 0) {
    shiny::div(
      class = "gp-metric-row",
      lapply(metrics, function(m) {
        shiny::div(
          class = paste("gp-metric-card", if (isTRUE(m$accent)) "gp-metric-accent"),
          shiny::div(class = "gp-metric-label", m$label),
          shiny::div(class = "gp-metric-value", m$value)
        )
      })
    )
  }

  shiny::div(
    class = "gp-result-panel",
    shiny::div(class = "gp-result-title", x$test),
    shiny::div(class = "gp-result-meta", paste("Analysis:", x$analysis)),
    metric_cards,
    if (length(x$inputs) > 0) {
      shiny::div(
        class = "gp-result-block",
        shiny::div(class = "gp-result-block-title", "Inputs"),
        shiny::tags$pre(class = "gp-result-pre", paste(.format_named_values(x$inputs), collapse = "\n"))
      )
    },
    if (length(x$outputs) > 0) {
      shiny::div(
        class = "gp-result-block",
        shiny::div(class = "gp-result-block-title", "Outputs"),
        shiny::tags$pre(class = "gp-result-pre", paste(.format_named_values(x$outputs), collapse = "\n"))
      )
    },
    if (length(x$notes) > 0) {
      shiny::div(
        class = "gp-result-notes",
        shiny::tags$ul(lapply(x$notes, shiny::tags$li))
      )
    }
  )
}

.gp_result <- function(test, analysis, inputs, outputs,
                       notes = character(), distribution = list()) {
  ggpower_result(
    test = test,
    analysis = analysis,
    inputs = inputs,
    outputs = outputs,
    notes = notes,
    distribution = distribution
  )
}

test_that("all domain kernels run post_hoc without error", {
  biomarker <- c(
    "roc_auc_one", "roc_auc_two", "diagnostic_acc", "survival_logrank",
    "cox_regression", "discovery_fdr", "ttest_biomarker"
  )
  clinical <- c(
    "rct_superiority_continuous", "rct_superiority_binary",
    "rct_noninferiority_continuous", "rct_noninferiority_binary",
    "rct_equivalence_continuous", "rct_equivalence_proportion",
    "simon_two_stage", "cluster_rct", "multi_arm_superiority",
    "count_endpoint_poisson", "survival_pmu"
  )

  for (test in c(biomarker, clinical)) {
    result <- power_compute(test, analysis = "post_hoc")
    expect_s3_class(result, "ggpower_result")
    expect_true(!is.null(result$outputs$power) || !is.null(result$outputs$actual_power))
  }
})

test_that("format_result_html returns shiny tag", {
  result <- power_compute("t_one_sample", analysis = "post_hoc", d = 0.5, n = 40)
  html <- format_result_html(result)
  expect_s3_class(html, "shiny.tag")
})

test_that("layout script is bundled with the app", {
  path <- system.file("app/www/layout.js", package = "ggpower")
  expect_true(file.exists(path))
  expect_true(grepl("gp-layout-hd", readLines(path, warn = FALSE)[[1]], fixed = TRUE) ||
    any(grepl("gp-layout-hd", readLines(path, warn = FALSE))))
})

#' Plot a power curve
#'
#' @param test Character test id.
#' @param n_values Numeric vector of total sample sizes to evaluate.
#' @param analysis Power analysis mode used for fixed parameters.
#' @param ... Test-specific fixed parameters.
#'
#' @return A ggplot object.
#' @export
plot_power_curve <- function(test, n_values, analysis = "post_hoc", ...) {
  params <- list(...)
  dat <- do.call(rbind, lapply(n_values, function(n_total) {
    trial <- .gp_set_total_n(test, params, n_total)
    result <- do.call(power_compute, c(list(test = test, analysis = "post_hoc"), trial))
    data.frame(total_n = .gp_current_total_n_plot(test, trial), power = .gp_output_power(result))
  }))

  ggplot2::ggplot(dat, ggplot2::aes(x = total_n, y = power)) +
    ggplot2::geom_line(color = "#2468a2", linewidth = 1.1) +
    ggplot2::geom_point(color = "#8f2d1c", size = 2) +
    ggplot2::scale_y_continuous(limits = c(0, 1)) +
    ggplot2::labs(
      title = "Power curve",
      subtitle = ggpower_get_test(test)$label,
      x = "Total sample size",
      y = "Power (1 - beta)"
    ) +
    theme_ggpower()
}

#' Plot H0 and H1 distributions for a result
#'
#' @param result A `ggpower_result` object.
#'
#' @return A ggplot object.
#' @export
plot_distribution <- function(result) {
  .gp_distribution_plot_core(result) +
    theme_ggpower()
}

#' Save a ggpower plot
#'
#' @param plot A ggplot object.
#' @param filename Output filename.
#' @param width,height Plot dimensions.
#' @param dpi Resolution for raster outputs.
#'
#' @return The filename invisibly.
#' @export
save_power_plot <- function(plot, filename, width = 7, height = 5, dpi = 320) {
  ggplot2::ggsave(filename, plot = plot, width = width, height = height, dpi = dpi)
  invisible(filename)
}

#' @rdname save_power_plot
#' @export
save_distribution_plot <- save_power_plot

.gp_current_total_n_plot <- function(test, params) {
  if (test %in% c("t_two_sample", "t_linear_regression_two_groups", "f_variance_two", "exact_fisher", "z_corr_independent", "wilcoxon_mann_whitney")) {
    return((params$n1 %||% params$n_per_group %||% 30) + (params$n2 %||% params$n_per_group %||% 30))
  }
  params$total_n %||% params$n %||% 80
}

.gp_distribution_plot_core <- function(result) {
  dist <- result$distribution
  if (is.null(dist$type)) {
    return(ggplot2::ggplot() + ggplot2::theme_void())
  }

  if (dist$type == "t") {
    upper <- stats::qt(0.999, dist$df, ncp = dist$ncp)
    lower <- stats::qt(0.001, dist$df, ncp = min(0, dist$ncp))
    x <- seq(lower, upper, length.out = 500)
    dat <- rbind(
      data.frame(x = x, density = stats::dt(x, dist$df), distribution = "H0"),
      data.frame(x = x, density = stats::dt(x, dist$df, ncp = dist$ncp), distribution = "H1")
    )
  } else if (dist$type == "F") {
    x <- seq(0, stats::qf(0.995, dist$df1, dist$df2, ncp = dist$ncp %||% 0), length.out = 500)
    dat <- rbind(
      data.frame(x = x, density = stats::df(x, dist$df1, dist$df2), distribution = "H0"),
      data.frame(x = x, density = stats::df(x, dist$df1, dist$df2, ncp = dist$ncp %||% 0), distribution = "H1")
    )
  } else if (dist$type == "chi-square") {
    x <- seq(0, stats::qchisq(0.995, dist$df, ncp = dist$ncp %||% 0), length.out = 500)
    dat <- rbind(
      data.frame(x = x, density = stats::dchisq(x, dist$df), distribution = "H0"),
      data.frame(x = x, density = stats::dchisq(x, dist$df, ncp = dist$ncp %||% 0), distribution = "H1")
    )
  } else {
    mean_h1 <- dist$mean_h1 %||% 0
    sd_h1 <- dist$sd_h1 %||% 1
    x <- seq(min(-4, mean_h1 - 4 * sd_h1), max(4, mean_h1 + 4 * sd_h1), length.out = 500)
    dat <- rbind(
      data.frame(x = x, density = stats::dnorm(x), distribution = "H0"),
      data.frame(x = x, density = stats::dnorm(x, mean = mean_h1, sd = sd_h1), distribution = "H1")
    )
  }

  ggplot2::ggplot(dat, ggplot2::aes(x = x, y = density, color = distribution, fill = distribution)) +
    ggplot2::geom_area(alpha = 0.08, position = "identity") +
    ggplot2::geom_line(linewidth = 1) +
    ggplot2::labs(
      title = "Central and noncentral distributions",
      subtitle = result$test,
      x = "Test statistic",
      y = "Density"
    )
}

test_that("Fisher exact power uses exact enumeration for feasible grids", {
  result <- power_compute(
    "exact_fisher",
    "post_hoc",
    p0 = 0.4,
    p1 = 0.7,
    n1 = 12,
    n2 = 12,
    alpha = 0.05,
    tails = "greater"
  )

  expect_true(result$outputs$power > 0)
  expect_true(result$outputs$power < 1)
  expect_match(result$notes, "enumerates")
})

test_that("Fisher exact one-sided direction follows the alternative group", {
  result <- power_compute(
    "exact_fisher",
    "post_hoc",
    p0 = 0.4,
    p1 = 0.7,
    n1 = 12,
    n2 = 12,
    alpha = 0.05,
    tails = "one"
  )

  expect_true(result$outputs$power > 0.1)
  expect_equal(result$inputs$tails, "less")
})

test_that("linear regression one-group slope reproduces manual formula", {
  result <- power_compute(
    "t_linear_regression",
    "post_hoc",
    slope_h1 = -0.0667,
    slope_h0 = 0,
    sd_x = 7.5,
    sd_y = 4,
    n = 100,
    alpha = 0.05,
    tails = "two"
  )

  expect_equal(abs(result$outputs$noncentrality_parameter), 1.250625, tolerance = 1e-5)
  expect_equal(result$outputs$df, 98)
})

test_that("two-group slope test uses N minus four degrees of freedom", {
  result <- power_compute(
    "t_linear_regression_two_groups",
    "post_hoc",
    delta_slope = 0.01592,
    residual_sd = 0.5578413,
    sd_x1 = 9.02914,
    sd_x2 = 11.86779,
    n1 = 163,
    n2 = 256,
    alpha = 0.05,
    tails = "two"
  )

  expect_equal(result$outputs$df, 415)
  expect_true(result$outputs$power > 0.79)
})

test_that("dependent correlation common-index validates correlation matrix", {
  expect_error(
    power_compute(
      "z_corr_dependent_common",
      "post_hoc",
      rho_ab = 0.4,
      rho_ac = -0.4,
      rho_bc = 0.99,
      n = 100
    ),
    "valid correlation matrix"
  )
})

test_that("dependent correlation kernels return valid power", {
  common <- power_compute(
    "z_corr_dependent_common",
    "post_hoc",
    rho_ab = 0.4,
    rho_ac = 0.2,
    rho_bc = 0.5,
    n = 144,
    alpha = 0.05,
    tails = "one"
  )
  no_common <- power_compute(
    "z_corr_dependent_no_common",
    "post_hoc",
    rho_ab = 0.1,
    rho_cd = 0.2,
    rho_ac = 0.5,
    rho_ad = 0.4,
    rho_bc = -0.4,
    rho_bd = 0.8,
    n = 200,
    alpha = 0.05,
    tails = "one"
  )

  expect_true(common$outputs$power > 0 && common$outputs$power < 1)
  expect_true(no_common$outputs$power > 0 && no_common$outputs$power < 1)
})

test_that("calculator exposes distribution helpers", {
  expect_equal(ggpower_calculator("2^3"), 8)
  expect_equal(ggpower_calculator("zinv(.975)"), stats::qnorm(.975))
  expect_equal(ggpower_calculator("ncfcdf(3, 2, 20, 4)"), stats::pf(3, 2, 20, ncp = 4))
})

test_that("publication plotting functions return ggplot objects", {
  result <- power_compute("t_one_sample", "post_hoc", d = 0.5, n = 40)
  expect_s3_class(plot_distribution(result), "ggplot")
  expect_s3_class(plot_power_curve("t_one_sample", n_values = c(20, 30, 40), d = 0.5), "ggplot")
})

## ----include=FALSE------------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(ggpower)

## -----------------------------------------------------------------------------
d <- effect_size_d(mean_h1 = 15, mean_h0 = 10, sd = 8)
d

## -----------------------------------------------------------------------------
power_compute("t_one_sample", "a_priori", d = d, alpha = 0.05,
              power = 0.95, tails = "one")

## -----------------------------------------------------------------------------
ggpower_tests()[, c("id", "family", "domain", "module")]

## ----modules, echo=FALSE------------------------------------------------------
data.frame(
  Question = c(
    "What sample size for a standard t test or ANOVA?",
    "Can my biomarker discriminate cases from controls (AUC)?",
    "Is my classifier sensitive and specific enough?",
    "Does a biomarker predict survival?",
    "How many patients for a Phase III superiority trial?",
    "Is treatment non-inferior to standard of care?",
    "Is a new formulation equivalent (bioequivalence)?",
    "Oncology single-arm Phase II with early stopping?"
  ),
  Module = c(
    "Power Workspace",
    "Biomarker Discovery",
    "Biomarker Discovery",
    "Biomarker Discovery",
    "Clinical Trials",
    "Clinical Trials",
    "Clinical Trials",
    "Clinical Trials"
  ),
  Example_test = c(
    "t_two_sample",
    "roc_auc_one",
    "diagnostic_acc",
    "cox_regression",
    "rct_superiority_continuous",
    "rct_noninferiority_binary",
    "rct_equivalence_continuous",
    "simon_two_stage"
  )
)

## ----filter-------------------------------------------------------------------
ggpower_tests(module = "biomarker")[, c("id", "label")]

## ----filter2------------------------------------------------------------------
ggpower_tests(module = "clinical")[, c("id", "label")]

#' Compute statistical power analyses
#'
#' @param test Character test id. Use `ggpower_tests()` to list ids.
#' @param analysis One of `a_priori`, `compromise`, `criterion`, `post_hoc`,
#'   or `sensitivity`.
#' @param ... Test-specific inputs.
#'
#' @return A `ggpower_result` list with components `test`, `analysis`,
#'   `inputs`, `outputs`, and optional `notes` and `distribution`. The
#'   `outputs` element contains the solved quantities (for example sample size,
#'   power, or effect size depending on the analysis mode). See [ggpower_result].
#' @export
power_compute <- function(test, analysis = "post_hoc", ...) {
  spec <- ggpower_get_test(test)
  analysis <- match.arg(
    analysis,
    choices = c("a_priori", "compromise", "criterion", "post_hoc", "sensitivity")
  )

  if (!analysis %in% spec$modes) {
    stop("'", analysis, "' is not supported for ", spec$label, ".", call. = FALSE)
  }

  params <- list(...)
  params$test <- spec$id
  params$analysis <- analysis

  switch(
    analysis,
    post_hoc = .gp_post_hoc(spec$id, params, analysis),
    a_priori = .gp_solve_n(spec, params),
    criterion = .gp_solve_alpha(spec, params),
    sensitivity = .gp_solve_effect(spec, params),
    compromise = .gp_solve_compromise(spec, params)
  )
}

.gp_solve_n <- function(spec, params) {
  target <- .gp_power_target(params)
  lower <- .gp_min_total_n(spec$id, params)
  upper <- max(50, lower * 2)

  objective <- function(n_total) {
    trial <- .gp_set_total_n(spec$id, params, ceiling(n_total))
    .gp_output_power(.gp_post_hoc(spec$id, trial, "a_priori")) - target
  }

  while (objective(upper) < 0 && upper < 1e7) {
    upper <- upper * 2
  }
  if (objective(upper) < 0) {
    stop("Requested power could not be reached before N = 10,000,000.", call. = FALSE)
  }

  if (objective(lower) >= 0) {
    n_total <- lower
  } else {
    n_total <- ceiling(.gp_solve_root(objective, lower, upper, "sample size"))
    while (objective(n_total) < 0) {
      n_total <- n_total + 1
    }
  }

  out <- .gp_post_hoc(spec$id, .gp_set_total_n(spec$id, params, n_total), "a_priori")
  out$inputs$target_power <- target
  out$outputs$actual_power <- out$outputs$power
  out$outputs$power <- NULL
  out$notes <- c(out$notes, "A priori sample sizes are rounded up to integer values and actual power is recomputed.")
  out
}

.gp_solve_alpha <- function(spec, params) {
  target <- .gp_power_target(params)
  objective <- function(alpha) {
    trial <- params
    trial$alpha <- alpha
    .gp_output_power(.gp_post_hoc(spec$id, trial, "criterion")) - target
  }

  alpha <- .gp_solve_root(objective, 1e-8, 0.999999, "alpha")
  out <- .gp_post_hoc(spec$id, within(params, alpha <- alpha), "criterion")
  out$inputs$target_power <- target
  out$outputs$alpha <- alpha
  out$outputs$beta <- 1 - out$outputs$power
  out
}

.gp_solve_effect <- function(spec, params) {
  target <- .gp_power_target(params)
  effect_name <- spec$effect

  if (effect_name %in% c("variance_ratio", "odds_ratio", "exp_beta1", "p1", "auc", "auc1", "hazard_ratio", "sensitivity")) {
    null <- switch(
      effect_name,
      p1 = .gp_numeric(params, "p0", 0.5, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE),
      auc = .gp_numeric(params, "auc0", 0.5, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE),
      auc1 = .gp_arg(params, "auc2", 0.65),
      hazard_ratio = 1,
      sensitivity = 0.5,
      1
    )
    start <- .gp_arg(params, effect_name, .gp_default_effect(effect_name))
    direction <- if (is.numeric(start) && length(start) == 1 && start < null) -1 else 1
    max_delta <- if (effect_name == "p1") {
      if (direction > 0) 1 - null - 1e-8 else null - 1e-8
    } else if (effect_name %in% c("auc", "sensitivity")) {
      if (direction > 0) 1 - null - 1e-8 else null - 1e-8
    } else if (effect_name == "hazard_ratio") {
      if (direction > 0) 1e4 - null else null - 1e-8
    } else if (direction > 0) {
      1e4 - null
    } else {
      null - 1e-8
    }
    if (max_delta <= 1e-8) {
      stop("Requested effect-size direction leaves no valid search range.", call. = FALSE)
    }

    lower <- 1e-8
    upper <- min(max(abs(start - null), 0.1), max_delta)
    ratio_objective <- function(delta) {
      trial <- params
      trial[[effect_name]] <- null + direction * delta
      .gp_output_power(.gp_post_hoc(spec$id, trial, "sensitivity")) - target
    }

    while (ratio_objective(upper) < 0 && upper < max_delta) {
      upper <- min(upper * 2, max_delta)
    }
    if (ratio_objective(upper) < 0) {
      stop("Requested power could not be reached for the effect-size search range.", call. = FALSE)
    }

    delta <- if (ratio_objective(lower) >= 0) {
      lower
    } else {
      .gp_solve_root(ratio_objective, lower, upper, "effect size")
    }
    params[[effect_name]] <- null + direction * delta
    out <- .gp_post_hoc(spec$id, params, "sensitivity")
    out$inputs$target_power <- target
    out$outputs[[effect_name]] <- params[[effect_name]]
    return(out)
  }

  if (effect_name %in% c("rho", "rho_ac", "rho_cd")) {
    null <- switch(
      effect_name,
      rho = .gp_arg(params, "rho0", 0),
      rho_ac = .gp_arg(params, "rho_ab", 0.4),
      rho_cd = .gp_arg(params, "rho_ab", 0.1)
    )
    null <- .gp_check_correlation(null, paste0(effect_name, "_null"))
    start <- switch(
      effect_name,
      rho = .gp_arg(params, "rho", .gp_default_effect("rho")),
      rho_ac = .gp_arg(params, "rho_ac", 0.2),
      rho_cd = .gp_arg(params, "rho_cd", 0.2)
    )
    preferred_direction <- if (is.numeric(start) && length(start) == 1 && start < null) -1 else 1
    directions <- unique(c(preferred_direction, -preferred_direction))

    for (direction in directions) {
      max_delta <- if (direction > 0) 1 - null - 1e-8 else null + 1 - 1e-8
      if (max_delta <= 1e-8) {
        next
      }

      lower <- 1e-8
      upper <- min(max(abs(start - null), 0.1), max_delta)
      correlation_objective <- function(delta) {
        trial <- params
        trial[[effect_name]] <- null + direction * delta
        .gp_output_power(.gp_post_hoc(spec$id, trial, "sensitivity")) - target
      }

      found_upper <- FALSE
      while (TRUE) {
        upper_value <- tryCatch(correlation_objective(upper), error = function(e) NA_real_)
        if (is.finite(upper_value) && upper_value >= 0) {
          found_upper <- TRUE
          break
        }
        if (upper >= max_delta) {
          break
        }
        upper <- min(upper * 2, max_delta)
      }
      if (!found_upper) {
        next
      }

      delta <- if (correlation_objective(lower) >= 0) {
        lower
      } else {
        .gp_solve_root(correlation_objective, lower, upper, "effect size")
      }
      params[[effect_name]] <- null + direction * delta
      out <- .gp_post_hoc(spec$id, params, "sensitivity")
      out$inputs$target_power <- target
      out$outputs[[effect_name]] <- params[[effect_name]]
      return(out)
    }

    stop("Requested power could not be reached for the effect-size search range.", call. = FALSE)
  }

  start <- abs(.gp_arg(params, effect_name, .gp_default_effect(effect_name)))
  lower <- 1e-8
  upper <- max(start, 0.1)

  effect_objective <- function(effect) {
    trial <- params
    trial[[effect_name]] <- .gp_effect_sign(params, effect_name) * effect
    .gp_output_power(.gp_post_hoc(spec$id, trial, "sensitivity")) - target
  }

  while (effect_objective(upper) < 0 && upper < 1e4) {
    upper <- upper * 2
  }
  if (effect_objective(upper) < 0) {
    stop("Requested power could not be reached for the effect-size search range.", call. = FALSE)
  }

  effect <- .gp_solve_root(effect_objective, lower, upper, "effect size")
  params[[effect_name]] <- .gp_effect_sign(params, effect_name) * effect
  out <- .gp_post_hoc(spec$id, params, "sensitivity")
  out$inputs$target_power <- target
  out$outputs[[effect_name]] <- params[[effect_name]]
  out
}

.gp_solve_compromise <- function(spec, params) {
  q <- .gp_numeric(params, "q", 1, min = 0, strict_min = TRUE)

  objective <- function(alpha) {
    trial <- params
    trial$alpha <- alpha
    power <- .gp_output_power(.gp_post_hoc(spec$id, trial, "compromise"))
    beta <- 1 - power
    beta / alpha - q
  }

  alpha <- tryCatch(
    .gp_solve_root(objective, 1e-8, 0.999999, "alpha-beta compromise"),
    error = function(e) {
      opt <- stats::optimize(function(a) abs(objective(a)), c(1e-8, 0.999999))
      opt$minimum
    }
  )

  out <- .gp_post_hoc(spec$id, within(params, alpha <- alpha), "compromise")
  out$outputs$alpha <- alpha
  out$outputs$beta <- 1 - out$outputs$power
  out$outputs$beta_alpha_ratio <- out$outputs$beta / alpha
  out$notes <- c(out$notes, "Compromise analysis solves alpha so beta / alpha matches the requested ratio as closely as possible.")
  out
}

.gp_post_hoc <- function(test, params, analysis = "post_hoc") {
  switch(
    test,
    t_one_sample = .gp_t_one_sample(params, analysis),
    t_paired = .gp_t_paired(params, analysis),
    t_two_sample = .gp_t_two_sample(params, analysis),
    t_point_biserial = .gp_t_point_biserial(params, analysis),
    t_linear_regression = .gp_t_linear_regression(params, analysis),
    t_linear_regression_two_groups = .gp_t_linear_regression_two_groups(params, analysis),
    t_generic = .gp_t_generic(params, analysis),
    f_anova_one_way = .gp_f_anova_one_way(params, analysis),
    f_anova_special = .gp_f_anova_special(params, analysis),
    f_mreg_omnibus = .gp_f_mreg_omnibus(params, analysis),
    f_mreg_increase = .gp_f_mreg_increase(params, analysis),
    f_variance_two = .gp_f_variance_two(params, analysis),
    chisq_variance_one = .gp_chisq_variance_one(params, analysis),
    chisq_gof = .gp_chisq_power(params, analysis, "Goodness-of-fit tests: Contingency tables"),
    chisq_contingency = .gp_chisq_power(params, analysis, "Contingency tables"),
    exact_binomial = .gp_exact_binomial(params, analysis, "Generic binomial test"),
    exact_one_proportion = .gp_exact_binomial(params, analysis, "Proportion: Difference from constant"),
    exact_sign = .gp_exact_binomial(params, analysis, "Proportion: Sign test"),
    exact_mcnemar = .gp_exact_binomial(params, analysis, "McNemar test approximation through discordant-pair binomial test"),
    exact_fisher = .gp_exact_fisher(params, analysis),
    exact_correlation = .gp_exact_correlation(params, analysis),
    exact_mreg_random = .gp_exact_mreg_random(params, analysis),
    z_corr_independent = .gp_z_corr_independent(params, analysis),
    z_corr_dependent_common = .gp_z_corr_dependent_common(params, analysis),
    z_corr_dependent_no_common = .gp_z_corr_dependent_no_common(params, analysis),
    z_logistic = .gp_z_logistic(params, analysis),
    z_poisson = .gp_z_poisson(params, analysis),
    z_tetrachoric = .gp_z_tetrachoric(params, analysis),
    wilcoxon_signed = .gp_wilcoxon_signed(params, analysis),
    wilcoxon_mann_whitney = .gp_wilcoxon_mann_whitney(params, analysis),
    roc_auc_one = .gp_roc_auc_one(params, analysis),
    roc_auc_two = .gp_roc_auc_two(params, analysis),
    diagnostic_acc = .gp_diagnostic_acc(params, analysis),
    survival_logrank = .gp_survival_logrank(params, analysis),
    cox_regression = .gp_cox_regression(params, analysis),
    discovery_fdr = .gp_discovery_fdr(params, analysis),
    ttest_biomarker = .gp_ttest_biomarker(params, analysis),
    rct_superiority_continuous = .gp_rct_superiority_continuous(params, analysis),
    rct_superiority_binary = .gp_rct_superiority_binary(params, analysis),
    rct_noninferiority_continuous = .gp_rct_noninferiority_continuous(params, analysis),
    rct_noninferiority_binary = .gp_rct_noninferiority_binary(params, analysis),
    rct_equivalence_continuous = .gp_rct_equivalence_continuous(params, analysis),
    rct_equivalence_proportion = .gp_rct_equivalence_proportion(params, analysis),
    simon_two_stage = .gp_simon_two_stage(params, analysis),
    cluster_rct = .gp_cluster_rct(params, analysis),
    multi_arm_superiority = .gp_multi_arm_superiority(params, analysis),
    count_endpoint_poisson = .gp_count_endpoint_poisson(params, analysis),
    survival_pmu = .gp_survival_pmu(params, analysis),
    stop("No compute kernel registered for '", test, "'.", call. = FALSE)
  )
}

.gp_t_one_sample <- function(params, analysis) {
  d <- .gp_numeric(params, "d", 0.5)
  n <- .gp_count(params, "n", .gp_arg(params, "total_n", 40), min = 2)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)
  df <- n - 1
  ncp <- d * sqrt(n)
  pwr <- .gp_power_t(df, ncp, alpha, tails)

  .gp_result(
    "t test: Means - difference from constant (one sample case)", analysis,
    list(tails = pwr$direction, effect_size_d = d, alpha = alpha, total_sample_size = n),
    list(noncentrality_parameter = ncp, critical_t = pwr$critical, df = df, power = pwr$power),
    distribution = list(type = "t", df = df, ncp = ncp)
  )
}

.gp_t_paired <- function(params, analysis) {
  result <- .gp_t_one_sample(params, analysis)
  result$test <- "t test: Means - difference between two dependent means (matched pairs)"
  names(result$inputs)[names(result$inputs) == "effect_size_d"] <- "effect_size_dz"
  result
}

.gp_t_two_sample <- function(params, analysis) {
  d <- .gp_numeric(params, "d", 0.5)
  n1 <- .gp_count(params, "n1", .gp_arg(params, "n_per_group", 30), min = 2)
  n2 <- .gp_count(params, "n2", .gp_arg(params, "n_per_group", n1), min = 2)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)
  df <- n1 + n2 - 2
  ncp <- d * sqrt(n1 * n2 / (n1 + n2))
  pwr <- .gp_power_t(df, ncp, alpha, tails)

  .gp_result(
    "t test: Means - difference between two independent means (two groups)", analysis,
    list(tails = pwr$direction, effect_size_d = d, alpha = alpha, sample_size_group_1 = n1, sample_size_group_2 = n2),
    list(noncentrality_parameter = ncp, critical_t = pwr$critical, df = df, total_sample_size = n1 + n2, power = pwr$power),
    distribution = list(type = "t", df = df, ncp = ncp)
  )
}

.gp_t_point_biserial <- function(params, analysis) {
  rho <- abs(.gp_check_correlation(.gp_numeric(params, "rho", 0.3), "rho"))
  n <- .gp_count(params, "n", .gp_arg(params, "total_n", 100), min = 4)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)
  df <- n - 2
  ncp <- sqrt(rho^2 * n / (1 - rho^2))
  pwr <- .gp_power_t(df, ncp, alpha, tails)

  .gp_result(
    "t test: Correlation - point biserial model", analysis,
    list(tails = pwr$direction, effect_size_rho = rho, alpha = alpha, total_sample_size = n),
    list(noncentrality_parameter = ncp, critical_t = pwr$critical, df = df, power = pwr$power),
    distribution = list(type = "t", df = df, ncp = ncp)
  )
}

.gp_t_generic <- function(params, analysis) {
  ncp <- .gp_numeric(params, "ncp", 1)
  df <- .gp_numeric(params, "df", .gp_numeric(params, "n", 30) - 1, min = 1)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)
  pwr <- .gp_power_t(df, ncp, alpha, tails)

  .gp_result(
    "t test: Generic case", analysis,
    list(tails = pwr$direction, noncentrality_parameter = ncp, alpha = alpha, df = df),
    list(critical_t = pwr$critical, power = pwr$power),
    notes = "Generic t tests do not have a unique sample-size definition, so a priori mode is not available.",
    distribution = list(type = "t", df = df, ncp = ncp)
  )
}

.gp_t_linear_regression <- function(params, analysis) {
  slope_h1 <- .gp_numeric(params, "slope_h1", -0.0667)
  slope_h0 <- .gp_numeric(params, "slope_h0", 0)
  sd_x <- .gp_numeric(params, "sd_x", 1, min = 0, strict_min = TRUE)
  sd_y <- .gp_numeric(params, "sd_y", 1, min = 0, strict_min = TRUE)
  n <- .gp_count(params, "n", .gp_arg(params, "total_n", 100), min = 3)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)
  df <- n - 2
  ncp <- sqrt(n) * sd_x * (slope_h1 - slope_h0) / sd_y
  pwr <- .gp_power_t(df, ncp, alpha, tails)

  .gp_result(
    "t test: Linear Regression (size of slope, one group)", analysis,
    list(tails = pwr$direction, slope_h1 = slope_h1, slope_h0 = slope_h0, sd_x = sd_x, sd_y = sd_y, alpha = alpha, total_sample_size = n),
    list(noncentrality_parameter = ncp, critical_t = pwr$critical, df = df, power = pwr$power),
    distribution = list(type = "t", df = df, ncp = ncp)
  )
}

.gp_t_linear_regression_two_groups <- function(params, analysis) {
  delta_slope <- .gp_numeric(params, "delta_slope", 0.1)
  residual_sd <- .gp_numeric(params, "residual_sd", 1, min = 0, strict_min = TRUE)
  sd_x1 <- .gp_numeric(params, "sd_x1", 1, min = 0, strict_min = TRUE)
  sd_x2 <- .gp_numeric(params, "sd_x2", 1, min = 0, strict_min = TRUE)
  n1 <- .gp_count(params, "n1", 30, min = 3)
  n2 <- .gp_count(params, "n2", .gp_arg(params, "n_per_group", n1), min = 3)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)
  df <- n1 + n2 - 4
  se <- residual_sd * sqrt(1 / (n1 * sd_x1^2) + 1 / (n2 * sd_x2^2))
  ncp <- delta_slope / se
  pwr <- .gp_power_t(df, ncp, alpha, tails)

  .gp_result(
    "t test: Linear Regression (two groups)", analysis,
    list(tails = pwr$direction, delta_slope = delta_slope, residual_sd = residual_sd, sd_x1 = sd_x1, sd_x2 = sd_x2, alpha = alpha, sample_size_group_1 = n1, sample_size_group_2 = n2),
    list(noncentrality_parameter = ncp, critical_t = pwr$critical, df = df, total_sample_size = n1 + n2, power = pwr$power),
    distribution = list(type = "t", df = df, ncp = ncp)
  )
}

.gp_f_anova_one_way <- function(params, analysis) {
  f <- .gp_numeric(params, "f", 0.25, min = 0, strict_min = TRUE)
  groups <- .gp_count(params, "groups", 3, min = 2)
  total_n <- .gp_count(params, "total_n", .gp_arg(params, "n", 60), min = groups + 2)
  alpha <- .gp_alpha(params)
  df1 <- groups - 1
  df2 <- total_n - groups
  ncp <- f^2 * total_n
  pwr <- .gp_power_f(df1, df2, ncp, alpha)

  .gp_result(
    "F test: Fixed effects ANOVA - one way", analysis,
    list(effect_size_f = f, alpha = alpha, total_sample_size = total_n, groups = groups),
    list(noncentrality_parameter = ncp, critical_f = pwr$critical, numerator_df = df1, denominator_df = df2, power = pwr$power),
    distribution = list(type = "F", df1 = df1, df2 = df2, ncp = ncp)
  )
}

.gp_f_anova_special <- function(params, analysis) {
  f <- .gp_numeric(params, "f", 0.25, min = 0, strict_min = TRUE)
  df1 <- .gp_numeric(params, "df1", 1, min = 1)
  groups <- .gp_count(params, "groups", 4, min = 2)
  total_n <- .gp_count(params, "total_n", .gp_arg(params, "n", 80), min = groups + 2)
  alpha <- .gp_alpha(params)
  df2 <- total_n - groups
  ncp <- f^2 * total_n
  pwr <- .gp_power_f(df1, df2, ncp, alpha)

  .gp_result(
    "F test: Fixed effects ANOVA - special, main effects and interactions", analysis,
    list(effect_size_f = f, alpha = alpha, total_sample_size = total_n, numerator_df = df1, groups = groups),
    list(noncentrality_parameter = ncp, critical_f = pwr$critical, denominator_df = df2, power = pwr$power),
    distribution = list(type = "F", df1 = df1, df2 = df2, ncp = ncp)
  )
}

.gp_f_mreg_omnibus <- function(params, analysis) {
  f2 <- .gp_numeric(params, "f2", 0.15, min = 0, strict_min = TRUE)
  predictors <- .gp_count(params, "predictors", 1, min = 1)
  total_n <- .gp_count(params, "total_n", .gp_arg(params, "n", 100), min = predictors + 3)
  alpha <- .gp_alpha(params)
  df1 <- predictors
  df2 <- total_n - predictors - 1
  ncp <- f2 * total_n
  pwr <- .gp_power_f(df1, df2, ncp, alpha)

  .gp_result(
    "F test: Multiple Regression - omnibus (deviation of R2 from zero), fixed model", analysis,
    list(effect_size_f2 = f2, alpha = alpha, total_sample_size = total_n, predictors = predictors),
    list(noncentrality_parameter = ncp, critical_f = pwr$critical, numerator_df = df1, denominator_df = df2, power = pwr$power),
    distribution = list(type = "F", df1 = df1, df2 = df2, ncp = ncp)
  )
}

.gp_f_mreg_increase <- function(params, analysis) {
  f2 <- .gp_numeric(params, "f2", 0.15, min = 0, strict_min = TRUE)
  tested <- .gp_count(params, "tested_predictors", .gp_arg(params, "df1", 1), min = 1)
  predictors <- .gp_count(params, "predictors", tested + 1, min = tested)
  total_n <- .gp_count(params, "total_n", .gp_arg(params, "n", 120), min = predictors + 3)
  alpha <- .gp_alpha(params)
  df2 <- total_n - predictors - 1
  ncp <- f2 * total_n
  pwr <- .gp_power_f(tested, df2, ncp, alpha)

  .gp_result(
    "F test: Multiple Regression - special (increase of R2), fixed model", analysis,
    list(effect_size_f2 = f2, alpha = alpha, total_sample_size = total_n, tested_predictors = tested, total_predictors = predictors),
    list(noncentrality_parameter = ncp, critical_f = pwr$critical, numerator_df = tested, denominator_df = df2, power = pwr$power),
    distribution = list(type = "F", df1 = tested, df2 = df2, ncp = ncp)
  )
}

.gp_f_variance_two <- function(params, analysis) {
  ratio <- .gp_numeric(params, "variance_ratio", 1.5, min = 0, strict_min = TRUE)
  n1 <- .gp_count(params, "n1", 30, min = 2)
  n2 <- .gp_count(params, "n2", .gp_arg(params, "n_per_group", n1), min = 2)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)
  df1 <- n1 - 1
  df2 <- n2 - 1

  if (tails == "two") {
    lower <- stats::qf(alpha / 2, df1, df2)
    upper <- stats::qf(1 - alpha / 2, df1, df2)
    power <- stats::pf(lower / ratio, df1, df2) + 1 - stats::pf(upper / ratio, df1, df2)
    critical <- c(lower = lower, upper = upper)
    direction <- "two"
  } else if (ratio < 1) {
    critical <- stats::qf(alpha, df1, df2)
    power <- stats::pf(critical / ratio, df1, df2)
    direction <- "less"
  } else {
    critical <- stats::qf(1 - alpha, df1, df2)
    power <- 1 - stats::pf(critical / ratio, df1, df2)
    direction <- "greater"
  }

  .gp_result(
    "F test: Inequality of two variances", analysis,
    list(tails = direction, ratio_var1_var0 = ratio, alpha = alpha, sample_size_group_1 = n1, sample_size_group_2 = n2),
    list(critical_f = critical, numerator_df = df1, denominator_df = df2, total_sample_size = n1 + n2, power = power),
    notes = "Two-variance power uses the scaled central F distribution.",
    distribution = list(type = "F", df1 = df1, df2 = df2, scale = ratio)
  )
}

.gp_chisq_variance_one <- function(params, analysis) {
  ratio <- .gp_numeric(params, "variance_ratio", 0.667, min = 0, strict_min = TRUE)
  n <- .gp_count(params, "n", .gp_arg(params, "total_n", 80), min = 2)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)
  df <- n - 1

  if (tails == "two") {
    lower <- stats::qchisq(alpha / 2, df)
    upper <- stats::qchisq(1 - alpha / 2, df)
    power <- stats::pchisq(lower / ratio, df) + 1 - stats::pchisq(upper / ratio, df)
    critical <- c(lower = lower, upper = upper)
    direction <- "two"
  } else if (ratio < 1) {
    critical <- stats::qchisq(alpha, df)
    power <- stats::pchisq(critical / ratio, df)
    direction <- "less"
  } else {
    critical <- stats::qchisq(1 - alpha, df)
    power <- 1 - stats::pchisq(critical / ratio, df)
    direction <- "greater"
  }

  .gp_result(
    "chi-square test: Variance - difference from constant (one sample case)", analysis,
    list(tails = direction, ratio_var1_var0 = ratio, alpha = alpha, total_sample_size = n),
    list(critical_chisq = critical, df = df, power = power),
    distribution = list(type = "chi-square", df = df, scale = ratio)
  )
}

.gp_chisq_power <- function(params, analysis, label) {
  w <- .gp_numeric(params, "w", 0.3, min = 0, strict_min = TRUE)
  df <- .gp_numeric(params, "df", 1, min = 1)
  total_n <- .gp_count(params, "total_n", .gp_arg(params, "n", 100), min = 2)
  alpha <- .gp_alpha(params)
  ncp <- total_n * w^2
  pwr <- .gp_power_chisq(df, ncp, alpha)

  .gp_result(
    paste("chi-square test:", label), analysis,
    list(effect_size_w = w, alpha = alpha, total_sample_size = total_n, df = df),
    list(noncentrality_parameter = ncp, critical_chisq = pwr$critical, power = pwr$power),
    distribution = list(type = "chi-square", df = df, ncp = ncp)
  )
}

.gp_exact_binomial <- function(params, analysis, label) {
  p0 <- .gp_numeric(params, "p0", 0.5, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  p1 <- .gp_numeric(params, "p1", 0.65, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  n <- .gp_count(params, "n", .gp_arg(params, "total_n", 50), min = 1)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)
  alternative <- if (tails == "two") "two.sided" else if (p1 < p0) "less" else "greater"
  power <- .gp_binomial_power(n, p0, p1, alpha, alternative)

  .gp_result(
    paste("Exact:", label), analysis,
    list(tails = alternative, p_h0 = p0, p_h1 = p1, alpha = alpha, total_sample_size = n),
    list(power = power),
    notes = "Exact binomial power sums probabilities for outcomes whose exact binomial-test p-value is at or below alpha.",
    distribution = list(type = "binomial", n = n, p0 = p0, p1 = p1)
  )
}

.gp_exact_fisher <- function(params, analysis) {
  p0 <- .gp_numeric(params, "p0", 0.5, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  p1 <- .gp_numeric(params, "p1", 0.65, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  n1 <- .gp_count(params, "n1", 40, min = 2)
  n2 <- .gp_count(params, "n2", .gp_arg(params, "n_per_group", n1), min = 2)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)
  alternative <- if (tails == "two") "two.sided" else if (p1 < p0) "greater" else "less"
  exact_power <- .gp_fisher_power(n1, n2, p0, p1, alpha, alternative)
  h <- .gp_phi_to_h(p1, p0)

  .gp_result(
    "Exact: Proportions - inequality of two independent groups (Fisher exact)", analysis,
    list(tails = alternative, p_group_1 = p0, p_group_2 = p1, alpha = alpha, sample_size_group_1 = n1, sample_size_group_2 = n2),
    list(effect_size_h = h, total_sample_size = n1 + n2, power = exact_power),
    notes = "Fisher exact power enumerates all two-binomial outcome pairs and sums outcomes rejected by Fisher's exact test.",
    distribution = list(type = "fisher_exact", n1 = n1, n2 = n2, p0 = p0, p1 = p1)
  )
}

.gp_exact_correlation <- function(params, analysis) {
  rho0 <- .gp_check_correlation(.gp_numeric(params, "rho0", 0), "rho0")
  rho <- .gp_check_correlation(.gp_numeric(params, "rho", 0.3), "rho")
  n <- .gp_count(params, "n", .gp_arg(params, "total_n", 100), min = 5)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)
  q <- .gp_fisher_z(rho) - .gp_fisher_z(rho0)
  mean_h1 <- q * sqrt(n - 3)
  pwr <- .gp_power_z(mean_h1, alpha, tails)

  .gp_result(
    "Exact: Correlation - difference from constant (one sample case)", analysis,
    list(tails = pwr$direction, rho_h0 = rho0, rho_h1 = rho, alpha = alpha, total_sample_size = n),
    list(effect_size_q = q, critical_z = pwr$critical, power = pwr$power),
    notes = "Correlation support uses Fisher Z approximation; exact small-sample correlation distribution is planned.",
    distribution = list(type = "normal", mean_h1 = mean_h1)
  )
}

.gp_exact_mreg_random <- function(params, analysis) {
  rho2 <- .gp_numeric(params, "rho2", .gp_arg(params, "r2", 0.1), min = 0, max = 1, strict_max = TRUE)
  predictors <- .gp_count(params, "predictors", 1, min = 1)
  total_n <- .gp_count(params, "total_n", .gp_arg(params, "n", 100), min = predictors + 3)
  alpha <- .gp_alpha(params)
  df1 <- predictors
  df2 <- total_n - predictors - 1
  f2 <- rho2 / (1 - rho2)
  ncp <- f2 * total_n
  pwr <- .gp_power_f(df1, df2, ncp, alpha)

  .gp_result(
    "Exact test: Multiple Regression - random model", analysis,
    list(rho2 = rho2, alpha = alpha, total_sample_size = total_n, predictors = predictors),
    list(effect_size_f2 = f2, noncentrality_parameter = ncp, critical_f = pwr$critical, numerator_df = df1, denominator_df = df2, power = pwr$power),
    notes = "Random-model multiple regression uses the noncentral F representation for R2 planning.",
    distribution = list(type = "F", df1 = df1, df2 = df2, ncp = ncp)
  )
}

.gp_z_corr_independent <- function(params, analysis) {
  q_effect <- .gp_numeric(params, "q_effect", .gp_arg(params, "q_corr", 0.3))
  n1 <- .gp_count(params, "n1", 50, min = 4)
  n2 <- .gp_count(params, "n2", .gp_arg(params, "n_per_group", n1), min = 4)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)
  se <- sqrt(1 / (n1 - 3) + 1 / (n2 - 3))
  mean_h1 <- q_effect / se
  pwr <- .gp_power_z(mean_h1, alpha, tails)

  .gp_result(
    "z test: Correlation - inequality of two independent Pearson r's", analysis,
    list(tails = pwr$direction, effect_size_q = q_effect, alpha = alpha, sample_size_group_1 = n1, sample_size_group_2 = n2),
    list(critical_z = pwr$critical, total_sample_size = n1 + n2, power = pwr$power),
    distribution = list(type = "normal", mean_h1 = mean_h1)
  )
}

.gp_z_corr_dependent_common <- function(params, analysis) {
  rho_ab <- .gp_check_correlation(.gp_numeric(params, "rho_ab", 0.4), "rho_ab")
  rho_ac <- .gp_check_correlation(.gp_numeric(params, "rho_ac", 0.2), "rho_ac")
  rho_bc <- .gp_check_correlation(.gp_numeric(params, "rho_bc", 0.5), "rho_bc")
  n <- .gp_count(params, "n", .gp_arg(params, "total_n", 100), min = 5)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)

  mat_h1 <- matrix(c(1, rho_ab, rho_ac, rho_ab, 1, rho_bc, rho_ac, rho_bc, 1), 3, 3)
  if (!.gp_positive_semidefinite(mat_h1)) {
    stop("The supplied correlations do not form a valid correlation matrix.", call. = FALSE)
  }

  c0 <- .gp_corr_cov_common(rho_ab, rho_ab, rho_bc)
  c1 <- .gp_corr_cov_common(rho_ab, rho_ac, rho_bc)
  s0 <- sqrt((2 - 2 * c0) / (n - 3))
  s1 <- sqrt((2 - 2 * c1) / (n - 3)) / s0
  mean_h1 <- (.gp_fisher_z(rho_ab) - .gp_fisher_z(rho_ac)) / s0
  pwr <- .gp_power_z(mean_h1, alpha, tails, sd_h1 = s1)

  .gp_result(
    "z test: Correlation - inequality of two dependent Pearson r's (common index)", analysis,
    list(tails = pwr$direction, rho_ab = rho_ab, rho_ac = rho_ac, rho_bc = rho_bc, alpha = alpha, total_sample_size = n),
    list(critical_z = pwr$critical, power = pwr$power),
    distribution = list(type = "normal", mean_h1 = mean_h1, sd_h1 = s1)
  )
}

.gp_z_corr_dependent_no_common <- function(params, analysis) {
  rho_ab <- .gp_check_correlation(.gp_numeric(params, "rho_ab", 0.1), "rho_ab")
  rho_cd <- .gp_check_correlation(.gp_numeric(params, "rho_cd", 0.2), "rho_cd")
  rho_ac <- .gp_check_correlation(.gp_numeric(params, "rho_ac", 0.5), "rho_ac")
  rho_ad <- .gp_check_correlation(.gp_numeric(params, "rho_ad", 0.4), "rho_ad")
  rho_bc <- .gp_check_correlation(.gp_numeric(params, "rho_bc", -0.4), "rho_bc")
  rho_bd <- .gp_check_correlation(.gp_numeric(params, "rho_bd", 0.8), "rho_bd")
  n <- .gp_count(params, "n", .gp_arg(params, "total_n", 100), min = 5)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)

  mat_h1 <- matrix(
    c(1, rho_ab, rho_ac, rho_ad,
      rho_ab, 1, rho_bc, rho_bd,
      rho_ac, rho_bc, 1, rho_cd,
      rho_ad, rho_bd, rho_cd, 1),
    4, 4
  )
  if (!.gp_positive_semidefinite(mat_h1)) {
    stop("The supplied correlations do not form a valid correlation matrix.", call. = FALSE)
  }

  c0 <- .gp_corr_cov_no_common(rho_ab, rho_ab, rho_ac, rho_ad, rho_bc, rho_bd)
  c1 <- .gp_corr_cov_no_common(rho_ab, rho_cd, rho_ac, rho_ad, rho_bc, rho_bd)
  s0 <- sqrt((2 - 2 * c0) / (n - 3))
  s1 <- sqrt((2 - 2 * c1) / (n - 3)) / s0
  mean_h1 <- (.gp_fisher_z(rho_ab) - .gp_fisher_z(rho_cd)) / s0
  pwr <- .gp_power_z(mean_h1, alpha, tails, sd_h1 = s1)

  .gp_result(
    "z test: Correlation - inequality of two dependent Pearson r's (no common index)", analysis,
    list(tails = pwr$direction, rho_ab = rho_ab, rho_cd = rho_cd, rho_ac = rho_ac, rho_ad = rho_ad, rho_bc = rho_bc, rho_bd = rho_bd, alpha = alpha, total_sample_size = n),
    list(critical_z = pwr$critical, power = pwr$power),
    distribution = list(type = "normal", mean_h1 = mean_h1, sd_h1 = s1)
  )
}

.gp_z_logistic <- function(params, analysis) {
  odds_ratio <- .gp_numeric(params, "odds_ratio", 1.5, min = 0, strict_min = TRUE)
  p0 <- .gp_numeric(params, "p0", 0.5, min = 0, max = 1, strict_min = TRUE, strict_max = TRUE)
  total_n <- .gp_count(params, "total_n", .gp_arg(params, "n", 300), min = 5)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)
  r2 <- .gp_numeric(params, "r2_other", 0, min = 0, max = 1)
  x_var <- .gp_var_x(params)
  beta <- log(odds_ratio)
  mean_h1 <- beta * sqrt(total_n * (1 - r2) * p0 * (1 - p0) * x_var)
  pwr <- .gp_power_z(mean_h1, alpha, tails)

  .gp_result(
    "z test: Multiple logistic regression", analysis,
    list(tails = pwr$direction, odds_ratio = odds_ratio, p_h0 = p0, alpha = alpha, total_sample_size = total_n, r2_other_x = r2, x_variance = x_var),
    list(critical_z = pwr$critical, beta1 = beta, power = pwr$power),
    notes = "Logistic regression support uses a large-sample Wald approximation suitable for planning; enumeration and Demidenko variants can be added later.",
    distribution = list(type = "normal", mean_h1 = mean_h1)
  )
}

.gp_z_poisson <- function(params, analysis) {
  exp_beta1 <- .gp_numeric(params, "exp_beta1", 1.3, min = 0, strict_min = TRUE)
  base_rate <- .gp_numeric(params, "base_rate", 0.85, min = 0, strict_min = TRUE)
  exposure <- .gp_numeric(params, "exposure", 1, min = 0, strict_min = TRUE)
  total_n <- .gp_count(params, "total_n", .gp_arg(params, "n", 300), min = 5)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)
  r2 <- .gp_numeric(params, "r2_other", 0, min = 0, max = 1)
  x_var <- .gp_var_x(params)
  beta <- log(exp_beta1)
  mean_h1 <- beta * sqrt(total_n * (1 - r2) * base_rate * exposure * x_var)
  pwr <- .gp_power_z(mean_h1, alpha, tails)

  .gp_result(
    "z test: Poisson regression", analysis,
    list(tails = pwr$direction, exp_beta1 = exp_beta1, base_rate = base_rate, exposure = exposure, alpha = alpha, total_sample_size = total_n, r2_other_x = r2, x_variance = x_var),
    list(critical_z = pwr$critical, beta1 = beta, power = pwr$power),
    notes = "Poisson regression support uses a large-sample Wald approximation; exact enumeration is a future refinement.",
    distribution = list(type = "normal", mean_h1 = mean_h1)
  )
}

.gp_z_tetrachoric <- function(params, analysis) {
  rho0 <- .gp_check_correlation(.gp_numeric(params, "rho0", 0), "rho0")
  rho <- .gp_check_correlation(.gp_numeric(params, "rho", 0.3), "rho")
  n <- .gp_count(params, "n", .gp_arg(params, "total_n", 100), min = 5)
  alpha <- .gp_alpha(params)
  tails <- .gp_tail(params)
  mean_h1 <- (.gp_fisher_z(rho) - .gp_fisher_z(rho0)) * sqrt(n - 3)
  pwr <- .gp_power_z(mean_h1, alpha, tails)

  .gp_result(
    "z test: Tetrachoric correlation", analysis,
    list(tails = pwr$direction, rho_h0 = rho0, rho_h1 = rho, alpha = alpha, total_sample_size = n),
    list(critical_z = pwr$critical, power = pwr$power),
    notes = "Tetrachoric correlation currently uses the large-sample Fisher-Z style planning approximation.",
    distribution = list(type = "normal", mean_h1 = mean_h1)
  )
}

.gp_wilcoxon_signed <- function(params, analysis) {
  are <- .gp_numeric(params, "are", 3 / pi, min = 0, strict_min = TRUE)
  trial <- params
  trial$n <- max(2, floor(.gp_count(params, "n", 40, min = 2) * are))
  out <- .gp_t_one_sample(trial, analysis)
  out$test <- "Wilcoxon signed-rank test: Means - difference from constant or matched pairs"
  out$inputs$asymptotic_relative_efficiency <- are
  out$notes <- c(out$notes, "Wilcoxon signed-rank support uses the A.R.E. method and reuses the matched/one-sample t-test kernel.")
  out
}

.gp_wilcoxon_mann_whitney <- function(params, analysis) {
  are <- .gp_numeric(params, "are", 3 / pi, min = 0, strict_min = TRUE)
  trial <- params
  trial$n1 <- max(2, floor(.gp_count(params, "n1", 30, min = 2) * are))
  trial$n2 <- max(2, floor(.gp_count(params, "n2", trial$n1, min = 2) * are))
  out <- .gp_t_two_sample(trial, analysis)
  out$test <- "Wilcoxon-Mann-Whitney test of a difference between two independent means"
  out$inputs$asymptotic_relative_efficiency <- are
  out$notes <- c(out$notes, "Wilcoxon-Mann-Whitney support uses the A.R.E. method and reuses the two-sample t-test kernel.")
  out
}

.gp_output_power <- function(result) {
  result$outputs$power %||% result$outputs$actual_power
}

.gp_min_total_n <- function(test, params) {
  switch(
    test,
    t_point_biserial = 4,
    t_linear_regression = 3,
    t_linear_regression_two_groups = 6,
    t_two_sample = 4,
    f_anova_one_way = max(4, .gp_count(params, "groups", 3, min = 2) + 2),
    f_anova_special = max(4, .gp_count(params, "groups", 4, min = 2) + 2),
    f_mreg_omnibus = .gp_count(params, "predictors", 1, min = 1) + 3,
    f_mreg_increase = .gp_count(params, "predictors", 2, min = 1) + 3,
    exact_correlation = 5,
    exact_mreg_random = .gp_count(params, "predictors", 1, min = 1) + 3,
    z_corr_independent = 8,
    z_corr_dependent_common = 5,
    z_corr_dependent_no_common = 5,
    z_logistic = 5,
    z_poisson = 5,
    z_tetrachoric = 5,
    exact_fisher = 4,
    wilcoxon_mann_whitney = 4,
    roc_auc_one = 4,
    roc_auc_two = 4,
    diagnostic_acc = 4,
    survival_logrank = 8,
    cox_regression = 10,
    discovery_fdr = 4,
    ttest_biomarker = 4,
    rct_superiority_continuous = 4,
    rct_superiority_binary = 4,
    rct_noninferiority_continuous = 4,
    rct_noninferiority_binary = 4,
    rct_equivalence_continuous = 4,
    rct_equivalence_proportion = 4,
    simon_two_stage = 4,
    cluster_rct = 4,
    multi_arm_superiority = max(4, .gp_count(params, "groups", 3, min = 2) + 2),
    count_endpoint_poisson = 5,
    survival_pmu = 8,
    2
  )
}

.gp_set_total_n <- function(test, params, n_total) {
  params$total_n <- ceiling(n_total)

  if (test %in% c("t_two_sample", "t_linear_regression_two_groups", "f_variance_two", "exact_fisher", "z_corr_independent", "wilcoxon_mann_whitney", "roc_auc_two", "ttest_biomarker", "rct_superiority_continuous", "rct_superiority_binary", "rct_noninferiority_continuous", "rct_noninferiority_binary", "rct_equivalence_continuous", "rct_equivalence_proportion")) {
    ratio <- .gp_numeric(params, "allocation_ratio", .gp_arg(params, "ratio", 1), min = 0, strict_min = TRUE)
    per_group_min <- if (test == "t_linear_regression_two_groups") 3 else 2
    n1 <- max(per_group_min, floor(n_total / (1 + ratio)))
    n2 <- max(per_group_min, ceiling(n1 * ratio))
    while (n1 + n2 < n_total) {
      n2 <- n2 + 1
    }
    params$n1 <- n1
    params$n2 <- n2
  } else if (test == "roc_auc_one") {
    half <- max(2, floor(n_total / 2))
    params$n_pos <- half
    params$n_neg <- max(2, n_total - half)
  } else if (test == "diagnostic_acc") {
    half <- max(2, floor(n_total / 2))
    params$n_pos <- half
    params$n_neg <- max(2, n_total - half)
  } else if (test == "cluster_rct") {
    params$n_clusters <- max(2, floor(n_total / (2 * .gp_count(params, "cluster_size", 10, min = 2))))
  } else if (test == "cox_regression") {
    params$events <- ceiling(n_total)
  } else if (!test %in% c("f_anova_one_way", "f_anova_special", "f_mreg_omnibus", "f_mreg_increase", "chisq_gof", "chisq_contingency", "z_logistic", "z_poisson", "exact_mreg_random", "multi_arm_superiority", "count_endpoint_poisson", "survival_logrank", "survival_pmu", "discovery_fdr", "simon_two_stage")) {
    params$n <- ceiling(n_total)
  }

  params
}

.gp_default_effect <- function(effect_name) {
  switch(
    effect_name,
    d = 0.5,
    f = 0.25,
    f2 = 0.15,
    w = 0.3,
    rho = 0.3,
    q_effect = 0.3,
    slope_h1 = 0.1,
    delta_slope = 0.1,
    p1 = 0.65,
    auc = 0.75,
    auc1 = 0.75,
    hazard_ratio = 0.65,
    sensitivity = 0.85,
    effect_d = 0.5,
    rho2 = 0.1,
    variance_ratio = 1.5,
    odds_ratio = 1.5,
    exp_beta1 = 1.3,
    ncp = 1,
    0.5
  )
}

.gp_effect_sign <- function(params, effect_name) {
  value <- .gp_arg(params, effect_name, .gp_default_effect(effect_name))
  if (is.numeric(value) && length(value) == 1 && value < 0) -1 else 1
}

.gp_binomial_power <- function(n, p0, p1, alpha, alternative) {
  if (n > 5000) {
    zmean <- (p1 - p0) / sqrt(p0 * (1 - p0) / n)
    tails <- if (alternative == "two.sided") "two" else "one"
    return(.gp_power_z(zmean, alpha, tails)$power)
  }

  k <- 0:n
  p_values <- vapply(k, function(x) {
    stats::binom.test(x, n, p = p0, alternative = alternative)$p.value
  }, numeric(1))
  sum(stats::dbinom(k[p_values <= alpha], n, p1))
}

.gp_fisher_power <- function(n1, n2, p0, p1, alpha, alternative) {
  if (n1 * n2 > 40000) {
    h <- .gp_phi_to_h(p1, p0)
    zmean <- h / sqrt(1 / n1 + 1 / n2)
    tails <- if (alternative == "two.sided") "two" else "one"
    warning("Large Fisher exact grids use Cohen's h normal approximation for speed.", call. = FALSE)
    return(.gp_power_z(zmean, alpha, tails)$power)
  }

  x1 <- 0:n1
  x2 <- 0:n2
  p_x1 <- stats::dbinom(x1, n1, p0)
  p_x2 <- stats::dbinom(x2, n2, p1)
  power <- 0

  for (i in seq_along(x1)) {
    for (j in seq_along(x2)) {
      p_value <- .gp_fisher_p_value(x1[[i]], x2[[j]], n1, n2, alternative)
      if (is.finite(p_value) && p_value <= alpha) {
        power <- power + p_x1[[i]] * p_x2[[j]]
      }
    }
  }

  power
}

.gp_fisher_p_value <- function(x1, x2, n1, n2, alternative) {
  successes <- x1 + x2
  failures <- n1 + n2 - successes
  observed <- x1

  if (alternative == "greater") {
    return(stats::phyper(observed - 1, successes, failures, n1, lower.tail = FALSE))
  }
  if (alternative == "less") {
    return(stats::phyper(observed, successes, failures, n1))
  }

  lower <- max(0, n1 - failures)
  upper <- min(n1, successes)
  support <- lower:upper
  probabilities <- stats::dhyper(support, successes, failures, n1)
  observed_probability <- stats::dhyper(observed, successes, failures, n1)
  sum(probabilities[probabilities <= observed_probability * (1 + 1e-7)])
}

.gp_corr_cov_common <- function(r_ab, r_ac, r_bc) {
  psi <- r_bc * (1 - r_ab^2 - r_ac^2) -
    r_ab * r_ac * (1 - r_ab^2 - r_ac^2 - r_bc^2) / 2
  psi / ((1 - r_ab^2) * (1 - r_ac^2))
}

.gp_corr_cov_no_common <- function(r_ab, r_cd, r_ac, r_ad, r_bc, r_bd) {
  psi <- (
    (r_ac - r_ab * r_bc) * (r_bd - r_bc * r_cd) +
      (r_ad - r_ac * r_cd) * (r_bc - r_ab * r_ac) +
      (r_ac - r_ad * r_cd) * (r_bd - r_ab * r_ad) +
      (r_ad - r_ab * r_bd) * (r_bc - r_bd * r_cd)
  ) / 2
  psi / ((1 - r_ab^2) * (1 - r_cd^2))
}

test_that("a priori solver rounds up to actual power at or above target", {
  result <- power_compute(
    "t_two_sample",
    "a_priori",
    d = 0.5,
    alpha = 0.05,
    power = 0.8,
    tails = "two",
    allocation_ratio = 1
  )

  expect_true(result$outputs$actual_power >= 0.8)
  expect_true(result$outputs$total_sample_size >= 2)
})

test_that("a priori solver starts inside each kernel's valid sample-size range", {
  point_biserial <- power_compute("t_point_biserial", "a_priori")
  slope_groups <- power_compute("t_linear_regression_two_groups", "a_priori")

  expect_true(point_biserial$inputs$total_sample_size >= 4)
  expect_true(slope_groups$inputs$sample_size_group_1 >= 3)
  expect_true(slope_groups$inputs$sample_size_group_2 >= 3)
  expect_true(point_biserial$outputs$actual_power >= point_biserial$inputs$target_power)
  expect_true(slope_groups$outputs$actual_power >= slope_groups$inputs$target_power)
})

test_that("criterion solver computes alpha for requested power", {
  result <- power_compute(
    "t_one_sample",
    "criterion",
    d = 0.5,
    n = 40,
    power = 0.8,
    tails = "two"
  )

  expect_true(result$outputs$alpha > 0)
  expect_true(result$outputs$alpha < 1)
  expect_equal(result$outputs$power, 0.8, tolerance = 1e-5)
})

test_that("sensitivity solver computes an effect size", {
  result <- power_compute(
    "f_mreg_omnibus",
    "sensitivity",
    alpha = 0.05,
    power = 0.8,
    total_n = 100,
    predictors = 3
  )

  expect_true(result$outputs$f2 > 0)
  expect_equal(result$outputs$power, 0.8, tolerance = 1e-5)
})

test_that("sensitivity solver handles bounded and null-centered effects", {
  variance <- power_compute("f_variance_two", "sensitivity")
  common <- power_compute("z_corr_dependent_common", "sensitivity")
  no_common <- power_compute("z_corr_dependent_no_common", "sensitivity")

  expect_true(variance$outputs$variance_ratio > 1)
  expect_true(common$outputs$rho_ac >= -1 && common$outputs$rho_ac <= 1)
  expect_true(no_common$outputs$rho_cd >= -1 && no_common$outputs$rho_cd <= 1)
  expect_equal(variance$outputs$power, 0.8, tolerance = 1e-5)
  expect_equal(common$outputs$power, 0.8, tolerance = 1e-5)
  expect_equal(no_common$outputs$power, 0.8, tolerance = 1e-5)
})

test_that("compromise solver returns alpha and beta ratio", {
  result <- power_compute(
    "t_one_sample",
    "compromise",
    d = 0.5,
    n = 40,
    q = 1,
    tails = "two"
  )

  expect_true(result$outputs$alpha > 0)
  expect_true(result$outputs$beta > 0)
  expect_equal(result$outputs$beta_alpha_ratio, 1, tolerance = 1e-4)
})

test_that("biomarker kernels return valid post_hoc power", {
  r1 <- power_compute("roc_auc_one", analysis = "post_hoc", auc = 0.8, auc0 = 0.5, n_pos = 60, n_neg = 60)
  expect_s3_class(r1, "ggpower_result")
  expect_true(r1$outputs$power > 0.5)

  r2 <- power_compute("roc_auc_two", analysis = "post_hoc", auc1 = 0.78, auc2 = 0.62, n1 = 80, n2 = 80)
  expect_true(r2$outputs$power > 0.4)

  r3 <- power_compute(
    "diagnostic_acc", analysis = "post_hoc",
    sensitivity = 0.9, specificity = 0.9, n_pos = 80, n_neg = 80
  )
  expect_true(r3$outputs$power > 0.5)

  r4 <- power_compute(
    "survival_logrank", analysis = "post_hoc",
    hazard_ratio = 0.6, event_rate = 0.5, total_n = 200
  )
  expect_true(r4$outputs$power > 0.5)

  r5 <- power_compute("cox_regression", analysis = "post_hoc", hazard_ratio = 1.6, events = 120)
  expect_true(r5$outputs$power > 0.5)

  r6 <- power_compute(
    "discovery_fdr", analysis = "post_hoc",
    m_tests = 500, pi0 = 0.85, effect_d = 0.6, n = 40
  )
  expect_true(r6$outputs$power > 0)

  r7 <- power_compute("ttest_biomarker", analysis = "post_hoc", d = 0.6, n1 = 40, n2 = 40)
  expect_true(r7$outputs$power > 0.5)
})

test_that("biomarker a priori and sensitivity solvers run", {
  r1 <- power_compute("roc_auc_one", analysis = "a_priori", auc = 0.75, auc0 = 0.5, power = 0.8)
  expect_true((r1$inputs$n_positive + r1$inputs$n_negative) >= 4)

  r2 <- power_compute(
    "survival_logrank", analysis = "sensitivity",
    hazard_ratio = 0.7, event_rate = 0.5, total_n = 200, power = 0.8
  )
  expect_true(r2$outputs$hazard_ratio > 0)
})

test_that("one-sample t-test matches reference a priori example", {
  result <- power_compute(
    "t_one_sample",
    "a_priori",
    d = 0.625,
    alpha = 0.05,
    power = 0.95,
    tails = "one"
  )

  expect_equal(result$inputs$total_sample_size, 30)
  expect_equal(result$outputs$actual_power, 0.955144, tolerance = 1e-5)
  expect_equal(result$outputs$df, 29)
})

test_that("multiple regression omnibus matches reference example", {
  result <- power_compute(
    "f_mreg_omnibus",
    "post_hoc",
    f2 = 0.1111111,
    alpha = 0.05,
    total_n = 95,
    predictors = 5
  )

  expect_equal(result$outputs$noncentrality_parameter, 10.555555, tolerance = 1e-5)
  expect_equal(result$outputs$critical_f, 2.316858, tolerance = 1e-5)
  expect_equal(result$outputs$denominator_df, 89)
  expect_equal(result$outputs$power, 0.673586, tolerance = 1e-5)
})

test_that("ANOVA special example matches manual power value", {
  result <- power_compute(
    "f_anova_special",
    "post_hoc",
    f = 0.2450722,
    alpha = 0.05,
    total_n = 108,
    df1 = 4,
    groups = 36
  )

  expect_equal(result$outputs$noncentrality_parameter, 6.486521, tolerance = 1e-5)
  expect_equal(result$outputs$denominator_df, 72)
  expect_equal(result$outputs$power, 0.475635, tolerance = 1e-5)
})

test_that("independent two-sample t supports unequal sample sizes", {
  result <- power_compute(
    "t_two_sample",
    "post_hoc",
    d = 0.5,
    n1 = 4,
    n2 = 8,
    alpha = 0.05,
    tails = "one"
  )

  expect_equal(result$outputs$noncentrality_parameter, 0.816497, tolerance = 1e-5)
  expect_equal(result$outputs$df, 10)
  expect_equal(result$outputs$power, 0.188667, tolerance = 1e-5)
})

mod_power_core_ui <- function(id, config) {
  ns <- shiny::NS(id)
  registry <- ggpower_tests(module = config$module_filter)
  families <- unique(registry$family)
  first_family <- families[[1]]
  first_tests <- registry[registry$family == first_family, , drop = FALSE]
  first_spec <- ggpower_get_test(first_tests$id[[1]])
  first_analysis <- .gp_analysis_choices_for_modes(first_spec$modes)

  shiny::tagList(
    shiny::div(
      class = "gp-window gp-workspace",
      gp_page_header_ui(
        title = config$title,
        subtitle = config$subtitle,
        badges = lapply(config$badges, function(b) {
          shiny::span(class = b$class %||% "gp-badge", b$label)
        }),
        compact = TRUE
      ),
      if (!is.null(config$tip)) {
        gp_inline_tip_ui(config$tip)
      },
      shiny::div(
        class = "gp-selector-row gp-selector-grid",
        gp_select_ui(
          ns("family"),
          "Test family",
          choices = families,
          selected = first_family
        ),
        gp_select_ui(
          ns("test"),
          "Statistical test",
          choices = stats::setNames(first_tests$id, first_tests$label),
          selected = first_tests$id[[1]]
        ),
        gp_select_ui(
          ns("analysis"),
          "Type of power analysis",
          choices = first_analysis,
          selected = first_spec$modes[[1]]
        )
      ),
      shiny::div(
        class = "gp-main-grid",
        shiny::div(
          class = "gp-plot-panel",
          shiny::h4("Central and noncentral distributions"),
          shiny::div(
            class = "gp-plot-wrap",
            shiny::plotOutput(ns("distribution_plot"), height = "100%")
          )
        ),
        shiny::div(
          class = "gp-output-panel",
          shiny::h4("Output parameters"),
          shiny::uiOutput(ns("result_output"))
        ),
        shiny::div(
          class = "gp-input-panel",
          shiny::h4("Input parameters"),
          shiny::div(class = "gp-input-grid", shiny::uiOutput(ns("input_fields"))),
          shiny::div(
            class = "gp-action-row",
            shiny::actionButton(ns("determine"), "Determine"),
            shiny::actionButton(ns("calculate"), "Calculate", class = "btn-primary gp-btn-calculate"),
            shiny::actionButton(ns("reset"), "Reset")
          )
        ),
        shiny::div(
          class = "gp-xy-panel",
          shiny::h4("X-Y plot for a range of values"),
          shiny::div(
            class = "gp-plot-wrap gp-plot-wrap-sm",
            shiny::plotOutput(ns("xy_plot"), height = "100%")
          ),
          shiny::div(class = "gp-xy-table-wrap", shiny::tableOutput(ns("xy_table")))
        )
      )
    )
  )
}

mod_power_core_server <- function(id, config, protocol = NULL) {
  shiny::moduleServer(id, function(input, output, session) {
    ns <- session$ns
    registry <- ggpower_tests(module = config$module_filter)
    last_result <- shiny::reactiveVal(NULL)
    xy_data <- shiny::reactiveVal(data.frame())
    calculating <- shiny::reactiveVal(FALSE)

    shiny::observeEvent(input$family, {
      available <- registry[registry$family == input$family, , drop = FALSE]
      if (nrow(available) == 0) {
        return()
      }
      shiny::updateSelectInput(
        session,
        "test",
        choices = stats::setNames(available$id, available$label),
        selected = available$id[[1]]
      )
    }, ignoreInit = FALSE)

    shiny::observeEvent(input$test, {
      shiny::req(input$test)
      spec <- ggpower_get_test(input$test)
      choices <- .gp_analysis_choices_for_modes(spec$modes)
      selected <- if (input$analysis %in% spec$modes) input$analysis else spec$modes[[1]]
      shiny::updateSelectInput(session, "analysis", choices = choices, selected = selected)
    }, ignoreInit = TRUE)

    output$input_fields <- shiny::renderUI({
      shiny::req(input$test, input$analysis)
      spec <- ggpower_get_test(input$test)
      fields <- .gp_fields_for_mode(spec, input$analysis)
      shiny::tagList(lapply(fields, function(field) .gp_input_control(ns, field)))
    })

    shiny::observeEvent(input$determine, {
      shiny::req(input$test)
      shiny::showModal(.gp_effect_modal(input$test, ns))
    })

    shiny::observeEvent(input$reset, {
      spec <- ggpower_get_test(input$test %||% registry$id[[1]])
      for (field in spec$fields) {
        default <- .gp_field_defaults()[[field]] %||% NULL
        if (!is.null(default)) {
          shiny::updateNumericInput(session, field, value = default)
        }
      }
    })

    shiny::observeEvent(input$calculate, {
      shiny::req(input$test, input$analysis)
      calculating(TRUE)
      on.exit(calculating(FALSE))
      params <- .gp_collect_inputs(input, input$test)
      result <- tryCatch(
        do.call(power_compute, c(list(test = input$test, analysis = input$analysis), params)),
        error = function(e) e
      )

      if (inherits(result, "error")) {
        shiny::showNotification(result$message, type = "error")
        return()
      }

      last_result(result)
      if (!is.null(protocol)) {
        protocol(c(protocol(), paste(format(result), collapse = "\n")))
      }
      xy_data(.gp_make_xy_data(input$test, input$analysis, params))
    }, ignoreInit = TRUE)

    output$result_output <- shiny::renderUI({
      if (isTRUE(calculating())) {
        return(shiny::div(class = "gp-loading-state", shiny::tags$span(class = "gp-spinner"), "Recalculating..."))
      }
      result <- last_result()
      if (is.null(result)) {
        return(shiny::div(class = "gp-empty-state", "Click Calculate to run a power analysis."))
      }
      format_result_html(result)
    })

    output$distribution_plot <- shiny::renderPlot({
      result <- last_result()
      if (is.null(result)) {
        .gp_empty_plot("Run a calculation to see H0 and H1 distributions")
      } else {
        plot_distribution(result) + theme_ggpower()
      }
    })

    output$xy_plot <- shiny::renderPlot({
      dat <- xy_data()
      if (nrow(dat) == 0) {
        .gp_empty_plot("Run a calculation to generate an X-Y plot")
      } else {
        ggplot2::ggplot(dat, ggplot2::aes(x = total_n, y = power)) +
          ggplot2::geom_line(color = "#1f6f8b", linewidth = 1.1) +
          ggplot2::geom_point(color = "#c0392b", size = 2) +
          ggplot2::scale_y_continuous(limits = c(0, 1)) +
          ggplot2::labs(x = "Total sample size", y = "Power (1 - beta)") +
          theme_ggpower()
      }
    })

    output$xy_table <- shiny::renderTable({
      dat <- xy_data()
      if (nrow(dat) == 0) {
        return(NULL)
      }
      utils::head(dat, 10)
    }, digits = 4)
  })
}

testServer(
  mod_ggpower_two_sample_server,
  # Add here your module params
  args = list()
  , {
    ns <- session$ns
    expect_true(
      inherits(ns, "function")
    )
    expect_true(
      grepl(id, ns(""))
    )
    expect_true(
      grepl("test", ns("test"))
    )
    # Here are some examples of tests you can
    # run on your module
    # - Testing the setting of inputs
    # session$setInputs(x = 1)
    # expect_true(input$x == 1)
    # - If ever your input updates a reactiveValues
    # - Note that this reactiveValues must be passed
    # - to the testServer function via args = list()
    # expect_true(r$x == 1)
    # - Testing output
    # expect_true(inherits(output$tbl$html, "html"))
})
 
test_that("module ui works", {
  ui <- mod_ggpower_two_sample_ui(id = "test")
  golem::expect_shinytaglist(ui)
  # Check that formals have not been removed
  fmls <- formals(mod_ggpower_two_sample_ui)
  for (i in c("id")){
    expect_true(i %in% names(fmls))
  }
})
 

---
title: "Scenario Guide"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Scenario Guide}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(ggpower)
```

Use this guide to pick a module and test for your study design.

## Implemented scenarios (48 tests)

```{r counts, echo=FALSE}
tests <- ggpower_tests()
knitr::kable(as.data.frame(table(tests$module)), col.names = c("Module", "Tests"))
```

### Power Workspace (30 tests)

Classical families: t, F, chi-square, exact proportions, z correlations/GLM,
Wilcoxon rank tests.

### Biomarker Discovery (7 tests)

ROC/AUC, diagnostic accuracy, log-rank, Cox, FDR screening, differential expression.

### Clinical Trials (11 tests)

Superiority, NI, equivalence (TOST), Simon two-stage, cluster RCT, multi-arm ANOVA,
Poisson counts, survival.

## Decision table

| Study question | Module | Example test |
|----------------|--------|--------------|
| Two-group mean difference | Power Workspace or Clinical | `t_two_sample`, `rct_superiority_continuous` |
| Paired / pre-post | Power Workspace | `t_paired` |
| One-way ANOVA | Power Workspace or Clinical | `f_anova_one_way`, `multi_arm_superiority` |
| Multiple regression $R^2$ | Power Workspace | `f_mreg_omnibus` |
| Two proportions (Fisher) | Power Workspace or Clinical | `exact_fisher`, `rct_superiority_binary` |
| McNemar paired proportions | Power Workspace | `exact_mcnemar` |
| Correlation difference | Power Workspace | `z_corr_independent` |
| Logistic / Poisson GLM | Power Workspace or Clinical | `z_logistic`, `count_endpoint_poisson` |
| Biomarker AUC | Biomarker | `roc_auc_one` |
| Diagnostic sens/spec | Biomarker | `diagnostic_acc` |
| Survival / HR | Biomarker or Clinical | `cox_regression`, `survival_pmu` |
| Non-inferiority trial | Clinical | `rct_noninferiority_continuous` |
| Bioequivalence (TOST) | Clinical | `rct_equivalence_continuous` |
| Oncology Phase II Simon | Clinical | `simon_two_stage` |
| Cluster RCT | Clinical | `cluster_rct` |
| Omics screening + FDR | Biomarker | `discovery_fdr` |

## Not yet implemented

Document for planning only — use external tools or custom simulation:

- Group sequential / interim analyses
- Crossover or repeated-measures RCT
- Dunnett pairwise multi-arm comparisons
- Paired / correlated ROC comparison
- Partial AUC, NPV/PPV as primary endpoints
- Time-varying accrual and dropout in survival
- Bayesian assurance
- Simon optimal design search (minimax / optimal)

## Related

- [Choosing a power analysis](choosing-a-power-analysis.html)
- [Support matrix](support-matrix.html)
- [Formula reference](https://yaoxiangli.github.io/ggpower/articles/formula-reference.html) (pkgdown only)

---
title: "Reference Validation"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Reference Validation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(ggpower)
```

ggpower validates core kernels against published reference examples. Direct
noncentral t, F, normal, and chi-square procedures use tight tolerances. Exact
enumeration is used where the grid is computationally feasible. Approximation-backed
procedures report method notes in the result object.

## Example 1: One-sample t, a priori

Target: $d = 0.625$, $\alpha = 0.05$ (one-tailed), power $= 0.95$.

**Expected:** $n = 30$, actual power $\approx 0.955144$, $df = 29$.

```{r ex1}
r1 <- power_compute("t_one_sample", "a_priori", d = 0.625, alpha = 0.05,
                      power = 0.95, tails = "one")
r1$outputs[c("total_sample_size", "actual_power", "df")]
```

## Example 2: Multiple regression omnibus, post hoc

$f^2 = 0.1111111$, $\alpha = 0.05$, $N = 95$, 5 predictors.

**Expected:** $\lambda \approx 10.556$, critical $F \approx 2.317$, $df_2 = 89$, power $\approx 0.674$.

```{r ex2}
r2 <- power_compute("f_mreg_omnibus", "post_hoc", f2 = 0.1111111,
                      alpha = 0.05, total_n = 95, predictors = 5)
r2$outputs[c("noncentrality_parameter", "critical_f", "denominator_df", "power")]
```

## Example 3: ANOVA special, post hoc

$f = 0.2450722$, $N = 108$, $df_1 = 4$, 36 groups.

**Expected:** $\lambda \approx 6.487$, $df_2 = 72$, power $\approx 0.476$.

```{r ex3}
r3 <- power_compute("f_anova_special", "post_hoc", f = 0.2450722,
                      alpha = 0.05, total_n = 108, df1 = 4, groups = 36)
r3$outputs[c("noncentrality_parameter", "denominator_df", "power")]
```

## Example 4: Two-sample t, unequal n, post hoc

$d = 0.5$, $n_1 = 4$, $n_2 = 8$, one-tailed $\alpha = 0.05$.

**Expected:** $\delta \approx 0.816$, $df = 10$, power $\approx 0.189$.

```{r ex4}
r4 <- power_compute("t_two_sample", "post_hoc", d = 0.5, n1 = 4, n2 = 8,
                      alpha = 0.05, tails = "one")
r4$outputs[c("noncentrality_parameter", "df", "power")]
```

## Recommended tolerances

| Kernel type | Tolerance |
|-------------|-----------|
| Direct distribution (t, F, z, $\chi^2$) | $10^{-5}$ to $10^{-4}$ |
| Integer a priori solvers | Sample size exact; actual power $\geq$ target |
| Approximation-backed | Document method; validate with sensitivity plots |

## Related

- [Support matrix](support-matrix.html)
- [Approximation catalog](https://yaoxiangli.github.io/ggpower/articles/approximation-catalog.html) (pkgdown only)
- [Formula reference](https://yaoxiangli.github.io/ggpower/articles/formula-reference.html) (pkgdown only)

---
title: "Workspace Test Families"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Workspace Test Families}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(ggpower)
```

The **Power Workspace** module registers classical test families. This vignette
summarizes ANOVA/regression, exact proportions, correlations, GLM, and
nonparametric tests. See [t Tests](t-tests.html) for the t-test family.

## ANOVA and regression

```{r anova}
power_compute("f_anova_one_way", "a_priori", f = 0.25, alpha = 0.05,
              power = 0.8, groups = 4)
```

```{r mreg}
f2 <- effect_size_f2(0.10)
power_compute("f_mreg_omnibus", "post_hoc", f2 = f2,
              total_n = 95, predictors = 5)
```

## Exact and proportion tests

```{r binom}
power_compute("exact_binomial", "post_hoc", p0 = 0.5, p1 = 0.65,
              n = 80, alpha = 0.05, tails = "one")
```

```{r fisher}
power_compute("exact_fisher", "post_hoc", p0 = 0.4, p1 = 0.7,
              n1 = 12, n2 = 12, alpha = 0.05, tails = "greater")
```

```{r mcnemar}
power_compute("exact_mcnemar", "post_hoc", p0 = 0.5, p1 = 0.65, n = 60, alpha = 0.05)
```

## Correlation and z tests

```{r indep}
q <- effect_size_q(0.75, 0.88)
power_compute("z_corr_independent", "post_hoc", q_effect = q,
              n1 = 51, n2 = 260, alpha = 0.05)
```

```{r logist}
power_compute("z_logistic", "a_priori", odds_ratio = 1.5, p0 = 0.5,
              alpha = 0.05, power = 0.95, total_n = 300,
              r2_other = 0, x_variance = 1)
```

## Logistic and Poisson regression

```{r pois}
power_compute("z_poisson", "a_priori", exp_beta1 = 1.3,
              base_rate = 0.85, exposure = 1, alpha = 0.05,
              power = 0.95, r2_other = 0, x_variance = 0.25)
```

## Nonparametric tests

Nonparametric tests map rank-test planning to t-test noncentrality via ARE:
$d_{\text{eff}} = d \cdot \sqrt{\text{ARE}}$.

```{r wilcox-signed}
power_compute("wilcoxon_signed", "post_hoc", d = 0.5, n = 40,
              alpha = 0.05, are = 3 / pi)
```

```{r wilcox-mw}
power_compute("wilcoxon_mann_whitney", "post_hoc", d = 0.5,
              n1 = 30, n2 = 30, alpha = 0.05, are = 3 / pi)
```

## Related

- [t Tests](t-tests.html)
- [Analysis modes](analysis-modes.html)
- [Support matrix](support-matrix.html)

---
title: "Analysis Modes"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Analysis Modes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(ggpower)
```

ggpower supports five analysis modes. Each mode solves for a different unknown
given the others.

| Mode | Solves for | When to use |
|------|------------|-------------|
| `a_priori` | Sample size | Planning before data collection |
| `post_hoc` | Power | Fixed sample size, retrospective |
| `criterion` | Alpha | Choose significance level |
| `sensitivity` | Effect size | Minimum detectable effect |
| `compromise` | Alpha and beta | Balance $\alpha$ and $\beta$ via ratio $q = \beta/\alpha$ |

**Restrictions:** `t_generic` has no `a_priori`. `simon_two_stage` supports only
`post_hoc` and `sensitivity`.

## A priori — sample size

```{r a_priori}
power_compute("t_two_sample", "a_priori", d = 0.5, alpha = 0.05,
              power = 0.8, tails = "two", allocation_ratio = 1)
```

## Post hoc — achieved power

```{r post_hoc}
power_compute("t_one_sample", "post_hoc", d = 0.625, n = 30,
              alpha = 0.05, tails = "one")
```

## Criterion — alpha

```{r criterion}
power_compute("t_one_sample", "criterion", d = 0.5, n = 40,
              power = 0.8, tails = "two")
```

## Sensitivity — effect size

```{r sensitivity}
power_compute("f_mreg_omnibus", "sensitivity", alpha = 0.05, power = 0.8,
              total_n = 100, predictors = 3)
```

## Compromise — alpha and beta ratio

```{r compromise}
power_compute("t_one_sample", "compromise", d = 0.5, n = 40, q = 1, tails = "two")
```

## Effect size conversions

Helper functions convert study parameters into effect sizes used by `power_compute()`.

```{r d}
effect_size_d(mean_h1 = 15, mean_h0 = 10, sd = 8)
```

```{r f2}
effect_size_f2(r2 = 0.1)
```

```{r w}
effect_size_w(p0 = c(0.25, 0.25, 0.25, 0.25), p1 = c(0.4, 0.3, 0.2, 0.1))
```

See the pkgdown site for the full [effect size conversions](https://yaoxiangli.github.io/ggpower/articles/effect-size-conversions.html) article.

## Calculator

The **Calculator** module evaluates distribution-function scripts via `ggpower_calculator()`.

```{r calc}
ggpower_calculator("zinv(0.975)")
```

See the pkgdown site for the full [calculator](https://yaoxiangli.github.io/ggpower/articles/calculator.html) article.

## Related

- [Choosing a power analysis](choosing-a-power-analysis.html)
- [Support matrix](support-matrix.html)

---
title: "Biomarker Endpoints"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Biomarker Endpoints}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(ggpower)
```

The **Biomarker Discovery** module covers expression, classification, survival,
and screening endpoints. Detailed per-topic articles remain on the pkgdown site.

## Differential expression

Two-group mean difference for expression biomarkers (wrapper around `t_two_sample`).

```{r de}
power_compute("ttest_biomarker", "post_hoc", d = 0.6, n1 = 40, n2 = 40,
              alpha = 0.05, tails = "two")
```

```{r de-apriori}
power_compute("ttest_biomarker", "a_priori", d = 0.5, alpha = 0.05,
              power = 0.8, allocation_ratio = 1)
```

## ROC and AUC

```{r roc-one}
power_compute(
  "roc_auc_one",
  analysis = "a_priori",
  auc = 0.75,
  auc0 = 0.5,
  n_pos = 50,
  n_neg = 50,
  alpha = 0.05,
  power = 0.8,
  tails = "two"
)
```

```{r roc-two}
power_compute(
  "roc_auc_two",
  analysis = "post_hoc",
  auc1 = 0.78,
  auc2 = 0.62,
  n1 = 80,
  n2 = 80,
  alpha = 0.05,
  tails = "two"
)
```

## Diagnostic accuracy

```{r diag}
power_compute("diagnostic_acc", "post_hoc", sensitivity = 0.85, specificity = 0.85,
              n_pos = 50, n_neg = 50, alpha = 0.05)
```

```{r diag-apriori}
power_compute("diagnostic_acc", "a_priori", sensitivity = 0.9, specificity = 0.9,
              alpha = 0.05, power = 0.8, allocation_ratio = 1)
```

## Survival (log-rank)

```{r surv}
power_compute("survival_logrank", "post_hoc", hazard_ratio = 0.65,
              total_n = 200, event_rate = 0.5, alpha = 0.05)
```

## Cox prognostic models

```{r cox}
power_compute("cox_regression", "post_hoc", hazard_ratio = 0.65,
              events = 100, alpha = 0.05)
```

```{r cox-apriori}
power_compute("cox_regression", "a_priori", hazard_ratio = 0.7,
              alpha = 0.05, power = 0.8)
```

## Multiplicity and FDR

```{r fdr}
power_compute("discovery_fdr", "post_hoc", effect_d = 0.5, m_tests = 1000,
              pi0 = 0.9, fdr_level = 0.05, n = 40, alpha = 0.05)
```

## Related

- [Clinical trials](clinical-trials.html)
- [Scenario guide](scenario-guide.html)
- [Support matrix](support-matrix.html)

---
title: "t Tests"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{t Tests}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(ggpower)
```

t tests cover mean differences, matched pairs, point-biserial correlations,
regression slopes, and generic noncentrality.

## One sample

$$d = \frac{\mu_1 - \mu_0}{\sigma}, \quad \delta = d\sqrt{n}$$

```{r one}
power_compute("t_one_sample", "a_priori", d = 0.625, alpha = 0.05,
              power = 0.95, tails = "one")
```

## Two independent means

```{r two}
power_compute("t_two_sample", "a_priori", d = 0.5, alpha = 0.05,
              power = 0.8, tails = "two", allocation_ratio = 1)
```

## Matched pairs

```{r paired}
power_compute("t_paired", "post_hoc", d = 0.42, n = 50, alpha = 0.05)
```

## Point-biserial correlation

```{r pb}
power_compute("t_point_biserial", "a_priori", rho = 0.3, alpha = 0.05, power = 0.8)
```

## Linear regression slope

```{r slope}
power_compute("t_linear_regression", "post_hoc", slope_h1 = -0.0667,
              slope_h0 = 0, sd_x = 7.5, sd_y = 4, n = 100)
```

## Two-group slope difference

```{r slope2}
power_compute("t_linear_regression_two_groups", "a_priori", delta_slope = 0.1,
              sd_x1 = 1, sd_x2 = 1, residual_sd = 1, alpha = 0.05, power = 0.8)
```

## Generic t (direct NCP)

No `a_priori` mode — supply NCP and df directly.

```{r generic}
power_compute("t_generic", "post_hoc", ncp = 3, df = 29, alpha = 0.05, tails = "two")
```

## Related

- [Analysis modes](analysis-modes.html)
- [Workspace test families](workspace-test-families.html)

---
title: "Support Matrix"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Support Matrix}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(ggpower)
```

## Coverage table

Every registered test with domain, module, parity, method, and supported modes:

```{r matrix}
tests <- ggpower_tests()
knitr::kable(
  tests[, c("id", "module", "domain", "family", "parity", "method", "modes")],
  row.names = FALSE
)
```

## Counts by module

```{r counts}
as.data.frame(table(tests$module))
```

## Mode restrictions

| Test | Supported modes |
|------|-----------------|
| `t_generic` | `post_hoc`, `criterion`, `sensitivity`, `compromise` (no `a_priori`) |
| `simon_two_stage` | `post_hoc`, `sensitivity` only |

## Vignette index

CRAN ships consolidated vignettes; the pkgdown site adds per-test deep dives.

```{r links, echo=FALSE}
vignette_map <- data.frame(
  module = c(
    rep("getting started", 3), rep("workspace", 2), rep("biomarker", 1),
    rep("clinical", 1), rep("reference", 3)
  ),
  article = c(
    "choosing-a-power-analysis", "getting-started-gui", "scenario-guide",
    "t-tests", "workspace-test-families",
    "biomarker-endpoints", "clinical-trials",
    "analysis-modes", "support-matrix", "reference-validation"
  ),
  stringsAsFactors = FALSE
)
knitr::kable(vignette_map, col.names = c("Module area", "Vignette slug"))
```

Approximation limits and method notes: [approximation catalog](https://yaoxiangli.github.io/ggpower/articles/approximation-catalog.html) (pkgdown only).

## Related analyses

- [Reference validation](reference-validation.html)
- [Scenario guide](scenario-guide.html)
- [Formula reference](https://yaoxiangli.github.io/ggpower/articles/formula-reference.html) (pkgdown only)

---
title: "Getting Started with the GUI"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with the GUI}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(ggpower)
```

## In plain English

The ggpower app is a wide-screen research workspace. A **left sidebar** switches between
modules so you can run general statistical power analyses, biomarker discovery workflows, or
clinical trial designs without leaving the same session.

## Sidebar modules

| Module | Use when |
|--------|----------|
| **Power Workspace** | Classical test families (t, F, chi-square, exact, z, nonparametric) |
| **Biomarker Discovery** | ROC/AUC, diagnostic accuracy, survival, Cox, FDR screening |
| **Clinical Trials** | Superiority, NI, equivalence, Simon two-stage, cluster RCT |
| **Calculator** | Distribution-function expressions and calculator scripts |
| **Protocol** | Download a log of every analysis from the session |
| **Help** | Links to vignettes and reference articles |

## Launch the app

```{r launch, eval=FALSE}
run_app()
```

On a 1080p display you get a two-column analysis grid. At **1920px** and **2560px**
width the layout expands to a three-column grid with larger plot panels.

## Typical workflow

1. Pick a module from the sidebar.
2. Choose **test family**, **statistical test**, and **analysis mode**.
3. Enter inputs in the parameter panel.
4. Click **Calculate** — results appear as metric cards with full detail below.
5. Review the distribution plot and sample-size power curve.
6. Open **Protocol** to download the session log.

## Worked example (script)

The same calculation is available programmatically:

```{r example}
power_compute(
  "t_two_sample",
  analysis = "a_priori",
  d = 0.5,
  alpha = 0.05,
  power = 0.8,
  tails = "two"
)
```

## Related analyses

- [Choosing a power analysis](choosing-a-power-analysis.html)
- [Support matrix](support-matrix.html)

---
title: "Clinical Trials"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Clinical Trials}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(ggpower)
```

The **Clinical Trials** module covers Phase III superiority, non-inferiority,
equivalence, oncology two-stage, cluster RCT, survival, and count endpoints.

## Phase III superiority

```{r sup-cont}
power_compute(
  "rct_superiority_continuous",
  analysis = "post_hoc",
  d = 0.4,
  alpha = 0.025,
  n1 = 120,
  n2 = 120
)
```

```{r sup-binary}
power_compute(
  "rct_superiority_binary",
  analysis = "a_priori",
  p0 = 0.3,
  p1 = 0.45,
  alpha = 0.025,
  power = 0.9,
  n1 = 50,
  n2 = 50
)
```

```{r sup-multi}
power_compute("multi_arm_superiority", "a_priori", f = 0.25, groups = 3,
              alpha = 0.05, power = 0.8)
```

## Non-inferiority

```{r ni-cont}
power_compute("rct_noninferiority_continuous", "a_priori", d = 0.1,
              ni_margin = 0.2, alpha = 0.025, power = 0.8, n1 = 100, n2 = 100)
```

```{r ni-binary}
power_compute("rct_noninferiority_binary", "post_hoc", p0 = 0.5, p1 = 0.55,
              ni_margin = 0.1, alpha = 0.025, n1 = 200, n2 = 200)
```

## Equivalence (TOST)

```{r eq-cont}
power_compute("rct_equivalence_continuous", "a_priori", d = 0,
              eq_margin = 0.2, alpha = 0.05, power = 0.8, n1 = 80, n2 = 80)
```

```{r eq-prop}
power_compute("rct_equivalence_proportion", "post_hoc", p0 = 0.5, p1 = 0.52,
              eq_margin = 0.1, alpha = 0.05, n1 = 150, n2 = 150)
```

## Simon two-stage (Phase II)

`simon_two_stage` supports `post_hoc` and `sensitivity` only.

```{r simon}
power_compute("simon_two_stage", "post_hoc", p0 = 0.2, p1 = 0.4,
              r1 = 4, r = 10, n1 = 20, n2 = 20, alpha = 0.05)
```

## Cluster RCT

```{r cluster}
power_compute("cluster_rct", "a_priori", d = 0.4, icc = 0.05,
              cluster_size = 10, n_clusters = 20, alpha = 0.05, power = 0.8)
```

## Survival endpoints

```{r surv}
power_compute("survival_pmu", "a_priori", hazard_ratio = 0.65,
              event_rate = 0.5, alpha = 0.05, power = 0.8)
```

## Binary and count endpoints

```{r binary}
power_compute("rct_superiority_binary", "post_hoc", p0 = 0.3, p1 = 0.45,
              alpha = 0.025, n1 = 120, n2 = 120)
```

```{r count}
power_compute("count_endpoint_poisson", "a_priori", exp_beta1 = 1.3,
              base_rate = 0.85, exposure = 1, alpha = 0.05, power = 0.9,
              total_n = 250)
```

## Related

- [Biomarker endpoints](biomarker-endpoints.html)
- [Scenario guide](scenario-guide.html)
- [Support matrix](support-matrix.html)

---
title: "Choosing a Power Analysis"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Choosing a Power Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(ggpower)
```

ggpower organizes power analysis around the question being asked.

- Use **a priori** before data collection when sample size is unknown.
- Use **post hoc** when sample size is fixed and achieved power is needed.
- Use **criterion** when alpha should be selected to reach a target power.
- Use **sensitivity** when the smallest detectable effect size is the target.
- Use **compromise** when alpha and beta should be balanced with a chosen beta/alpha ratio.

## Example: Planning a One-Sample Mean Test

Suppose a clinical scale has a baseline mean of 10, the expected mean is 15,
and the standard deviation is 8. The effect size is:

```{r}
d <- effect_size_d(mean_h1 = 15, mean_h0 = 10, sd = 8)
d
```

For a one-tailed test with alpha = 0.05 and target power = 0.95:

```{r}
power_compute("t_one_sample", "a_priori", d = d, alpha = 0.05,
              power = 0.95, tails = "one")
```

## Choosing tests

Use `ggpower_tests()` to inspect supported families, domains, and modules.
The package registers **48 tests** across workspace, biomarker, and clinical workflows.

```{r}
ggpower_tests()[, c("id", "family", "domain", "module")]
```

## Sidebar modules

Not every research question belongs in the same module. Use this guide to pick the
sidebar entry that matches your endpoint and study design.

```{r modules, echo=FALSE}
data.frame(
  Question = c(
    "What sample size for a standard t test or ANOVA?",
    "Can my biomarker discriminate cases from controls (AUC)?",
    "Is my classifier sensitive and specific enough?",
    "Does a biomarker predict survival?",
    "How many patients for a Phase III superiority trial?",
    "Is treatment non-inferior to standard of care?",
    "Is a new formulation equivalent (bioequivalence)?",
    "Oncology single-arm Phase II with early stopping?"
  ),
  Module = c(
    "Power Workspace",
    "Biomarker Discovery",
    "Biomarker Discovery",
    "Biomarker Discovery",
    "Clinical Trials",
    "Clinical Trials",
    "Clinical Trials",
    "Clinical Trials"
  ),
  Example_test = c(
    "t_two_sample",
    "roc_auc_one",
    "diagnostic_acc",
    "cox_regression",
    "rct_superiority_continuous",
    "rct_noninferiority_binary",
    "rct_equivalence_continuous",
    "simon_two_stage"
  )
)
```

- **Power Workspace** — classical test families (t, F, chi-square, exact, z, nonparametric)
- **Biomarker Discovery** — ROC, diagnostic, survival, FDR
- **Clinical Trials** — superiority, NI, equivalence, Simon, cluster RCT

```{r filter}
ggpower_tests(module = "biomarker")[, c("id", "label")]
```

```{r filter2}
ggpower_tests(module = "clinical")[, c("id", "label")]
```

Stay in **Power Workspace** when you need specialized classical tests (McNemar,
tetrachoric correlation, Wilcoxon, etc.) even if the endpoint sounds clinical.

## Related

- [Getting started with the GUI](getting-started-gui.html)
- [Scenario guide](scenario-guide.html)
- [Support matrix](support-matrix.html)

\name{power_compute}
\alias{power_compute}
\title{Compute statistical power analyses}
\usage{
power_compute(test, analysis = "post_hoc", ...)
}
\arguments{
\item{test}{Character test id. Use \code{ggpower_tests()} to list available ids.}
\item{analysis}{One of \code{"a_priori"}, \code{"compromise"}, \code{"criterion"}, \code{"post_hoc"}, or \code{"sensitivity"}.}
\item{...}{Test-specific input parameters.}
}
\value{
A \code{ggpower_result} list with components \code{test}, \code{analysis},
\code{inputs}, \code{outputs}, and optional \code{notes} and \code{distribution}.
The \code{outputs} element contains the solved quantities (for example sample
size, power, or effect size depending on the analysis mode). See
\code{\link{ggpower_result}}.
}
\description{
Runs a power analysis using the shared ggpower compute engine. The function
supports classical test families and analysis modes.
}
\examples{
power_compute("t_one_sample", "a_priori", d = 0.625, alpha = 0.05,
  power = 0.95, tails = "one")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/power_one_sample.R
\name{power_t_one_sample}
\alias{power_t_one_sample}
\title{Compute Power for a One-Sample t-Test}
\usage{
power_t_one_sample(d, n, alpha = 0.05, tails = "two")
}
\arguments{
\item{d}{Numeric. The effect size (difference from the constant divided by sigma).}

\item{n}{Integer. Total sample size.}

\item{alpha}{Numeric. The significance level (default is 0.05).}

\item{tails}{Character. \code{"two"} for a two-tailed test or \code{"one"} for a one-tailed test.}
}
\value{
Numeric. The computed power (1 - beta).
}
\description{
Calculates the power for a one-sample t-test given the effect size (d),
total sample size (n), and significance level (alpha).
}
\examples{
# Calculate power for an effect size of 0.5 with n = 40 subjects
power_t_one_sample(d = 0.5, n = 40, alpha = 0.05)
}

\name{effect_size_helpers}
\alias{effect_size_d}
\alias{effect_size_f}
\alias{effect_size_f2}
\alias{effect_size_f2_increase}
\alias{effect_size_h}
\alias{effect_size_q}
\alias{effect_size_w}
\alias{eta2_from_f}
\alias{odds_ratio_from_probs}
\alias{r2_from_f2}
\title{Effect-size helper functions}
\usage{
effect_size_d(mean_h1, mean_h0 = 0, sd)
effect_size_f(eta2)
effect_size_f2(r2)
effect_size_f2_increase(r2_full, r2_reduced)
effect_size_h(p1, p2)
effect_size_q(r1, r2)
effect_size_w(p0, p1)
eta2_from_f(f)
odds_ratio_from_probs(p0, p1)
r2_from_f2(f2)
}
\arguments{
\item{mean_h1, mean_h0}{Means used to compute Cohen's d.}
\item{sd}{Common standard deviation.}
\item{eta2}{Eta-squared value.}
\item{r2, r2_full, r2_reduced}{R-squared values; \code{r2} is also the second correlation in \code{effect_size_q()}.}
\item{p0, p1, p2}{Probabilities or probability vectors.}
\item{r1}{First correlation in \code{effect_size_q()}.}
\item{f, f2}{Cohen effect-size values.}
}
\value{
A numeric effect-size or converted variance-explained value.
}
\description{
Helpers used by the GUI effect-size drawer and by scripting workflows.
}

\name{ggpower_calculator}
\alias{ggpower_calculator}
\title{Evaluate a ggpower calculator script}
\usage{
ggpower_calculator(script)
}
\arguments{
\item{script}{Character calculator script with arithmetic, assignments, comments,
and supported distribution helper functions.}
}
\value{
The value of the final expression.
}
\description{
Evaluates distribution-function calculator expressions, including helpers such as
\code{zcdf()}, \code{tinv()}, \code{ncfcdf()}, and \code{binocdf()}.
}
\examples{
ggpower_calculator("x <- 2^3\nx + zinv(.975)")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/power_ttest.R
\name{power_t_two_sample}
\alias{power_t_two_sample}
\title{Compute Power for a Two-Sample t-Test (Equal Sample Sizes)}
\usage{
power_t_two_sample(d, n_per_group, alpha = 0.05, tails = "two", n2 = NULL)
}
\arguments{
\item{d}{Numeric. The effect size (Cohen's d).}

\item{n_per_group}{Integer. The sample size per group.}

\item{alpha}{Numeric. The significance level (default is 0.05).}

\item{tails}{Character. \code{"two"} for a two-tailed test or \code{"one"} for a one-tailed test.}

\item{n2}{Optional second-group sample size. If omitted, equal group sizes are used.}
}
\value{
Numeric. The computed power (1 - beta).
}
\description{
This function calculates the power for a two-sample t-test when the two groups have equal sample sizes.
}
\examples{
# Compute power for an effect size d = 0.5 with 30 subjects per group
power_t_two_sample(d = 0.5, n_per_group = 30)
}

\name{power_t_paired}
\alias{power_t_paired}
\title{Compute power for a paired-samples t-test}
\usage{
power_t_paired(d, n, alpha = 0.05, tails = "two")
}
\arguments{
\item{d}{Numeric paired-samples effect size dz.}
\item{n}{Integer number of pairs.}
\item{alpha}{Numeric significance level.}
\item{tails}{Character, \code{"two"} or \code{"one"}.}
}
\value{
Numeric power.
}
\description{
Computes achieved power for a paired-samples t-test using the noncentral t kernel.
}
\examples{
power_t_paired(d = 0.5, n = 40)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_app.R
\name{run_app}
\alias{run_app}
\title{Run the Shiny Application}
\usage{
run_app(
  onStart = NULL,
  options = list(),
  enableBookmarking = NULL,
  uiPattern = "/",
  ...
)
}
\arguments{
\item{onStart}{A function that will be called before the app is actually run.
This is only needed for \code{shinyAppObj}, since in the \code{shinyAppDir}
case, a \code{global.R} file can be used for this purpose.}

\item{options}{Named options that should be passed to the \code{runApp} call
(these can be any of the following: "port", "launch.browser", "host", "quiet",
"display.mode" and "test.mode"). You can also specify \code{width} and
\code{height} parameters which provide a hint to the embedding environment
about the ideal height/width for the app.}

\item{enableBookmarking}{Can be one of \code{"url"}, \code{"server"}, or
\code{"disable"}. The default value, \code{NULL}, will respect the setting from
any previous calls to  \code{\link[shiny:enableBookmarking]{enableBookmarking()}}. See \code{\link[shiny:enableBookmarking]{enableBookmarking()}}
for more information on bookmarking your app.}

\item{uiPattern}{A regular expression that will be applied to each \code{GET}
request to determine whether the \code{ui} should be used to handle the
request. Note that the entire request path must match the regular
expression in order for the match to be considered successful.}

\item{...}{arguments to pass to golem_opts.
See \code{?golem::get_golem_options} for more details.}
}
\value{
A Shiny application object (class \code{"shiny.appobj"}), returned invisibly.
Launch the GUI with \code{shiny::runApp(run_app())} or from the development
helper described in the package overview.
}
\description{
Run the Shiny Application
}

\name{theme_ggpower}
\alias{theme_ggpower}
\title{Publication-ready ggpower theme}
\usage{
theme_ggpower(base_size = 12, base_family = "")
}
\arguments{
\item{base_size}{Base font size.}
\item{base_family}{Base font family.}
}
\value{
A ggplot2 theme.
}
\description{
Provides consistent typography, spacing, and grid styling for ggpower figures.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/power_result.R
\name{format_result_html}
\alias{format_result_html}
\title{Format a ggpower result as structured HTML for Shiny UI}
\usage{
format_result_html(x)
}
\arguments{
\item{x}{A \code{ggpower_result} object.}
}
\value{
A \code{shiny.tag} list suitable for \code{renderUI}.
}
\description{
Renders metric cards, input/output blocks, and notes for the Shiny app.
}

\name{plot_power_curve}
\alias{plot_power_curve}
\title{Plot a power curve}
\usage{
plot_power_curve(test, n_values, analysis = "post_hoc", ...)
}
\arguments{
\item{test}{Character test id.}
\item{n_values}{Numeric vector of total sample sizes.}
\item{analysis}{Power analysis mode used for fixed parameters.}
\item{...}{Test-specific fixed parameters.}
}
\value{
A ggplot object.
}
\description{
Builds a publication-ready power curve for a selected ggpower test.
}
\examples{
plot_power_curve("t_one_sample", n_values = c(20, 30, 40), d = 0.5)
}

\name{plot_distribution}
\alias{plot_distribution}
\title{Plot H0 and H1 distributions}
\usage{
plot_distribution(result)
}
\arguments{
\item{result}{A \code{ggpower_result} object.}
}
\value{
A ggplot object.
}
\description{
Builds a publication-ready distribution overlay for a computed power-analysis result.
}
\examples{
result <- power_compute("t_one_sample", "post_hoc", d = 0.5, n = 40)
plot_distribution(result)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_ttest.R
\name{ggpower_ttest}
\alias{ggpower_ttest}
\title{Plot Power Curve for a Two-Sample t-Test}
\usage{
ggpower_ttest(d, alpha = 0.05, n_range = seq(10, 100, by = 5),
  tails = "two")
}
\arguments{
\item{d}{Numeric. The effect size (Cohen's d).}

\item{alpha}{Numeric. The significance level (default 0.05).}

\item{n_range}{Numeric vector. A vector of sample sizes per group (default is seq(10, 100, by = 5)).}

\item{tails}{Character. \code{"two"} or \code{"one"}.}
}
\value{
A ggplot object showing the power curve.
}
\description{
This function creates a ggplot2 power curve for a two-sample t test.
}
\examples{
# Create a power curve for d = 0.5 over a range of sample sizes per group
ggpower_ttest(d = 0.5, alpha = 0.05, n_range = seq(10, 100, by = 5))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/power_registry.R
\name{ggpower_tests}
\alias{ggpower_tests}
\title{List supported statistical power tests}
\usage{
ggpower_tests(domain = NULL, module = NULL)
}
\arguments{
\item{domain}{Optional character vector to filter by domain (\code{general},
\code{biomarker}, \code{pharma}).}

\item{module}{Optional character vector to filter by app module (\code{workspace},
\code{biomarker}, \code{clinical}).}
}
\value{
A data frame describing tests available to \code{power_compute()}.
}
\description{
Lists the tests available to \code{power_compute()}.
}
\examples{
ggpower_tests()
ggpower_tests(module = "biomarker")
}

\name{save_power_plot}
\alias{save_power_plot}
\alias{save_distribution_plot}
\title{Save a ggpower plot}
\usage{
save_power_plot(plot, filename, width = 7, height = 5, dpi = 320)

save_distribution_plot(plot, filename, width = 7, height = 5, dpi = 320)
}
\arguments{
\item{plot}{A ggplot object.}
\item{filename}{Output filename.}
\item{width,height}{Plot dimensions.}
\item{dpi}{Resolution for raster outputs.}
}
\value{
The filename invisibly.
}
\description{
Exports publication-ready ggpower plots through \code{ggplot2::ggsave()}.
}

\name{ggpower_result}
\alias{ggpower_result}
\alias{print.ggpower_result}
\title{Create a ggpower result object}
\usage{
ggpower_result(test, analysis, inputs, outputs, notes = character(),
  distribution = list())
}
\arguments{
\item{test}{Character label for the selected test.}
\item{analysis}{Character label for the selected analysis mode.}
\item{inputs}{Named list of input parameters.}
\item{outputs}{Named list of computed output parameters.}
\item{notes}{Character vector with method notes or assumptions.}
\item{distribution}{Named list describing the H0/H1 distributions.}
}
\value{
An object of class \code{ggpower_result}.
}
\description{
Creates the common result object used by the scriptable API and Shiny GUI.
}
